/*
 * Copyright (c) 2017-2019 Cavium, Inc.
 * All rights reserved.                
 *                                     
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:                                                          
 *                                                                    
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *                                                                         
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE  
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE   
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR        
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF       
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS   
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN    
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)    
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE.                                                
 */    
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <stdio.h>

#include "bcm_osal.h"
#include "ecore.h"
#include "spad_layout.h"
#include "nvm_map.h"
#include "ecore_hsi_debug_tools.h"
#ifndef USE_DBG_BIN_FILE
#include "ecore_dbg_values_user.h"
#endif
#include "ecore_user_dbg_fw_funcs.h"

void *
zalloc(uint32_t size)
{
        caddr_t va;

        va = malloc((unsigned long)size);
        bzero(va, size);
        return ((void *)va);
}

/******************************* Data Types **********************************/

/* REG fifo element */
struct reg_fifo_element {
	u64 data;
#define REG_FIFO_ELEMENT_ADDRESS_SHIFT		0
#define REG_FIFO_ELEMENT_ADDRESS_MASK		0x7fffff
#define REG_FIFO_ELEMENT_ACCESS_SHIFT		23
#define REG_FIFO_ELEMENT_ACCESS_MASK		0x1
#define REG_FIFO_ELEMENT_PF_SHIFT		24
#define REG_FIFO_ELEMENT_PF_MASK		0xf
#define REG_FIFO_ELEMENT_VF_SHIFT		28
#define REG_FIFO_ELEMENT_VF_MASK		0xff
#define REG_FIFO_ELEMENT_PORT_SHIFT		36
#define REG_FIFO_ELEMENT_PORT_MASK		0x3
#define REG_FIFO_ELEMENT_PRIVILEGE_SHIFT	38
#define REG_FIFO_ELEMENT_PRIVILEGE_MASK		0x3
#define REG_FIFO_ELEMENT_PROTECTION_SHIFT	40
#define REG_FIFO_ELEMENT_PROTECTION_MASK	0x7
#define REG_FIFO_ELEMENT_MASTER_SHIFT		43
#define REG_FIFO_ELEMENT_MASTER_MASK		0xf
#define REG_FIFO_ELEMENT_ERROR_SHIFT		47
#define REG_FIFO_ELEMENT_ERROR_MASK		0x1f
};

/* REG fifo error element */
struct reg_fifo_err {
	u32 err_code;
	const char *err_msg;
};

/* IGU fifo element */
struct igu_fifo_element {
	u32 dword0;
#define IGU_FIFO_ELEMENT_DWORD0_FID_SHIFT	0
#define IGU_FIFO_ELEMENT_DWORD0_FID_MASK	0xff
#define IGU_FIFO_ELEMENT_DWORD0_IS_PF_SHIFT	8
#define IGU_FIFO_ELEMENT_DWORD0_IS_PF_MASK	0x1
#define IGU_FIFO_ELEMENT_DWORD0_SOURCE_SHIFT	9
#define IGU_FIFO_ELEMENT_DWORD0_SOURCE_MASK	0xf
#define IGU_FIFO_ELEMENT_DWORD0_ERR_TYPE_SHIFT	13
#define IGU_FIFO_ELEMENT_DWORD0_ERR_TYPE_MASK	0xf
#define IGU_FIFO_ELEMENT_DWORD0_CMD_ADDR_SHIFT	17
#define IGU_FIFO_ELEMENT_DWORD0_CMD_ADDR_MASK	0x7fff
	u32 dword1;
	u32 dword2;
#define IGU_FIFO_ELEMENT_DWORD12_IS_WR_CMD_SHIFT 0
#define IGU_FIFO_ELEMENT_DWORD12_IS_WR_CMD_MASK	 0x1
#define IGU_FIFO_ELEMENT_DWORD12_WR_DATA_SHIFT	 1
#define IGU_FIFO_ELEMENT_DWORD12_WR_DATA_MASK	 0xffffffff
	u32 reserved;
};

struct igu_fifo_wr_data {
	u32 data;
#define IGU_FIFO_WR_DATA_PROD_CONS_SHIFT		0
#define IGU_FIFO_WR_DATA_PROD_CONS_MASK			0xffffff
#define IGU_FIFO_WR_DATA_UPDATE_FLAG_SHIFT		24
#define IGU_FIFO_WR_DATA_UPDATE_FLAG_MASK		0x1
#define IGU_FIFO_WR_DATA_EN_DIS_INT_FOR_SB_SHIFT	25
#define IGU_FIFO_WR_DATA_EN_DIS_INT_FOR_SB_MASK		0x3
#define IGU_FIFO_WR_DATA_SEGMENT_SHIFT			27
#define IGU_FIFO_WR_DATA_SEGMENT_MASK			0x1
#define IGU_FIFO_WR_DATA_TIMER_MASK_SHIFT		28
#define IGU_FIFO_WR_DATA_TIMER_MASK_MASK		0x1
#define IGU_FIFO_WR_DATA_CMD_TYPE_SHIFT			31
#define IGU_FIFO_WR_DATA_CMD_TYPE_MASK			0x1
};

struct igu_fifo_cleanup_wr_data {
	u32 data;
#define IGU_FIFO_CLEANUP_WR_DATA_RESERVED_SHIFT		0
#define IGU_FIFO_CLEANUP_WR_DATA_RESERVED_MASK		0x7ffffff
#define IGU_FIFO_CLEANUP_WR_DATA_CLEANUP_VAL_SHIFT	27
#define IGU_FIFO_CLEANUP_WR_DATA_CLEANUP_VAL_MASK	0x1
#define IGU_FIFO_CLEANUP_WR_DATA_CLEANUP_TYPE_SHIFT	28
#define IGU_FIFO_CLEANUP_WR_DATA_CLEANUP_TYPE_MASK	0x7
#define IGU_FIFO_CLEANUP_WR_DATA_CMD_TYPE_SHIFT		31
#define IGU_FIFO_CLEANUP_WR_DATA_CMD_TYPE_MASK		0x1
};

/* Protection override element */
struct protection_override_element {
	u64 data;
#define PROTECTION_OVERRIDE_ELEMENT_ADDRESS_SHIFT		0
#define PROTECTION_OVERRIDE_ELEMENT_ADDRESS_MASK		0x7fffff
#define PROTECTION_OVERRIDE_ELEMENT_WINDOW_SIZE_SHIFT		23
#define PROTECTION_OVERRIDE_ELEMENT_WINDOW_SIZE_MASK		0xffffff
#define PROTECTION_OVERRIDE_ELEMENT_READ_SHIFT			47
#define PROTECTION_OVERRIDE_ELEMENT_READ_MASK			0x1
#define PROTECTION_OVERRIDE_ELEMENT_WRITE_SHIFT			48
#define PROTECTION_OVERRIDE_ELEMENT_WRITE_MASK			0x1
#define PROTECTION_OVERRIDE_ELEMENT_READ_PROTECTION_SHIFT	49
#define PROTECTION_OVERRIDE_ELEMENT_READ_PROTECTION_MASK	0x7
#define PROTECTION_OVERRIDE_ELEMENT_WRITE_PROTECTION_SHIFT	52
#define PROTECTION_OVERRIDE_ELEMENT_WRITE_PROTECTION_MASK	0x7
};

enum igu_fifo_sources {
	IGU_SRC_PXP0,
	IGU_SRC_PXP1,
	IGU_SRC_PXP2,
	IGU_SRC_PXP3,
	IGU_SRC_PXP4,
	IGU_SRC_PXP5,
	IGU_SRC_PXP6,
	IGU_SRC_PXP7,
	IGU_SRC_CAU,
	IGU_SRC_ATTN,
	IGU_SRC_GRC
};

enum igu_fifo_addr_types {
	IGU_ADDR_TYPE_MSIX_MEM,
	IGU_ADDR_TYPE_WRITE_PBA,
	IGU_ADDR_TYPE_WRITE_INT_ACK,
	IGU_ADDR_TYPE_WRITE_ATTN_BITS,
	IGU_ADDR_TYPE_READ_INT,
	IGU_ADDR_TYPE_WRITE_PROD_UPDATE,
	IGU_ADDR_TYPE_RESERVED
};

struct igu_fifo_addr_data {
	u16 start_addr;
	u16 end_addr;
	char *desc;
	char *vf_desc;
	enum igu_fifo_addr_types type;
};

/******************************** Constants **********************************/

#define MAX_MSG_LEN				1024

#define MCP_TRACE_MAX_MODULE_LEN		8
#define MCP_TRACE_FORMAT_MAX_PARAMS		3
#define MCP_TRACE_FORMAT_PARAM_WIDTH			(MCP_TRACE_FORMAT_P2_SIZE_OFFSET - MCP_TRACE_FORMAT_P1_SIZE_OFFSET)

#ifndef REG_FIFO_ELEMENT_DWORDS
#define REG_FIFO_ELEMENT_DWORDS			2
#endif
#define REG_FIFO_ELEMENT_ADDR_FACTOR		4
#define REG_FIFO_ELEMENT_IS_PF_VF_VAL		127

#ifndef IGU_FIFO_ELEMENT_DWORDS
#define IGU_FIFO_ELEMENT_DWORDS			4
#endif

#ifndef PROTECTION_OVERRIDE_ELEMENT_DWORDS
#define PROTECTION_OVERRIDE_ELEMENT_DWORDS	2
#endif
#define PROTECTION_OVERRIDE_ELEMENT_ADDR_FACTOR	4

#define DBG_BUS_SIGNATURE_LINE_NAME		"signature"
#define DBG_BUS_SIGNATURE_LINE_NUM		0

#define DBG_BUS_LATENCY_LINE_NAME		"latency"
#define DBG_BUS_LATENCY_LINE_NUM		1

/* Extra lines include a signature line + optional latency events line */
#define NUM_EXTRA_DBG_LINES_USER(block)			(1 + (block->has_latency_events ? 1 : 0))
#define NUM_DBG_LINES_USER(block)			(block->num_of_dbg_bus_lines + NUM_EXTRA_DBG_LINES_USER(block))

#ifndef LINUX_REMOVE
#define BYTES_IN_DWORD				4
#endif

/********************************* Macros ************************************/

#ifndef BYTES_TO_DWORDS
#define BYTES_TO_DWORDS(bytes)			((bytes) / sizeof(u32))
#endif

#define CHECK_ARR_SIZE(arr, size)			OSAL_BUILD_BUG_ON(!(OSAL_ARRAY_SIZE(arr) == size))

/***************************** Constant Arrays *******************************/

/* Storm names array */
static const char* s_storm_str[] = {"t", "m", "u", "x", "y", "p"};

/* Storm debug mode names array */
static const char* s_storm_mode_str[] = {
	/* DBG_BUS_STORM_MODE_PRINTF */
	"printf",

	/* DBG_BUS_STORM_MODE_PRAM_ADDR */
	"pram_addr",

	/* DBG_BUS_STORM_MODE_DRA_RW */
	"dra_rw",

	/* DBG_BUS_STORM_MODE_DRA_W */
	"dra_w",

	/* DBG_BUS_STORM_MODE_LD_ST_ADDR */
	"ld_st_addr",

	/* DBG_BUS_STORM_MODE_DRA_FSM */
	"dra_fsm",

	/* DBG_BUS_STORM_MODE_FAST_DBGMUX */
	"fast_dbgmux",

	/* DBG_BUS_STORM_MODE_RH */
	"rh",

	/* DBG_BUS_STORM_MODE_RH_WITH_STORE */
	"rh_with_store",

	/* DBG_BUS_STORM_MODE_FOC */
	"foc",

	/* DBG_BUS_STORM_MODE_EXT_STORE*/
	"ext_store"
};

/* Constraint operation names array */
static const char* s_constraint_op_str[MAX_DBG_BUS_CONSTRAINT_OPS] = {
	/* DBG_BUS_CONSTRAINT_OP_EQ */
	"eq",

	/* DBG_BUS_CONSTRAINT_OP_NE */
	"ne",

	/* DBG_BUS_CONSTRAINT_OP_LT */
	"lt",

	/* DBG_BUS_CONSTRAINT_OP_LTC */
	"ltc",

	/* DBG_BUS_CONSTRAINT_OP_LE */
	"le",

	/* DBG_BUS_CONSTRAINT_OP_LEC */
	"lec",

	/* DBG_BUS_CONSTRAINT_OP_GT */
	"gt",

	/* DBG_BUS_CONSTRAINT_OP_GTC */
	"gtc",

	/* DBG_BUS_CONSTRAINT_OP_GE */
	"ge",

	/* DBG_BUS_CONSTRAINT_OP_GEC */
	"gec"
};

/* Status string array */
static const char* s_status_str[] = {
	/* DBG_STATUS_OK */
	"Operation completed successfully",

	/* DBG_STATUS_APP_VERSION_NOT_SET */
	"Debug application version wasn't set",

	/* DBG_STATUS_UNSUPPORTED_APP_VERSION */
	"Unsupported debug application version",

	/* DBG_STATUS_DBG_BLOCK_NOT_RESET */
	"The debug block wasn't reset since the last recording",

	/* DBG_STATUS_INVALID_ARGS */
	"Invalid arguments",

	/* DBG_STATUS_OUTPUT_ALREADY_SET */
	"The debug output was already set",

	/* DBG_STATUS_INVALID_PCI_BUF_SIZE */
	"Invalid PCI buffer size",

	/* DBG_STATUS_PCI_BUF_ALLOC_FAILED */
	"PCI buffer allocation failed",

	/* DBG_STATUS_PCI_BUF_NOT_ALLOCATED */
	"A PCI buffer wasn't allocated",

	/* DBG_STATUS_INVALID_FILTER_TRIGGER_DWORDS */
	"The filter/trigger constraint dword offsets are not enabled for recording",

	/* DBG_STATUS_NO_MATCHING_FRAMING_MODE */
	"No matching framing mode found for the enabled blocks/Storms - use less dwords for blocks data",

	/* DBG_STATUS_VFC_READ_ERROR */
	"Error reading from VFC",

	/* DBG_STATUS_STORM_ALREADY_ENABLED */
	"The Storm was already enabled",

	/* DBG_STATUS_STORM_NOT_ENABLED */
	"The specified Storm wasn't enabled",

	/* DBG_STATUS_BLOCK_ALREADY_ENABLED */
	"The block was already enabled",

	/* DBG_STATUS_BLOCK_NOT_ENABLED */
	"The specified block wasn't enabled",

	/* DBG_STATUS_NO_INPUT_ENABLED */
	"No input was enabled for recording",

	/* DBG_STATUS_NO_FILTER_TRIGGER_256B */
	"Filters and triggers are not allowed in E4 256-bit mode",

	/* DBG_STATUS_FILTER_ALREADY_ENABLED */
	"The filter was already enabled",

	/* DBG_STATUS_TRIGGER_ALREADY_ENABLED */
	"The trigger was already enabled",

	/* DBG_STATUS_TRIGGER_NOT_ENABLED */
	"The trigger wasn't enabled",

	/* DBG_STATUS_CANT_ADD_CONSTRAINT */
	"A constraint can be added only after a filter was enabled or a trigger state was added",

	/* DBG_STATUS_TOO_MANY_TRIGGER_STATES */
	"Cannot add more than 3 trigger states",

	/* DBG_STATUS_TOO_MANY_CONSTRAINTS */
	"Cannot add more than 4 constraints per filter or trigger state",

	/* DBG_STATUS_RECORDING_NOT_STARTED */
	"The recording wasn't started",

	/* DBG_STATUS_DATA_DIDNT_TRIGGER */
	"A trigger was configured, but it didn't trigger",

	/* DBG_STATUS_NO_DATA_RECORDED */
	"No data was recorded",

	/* DBG_STATUS_DUMP_BUF_TOO_SMALL */
	"Dump buffer is too small",

	/* DBG_STATUS_DUMP_NOT_CHUNK_ALIGNED */
	"Dumped data is not aligned to chunks",

	/* DBG_STATUS_UNKNOWN_CHIP */
	"Unknown chip",

	/* DBG_STATUS_VIRT_MEM_ALLOC_FAILED */
	"Failed allocating virtual memory",

	/* DBG_STATUS_BLOCK_IN_RESET */
	"The input block is in reset",

	/* DBG_STATUS_INVALID_TRACE_SIGNATURE */
	"Invalid MCP trace signature found in NVRAM",

	/* DBG_STATUS_INVALID_NVRAM_BUNDLE */
	"Invalid bundle ID found in NVRAM",

	/* DBG_STATUS_NVRAM_GET_IMAGE_FAILED */
	"Failed getting NVRAM image",

	/* DBG_STATUS_NON_ALIGNED_NVRAM_IMAGE */
	"NVRAM image is not dword-aligned",

	/* DBG_STATUS_NVRAM_READ_FAILED */
	"Failed reading from NVRAM",

	/* DBG_STATUS_IDLE_CHK_PARSE_FAILED */
	"Idle check parsing failed",

	/* DBG_STATUS_MCP_TRACE_BAD_DATA */
	"MCP Trace data is corrupt",

	/* DBG_STATUS_MCP_TRACE_NO_META */
	"Dump doesn't contain meta data - it must be provided in image file",

	/* DBG_STATUS_MCP_COULD_NOT_HALT */
	"Failed to halt MCP",

	/* DBG_STATUS_MCP_COULD_NOT_RESUME */
	"Failed to resume MCP after halt",

	/* DBG_STATUS_RESERVED0 */
	"",

	/* DBG_STATUS_SEMI_FIFO_NOT_EMPTY */
	"Failed to empty SEMI sync FIFO",

	/* DBG_STATUS_IGU_FIFO_BAD_DATA */
	"IGU FIFO data is corrupt",

	/* DBG_STATUS_MCP_COULD_NOT_MASK_PRTY */
	"MCP failed to mask parities",

	/* DBG_STATUS_FW_ASSERTS_PARSE_FAILED */
	"FW Asserts parsing failed",

	/* DBG_STATUS_REG_FIFO_BAD_DATA */
	"GRC FIFO data is corrupt",

	/* DBG_STATUS_PROTECTION_OVERRIDE_BAD_DATA */
	"Protection Override data is corrupt",

	/* DBG_STATUS_DBG_ARRAY_NOT_SET */
	"Debug arrays were not set (when using binary files, dbg_set_bin_ptr must be called)",

	/* DBG_STATUS_RESERVED1 */
	"",

	/* DBG_STATUS_NON_MATCHING_LINES */
	"Non-matching debug lines - in E4, all lines must be of the same type (either 128b or 256b)",

	/* DBG_STATUS_INSUFFICIENT_HW_IDS */
	"Insufficient HW IDs. Try to record less Storms/blocks",

	/* DBG_STATUS_DBG_BUS_IN_USE */
	"The debug bus is in use",

	/* DBG_STATUS_INVALID_STORM_DBG_MODE */
	"The storm debug mode is not supported in the current chip",

	/* DBG_STATUS_OTHER_ENGINE_BB_ONLY */
	"Other engine is supported only in BB",

	/* DBG_STATUS_FILTER_SINGLE_HW_ID */
	"The configured filter mode requires a single Storm/block input",

	/* DBG_STATUS_TRIGGER_SINGLE_HW_ID */
	"The configured filter mode requires that all the constraints of a single trigger state will be defined on a single Storm/block input",
		
	/* DBG_STATUS_MISSING_TRIGGER_STATE_STORM */
	"When triggering on Storm data, the Storm to trigger on must be specified"
};

/* GRC Dump flag names array */
static const char* s_grc_param_str[] = {
	/* DBG_GRC_PARAM_DUMP_TSTORM */
	"tstorm",

	/* DBG_GRC_PARAM_DUMP_MSTORM */
	"mstorm",

	/* DBG_GRC_PARAM_DUMP_USTORM */
	"ustorm",

	/* DBG_GRC_PARAM_DUMP_XSTORM */
	"xstorm",

	/* DBG_GRC_PARAM_DUMP_YSTORM */
	"ystorm",

	/* DBG_GRC_PARAM_DUMP_PSTORM */
	"pstorm",

	/* DBG_GRC_PARAM_DUMP_REGS */
	"regs",

	/* DBG_GRC_PARAM_DUMP_RAM */
	"ram",

	/* DBG_GRC_PARAM_DUMP_PBUF */
	"pbuf",

	/* DBG_GRC_PARAM_DUMP_IOR */
	"ior",

	/* DBG_GRC_PARAM_DUMP_VFC */
	"vfc",

	/* DBG_GRC_PARAM_DUMP_CM_CTX */
	"ctx",

	/* DBG_GRC_PARAM_DUMP_PXP */
	"pxp",

	/* DBG_GRC_PARAM_DUMP_RSS */
	"rss",

	/* DBG_GRC_PARAM_DUMP_CAU */
	"cau",

	/* DBG_GRC_PARAM_DUMP_QM */
	"qm",

	/* DBG_GRC_PARAM_DUMP_MCP */
	"mcp",

	/* DBG_GRC_PARAM_DORQ */
	"dorq",

	/* DBG_GRC_PARAM_DUMP_CFC */
	"cfc",

	/* DBG_GRC_PARAM_DUMP_IGU */
	"igu",

	/* DBG_GRC_PARAM_DUMP_BRB */
	"brb",

	/* DBG_GRC_PARAM_DUMP_BTB */
	"btb",

	/* DBG_GRC_PARAM_DUMP_BMB */
	"bmb",

	/* DBG_GRC_PARAM_RESERVED1 */
	"reserved1",

	/* DBG_GRC_PARAM_DUMP_MULD */
	"muld",

	/* DBG_GRC_PARAM_DUMP_PRS */
	"prs",

	/* DBG_GRC_PARAM_DUMP_DMAE */
	"dmae",

	/* DBG_GRC_PARAM_DUMP_TM */
	"tm",

	/* DBG_GRC_PARAM_DUMP_SDM */
	"sdm",

	/* DBG_GRC_PARAM_DUMP_DIF */
	"dif",

	/* DBG_GRC_PARAM_DUMP_STATIC */
	"static",

	/* DBG_GRC_PARAM_UNSTALL */
	"unstall",

	/* DBG_GRC_PARAM_RESERVED2 */
	"reserved2",

	/* DBG_GRC_PARAM_MCP_TRACE_META_SIZE */
	"trace_meta_size",

	/* DBG_GRC_PARAM_EXCLUDE_ALL */
	"exclude_all",

	/* DBG_GRC_PARAM_CRASH */
	"crash",

	/* DBG_GRC_PARAM_PARITY_SAFE */
	"parity_safe",

	/* DBG_GRC_PARAM_DUMP_CM */
	"cm",

	/* DBG_GRC_PARAM_DUMP_PHY */
	"phy",

	/* DBG_GRC_PARAM_NO_MCP */
	"nomcp",

	/* DBG_GRC_PARAM_NO_FW_VER */
	"nofwver",

	/* DBG_GRC_PARAM_RESERVED3 */
	"reserved3",

	/* DBG_GRC_PARAM_DUMP_MCP_HW_DUMP */
	"mcp_hw_dump",

	/* DBG_GRC_PARAM_DUMP_ILT_CDUC */
	"cduc",

	/* DBG_GRC_PARAM_DUMP_ILT_CDUT */
	"cdut",

	/* DBG_GRC_PARAM_DUMP_CAU_EXT */
	"cau_ext"
};

/* Idle check severity names array */
static const char* s_idle_chk_severity_str[] = {
	"Error",
	"Error if no traffic",
	"Warning"};

/* MCP Trace level names array */
static const char* s_mcp_trace_level_str[] = {
	"ERROR",
	"TRACE",
	"DEBUG"};

/* Access type names array */
static const char *s_access_strs[] = {
	"read",
	"write"};

/* Privilege type names array */
static const char *s_privilege_strs[] = {
	"VF",
	"PDA",
	"HV",
	"UA"};

/* Protection type names array */
static const char *s_protection_strs[] = {
	"(default)",
	"(default)",
	"(default)",
	"(default)",
	"override VF",
	"override PDA",
	"override HV",
	"override UA"};

/* Master type names array */
static const char *s_master_strs[] = {
	"???",
	"pxp",
	"mcp",
	"msdm",
	"psdm",
	"ysdm",
	"usdm",
	"tsdm",
	"xsdm",
	"dbu",
	"dmae",
	"jdap",
	"???",
	"???",
	"???",
	"???"
};

/* REG FIFO error messages array */
static struct reg_fifo_err s_reg_fifo_errors[] = {
	{ 1,  "grc timeout" },
	{ 2,  "address doesn't belong to any block" },
	{ 4,  "reserved address in block or write to read-only address" },
	{ 8,  "privilege/protection mismatch" },
	{ 16, "path isolation error" },
	{ 17, "RSL error" }
};

/* IGU FIFO sources array */
static const char *s_igu_fifo_source_strs[] = {
	"TSTORM",
	"MSTORM",
	"USTORM",
	"XSTORM",
	"YSTORM",
	"PSTORM",
	"PCIE",
	"NIG_QM_PBF",
	"CAU",
	"ATTN",
	"GRC",
};

/* IGU FIFO error messages */
static const char *s_igu_fifo_error_strs[] = {
	"no error",
	"length error",
	"function disabled",
	"VF sent command to attention address",
	"host sent prod update command",
	"read of during interrupt register while in MIMD mode",
	"access to PXP BAR reserved address",
	"producer update command to attention index",
	"unknown error",
	"SB index not valid",
	"SB relative index and FID not found",
	"FID not match",
	"command with error flag asserted (PCI error or CAU discard)",
	"VF sent cleanup and RF cleanup is disabled",
	"cleanup command on type bigger than 4"
};

/* IGU FIFO address data */
static const struct igu_fifo_addr_data s_igu_fifo_addr_data[] = {
	{ 0x0,	 0x101, "MSI-X Memory", OSAL_NULL, IGU_ADDR_TYPE_MSIX_MEM },
	{ 0x102, 0x1ff, "reserved", OSAL_NULL , IGU_ADDR_TYPE_RESERVED },
	{ 0x200, 0x200, "Write PBA[0:63]", OSAL_NULL , IGU_ADDR_TYPE_WRITE_PBA },
	{ 0x201, 0x201, "Write PBA[64:127]", "reserved" , IGU_ADDR_TYPE_WRITE_PBA },
	{ 0x202, 0x202, "Write PBA[128]", "reserved" , IGU_ADDR_TYPE_WRITE_PBA },
	{ 0x203, 0x3ff, "reserved", OSAL_NULL , IGU_ADDR_TYPE_RESERVED },
	{ 0x400, 0x5ef, "Write interrupt acknowledgment", OSAL_NULL , IGU_ADDR_TYPE_WRITE_INT_ACK },
	{ 0x5f0, 0x5f0, "Attention bits update", OSAL_NULL , IGU_ADDR_TYPE_WRITE_ATTN_BITS },
	{ 0x5f1, 0x5f1, "Attention bits set", OSAL_NULL , IGU_ADDR_TYPE_WRITE_ATTN_BITS },
	{ 0x5f2, 0x5f2, "Attention bits clear", OSAL_NULL , IGU_ADDR_TYPE_WRITE_ATTN_BITS },
	{ 0x5f3, 0x5f3, "Read interrupt 0:63 with mask", OSAL_NULL , IGU_ADDR_TYPE_READ_INT },
	{ 0x5f4, 0x5f4, "Read interrupt 0:31 with mask", OSAL_NULL , IGU_ADDR_TYPE_READ_INT },
	{ 0x5f5, 0x5f5, "Read interrupt 32:63 with mask", OSAL_NULL , IGU_ADDR_TYPE_READ_INT },
	{ 0x5f6, 0x5f6, "Read interrupt 0:63 without mask", OSAL_NULL , IGU_ADDR_TYPE_READ_INT },
	{ 0x5f7, 0x5ff, "reserved", OSAL_NULL , IGU_ADDR_TYPE_RESERVED },
	{ 0x600, 0x7ff, "Producer update", OSAL_NULL , IGU_ADDR_TYPE_WRITE_PROD_UPDATE }
};

/******************************** Variables **********************************/

/* Temporary buffer, used for print size calculations */
static char s_temp_buf[MAX_MSG_LEN];

/**************************** Private Functions ******************************/

static void ecore_user_static_asserts(void)
{
	CHECK_ARR_SIZE(s_storm_str, MAX_DBG_STORMS);
	CHECK_ARR_SIZE(s_storm_mode_str, MAX_DBG_BUS_STORM_MODES);
	CHECK_ARR_SIZE(s_constraint_op_str, MAX_DBG_BUS_CONSTRAINT_OPS);
	CHECK_ARR_SIZE(s_status_str, MAX_DBG_STATUS);
	CHECK_ARR_SIZE(s_grc_param_str, MAX_DBG_GRC_PARAMS);
	CHECK_ARR_SIZE(s_idle_chk_severity_str, MAX_DBG_IDLE_CHK_SEVERITY_TYPES);
}

/* Returns true if the specified string is a number, false otherwise. */
static bool ecore_is_number(const char *str)
{
	osal_size_t i = 0, len = OSAL_STRLEN(str);

	for (i = 0; i < len; i++)
		if (str[i] < '0' || str[i] > '9')
			return false;

	return true;
}

static u32 ecore_cyclic_add(u32 a,
			    u32 b,
			    u32 size)
{
	return (a + b) % size;
}

static u32 ecore_cyclic_sub(u32 a,
			    u32 b,
			    u32 size)
{
	return (size + a - b) % size;
}

/* Reads the specified number of bytes from the specified cyclic buffer (up to
 * 4 bytes) and returns them as a dword value. the specified buffer offset is
 * updated.
 */
static u32 ecore_read_from_cyclic_buf(void *buf,
				      u32 *offset,
				      u32 buf_size,
				      u8 num_bytes_to_read)
{
	u8 i, *val_ptr, *bytes_buf = (u8 *)buf;
	u32 val = 0;

	val_ptr = (u8 *)&val;

	/* Assume running on a LITTLE ENDIAN and the buffer is network order
	 * (BIG ENDIAN), as high order bytes are placed in lower memory address.
	 */
	for (i = 0; i < num_bytes_to_read; i++) {
		val_ptr[i] = bytes_buf[*offset];
		*offset = ecore_cyclic_add(*offset, 1, buf_size);
	}

	return val;
}

/* Reads and returns the next byte from the specified buffer. the specified
 * buffer offset is updated.
 */
static u8 ecore_read_byte_from_buf(void *buf,
				   u32 *offset)
{
	return ((u8 *)buf)[(*offset)++];
}

/* Reads and returns the next dword from the specified buffer. the specified
 * buffer offset is updated.
 */
static u32 ecore_read_dword_from_buf(void *buf,
				     u32 *offset)
{
	u32 dword_val = *(u32 *)&((u8 *)buf)[*offset];

	*offset += 4;

	return dword_val;
}

/* Reads the next string from the specified buffer, and copies it to the
 * specified pointer the specified buffer offset is updated.
 */
static void ecore_read_str_from_buf(void *buf,
				    u32 *offset,
				    u32 size,
				    char *dest)
{
	const char *source_str = &((const char *)buf)[*offset];

	OSAL_STRNCPY(dest, source_str, size);
	dest[size - 1] = '\0';
	*offset += size;
}

/* Returns a pointer to the specified offset (in bytes) of the specified
 * buffer. if the specified buffer in OSAL_NULL, a temporary buffer pointer is
 * returned.
 */
static char *ecore_get_buf_ptr(void *buf,
			       u32 offset)
{
	return buf ? (char *)buf + offset : s_temp_buf;
}

/* Reads a param from the specified buffer. returns the number of dwords read.
 * if the returned str_param is OSAL_NULL, the param is numeric and its value is
 * returned in num_param. otherwise, the param is a string and its pointer is
 * returned in str_param.
 */
static u32 ecore_read_param(u32 *dump_buf,
			    const char **param_name,
			    const char **param_str_val,
			    u32 *param_num_val)
{
	char *char_buf = (char *)dump_buf;
	osal_size_t offset = 0;

	/* Extract param name */
	*param_name = char_buf;
	offset += OSAL_STRLEN(*param_name) + 1;

	/* Check param type */
	if (*(char_buf + offset++)) {
		/* String param */
		*param_str_val = char_buf + offset;
		*param_num_val = 0;
		offset += OSAL_STRLEN(*param_str_val) + 1;
		if (offset & 0x3)
			offset += (4 - (offset & 0x3));
	} else {
		/* Numeric param */
		*param_str_val = OSAL_NULL;
		if (offset & 0x3)
			offset += (4 - (offset & 0x3));
		*param_num_val = *(u32 *)(char_buf + offset);
		offset += 4;
	}

	return (u32)offset / 4;
}

/* Reads a section header from the specified buffer. Returns the number of
 * dwords read.
 */
static u32 ecore_read_section_hdr(u32 *dump_buf,
				  const char **section_name,
				  u32 *num_section_params)
{
	const char *param_str_val;

	return ecore_read_param(dump_buf, section_name, &param_str_val, num_section_params);
}

/* Reads section params from the specified buffer and prints them to the
 * results buffer. Returns the number of dwords read.
 */
static u32 ecore_print_section_params(u32 *dump_buf,
				      u32 num_section_params,
				      char *results_buf,
				      u32 *num_chars_printed)
{
	u32 i, dump_offset = 0, results_offset = 0;

	for (i = 0; i < num_section_params; i++) {
		const char *param_name, *param_str_val;
		u32 param_num_val = 0;

		dump_offset += ecore_read_param(dump_buf + dump_offset, &param_name, &param_str_val, &param_num_val);
		
		if (param_str_val)
			results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "%s: %s\n", param_name, param_str_val);
		else if (OSAL_STRCMP(param_name, "fw-timestamp"))
			results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "%s: %d\n", param_name, param_num_val);
	}

	results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "\n");

	*num_chars_printed = results_offset;

	return dump_offset;
}

#ifndef LINUX_REMOVE /* set_dbg_bin_buf() is also present in dbg_tools.c */
/* Sets pointer and size for the specified binary buffer type */
static void ecore_set_dbg_bin_buf(struct ecore_hwfn *p_hwfn,
				  enum bin_dbg_buffer_type bufType,
				  const u32 *ptr,
				  u32 size)
{
	struct virt_mem_desc *buf = &p_hwfn->dbg_arrays[bufType];
	buf->ptr = (void*)ptr;
	buf->size = size;
}
#endif

/* Returns the block name that matches the specified block ID,
 * or OSAL_NULL if not found.
 */
static const char *ecore_dbg_get_block_name(struct ecore_hwfn *p_hwfn,
					    enum block_id block_id)
{
	const struct dbg_block_user *block = (const struct dbg_block_user *)p_hwfn->dbg_arrays[BIN_BUF_DBG_BLOCKS_USER_DATA].ptr + block_id;

	return (const char *)block->name;
}

static struct dbg_tools_user_data *ecore_dbg_get_user_data(struct ecore_hwfn *p_hwfn)
{
	return (struct dbg_tools_user_data *)p_hwfn->dbg_user_info;
}

/* Parses the idle check rules and returns the number of characters printed.
 * in case of parsing error, returns 0.
 */
static u32 ecore_parse_idle_chk_dump_rules(struct ecore_hwfn *p_hwfn,
					   u32 *dump_buf,
					   u32 *dump_buf_end,
					   u32 num_rules,
					   bool print_fw_idle_chk,
					   char *results_buf,
					   u32 *num_errors,
					   u32 *num_warnings)
{
	/* Offset in results_buf in bytes */
	u32 results_offset = 0;
	
	u32 rule_idx; 
	u16 i, j;

	*num_errors = 0;
	*num_warnings = 0;

	/* Go over dumped results */
	for (rule_idx = 0; rule_idx < num_rules && dump_buf < dump_buf_end; rule_idx++) {
		const struct dbg_idle_chk_rule_parsing_data *rule_parsing_data;
		struct dbg_idle_chk_result_hdr *hdr;
		const char *parsing_str, *lsi_msg;
		u32 parsing_str_offset;
		bool has_fw_msg;
		u8 curr_reg_id;

		hdr = (struct dbg_idle_chk_result_hdr *)dump_buf;
		rule_parsing_data = (const struct dbg_idle_chk_rule_parsing_data *)p_hwfn->dbg_arrays[BIN_BUF_DBG_IDLE_CHK_PARSING_DATA].ptr + hdr->rule_id;
		parsing_str_offset = GET_FIELD(rule_parsing_data->data, DBG_IDLE_CHK_RULE_PARSING_DATA_STR_OFFSET);
		has_fw_msg = GET_FIELD(rule_parsing_data->data, DBG_IDLE_CHK_RULE_PARSING_DATA_HAS_FW_MSG) > 0;
		parsing_str = (const char *)p_hwfn->dbg_arrays[BIN_BUF_DBG_PARSING_STRINGS].ptr + parsing_str_offset;
		lsi_msg = parsing_str;
		curr_reg_id = 0;

		if (hdr->severity >= MAX_DBG_IDLE_CHK_SEVERITY_TYPES)
			return 0;

		/* Skip rule header */
		dump_buf += BYTES_TO_DWORDS(sizeof(*hdr));

		/* Update errors/warnings count */
		if (hdr->severity == IDLE_CHK_SEVERITY_ERROR || hdr->severity == IDLE_CHK_SEVERITY_ERROR_NO_TRAFFIC)
			(*num_errors)++;
		else
			(*num_warnings)++;

		/* Print rule severity: */
		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "%s: ", s_idle_chk_severity_str[hdr->severity]);

		/* Print rule message */
		if (has_fw_msg)
			parsing_str += OSAL_STRLEN(parsing_str) + 1;
		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "%s.", has_fw_msg && print_fw_idle_chk ? parsing_str : lsi_msg);
		parsing_str += OSAL_STRLEN(parsing_str) + 1;

		/* Print register values */
		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), " Registers:");
		for (i = 0; i < hdr->num_dumped_cond_regs + hdr->num_dumped_info_regs; i++) {
			struct dbg_idle_chk_result_reg_hdr *reg_hdr;
			bool is_mem;
			u8 reg_id;

			reg_hdr = (struct dbg_idle_chk_result_reg_hdr *)dump_buf;
			is_mem = GET_FIELD(reg_hdr->data, DBG_IDLE_CHK_RESULT_REG_HDR_IS_MEM);
			reg_id = GET_FIELD(reg_hdr->data, DBG_IDLE_CHK_RESULT_REG_HDR_REG_ID);

			/* Skip reg header */
			dump_buf += BYTES_TO_DWORDS(sizeof(*reg_hdr));

			/* Skip register names until the required reg_id is
			 * reached.
			 */
			for (; reg_id > curr_reg_id; curr_reg_id++, parsing_str += OSAL_STRLEN(parsing_str) + 1);

			results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), " %s", parsing_str);
			if (i < hdr->num_dumped_cond_regs && is_mem)
				results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "[%d]", hdr->mem_entry_id + reg_hdr->start_entry);
			results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "=");
			for (j = 0; j < reg_hdr->size; j++, dump_buf++) {
				results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "0x%x", *dump_buf);
				if (j < reg_hdr->size - 1)
					results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), ",");
			}
		}

		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "\n");
	}

	/* Check if end of dump buffer was exceeded */
	if (dump_buf > dump_buf_end)
		return 0;

	return results_offset;
}

/* Parses an idle check dump buffer. If result_buf is not OSAL_NULL, the idle check
 * results are printed to it. In any case, the required results buffer size is
 * assigned to parsed_results_bytes. The parsing status is returned.
 */
static enum dbg_status ecore_parse_idle_chk_dump(struct ecore_hwfn *p_hwfn,
						 u32 *dump_buf,
						 u32 num_dumped_dwords,
						 char *results_buf,
						 u32 *parsed_results_bytes,
						 u32 *num_errors,
						 u32 *num_warnings)
{
	const char *section_name, *param_name, *param_str_val;
	u32 *dump_buf_end = dump_buf + num_dumped_dwords;
	u32 num_section_params = 0, num_rules;
	
	/* Offset in results_buf in bytes */
	u32 results_offset = 0;

	*parsed_results_bytes = 0;
	*num_errors = 0;
	*num_warnings = 0;

	if (!p_hwfn->dbg_arrays[BIN_BUF_DBG_PARSING_STRINGS].ptr || !p_hwfn->dbg_arrays[BIN_BUF_DBG_IDLE_CHK_PARSING_DATA].ptr)
		return DBG_STATUS_DBG_ARRAY_NOT_SET;

	/* Read global_params section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "global_params"))
		return DBG_STATUS_IDLE_CHK_PARSE_FAILED;

	/* Print global params */
	dump_buf += ecore_print_section_params(dump_buf, num_section_params, results_buf, &results_offset);

	/* Read idle_chk section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "idle_chk") || num_section_params != 1)
		return DBG_STATUS_IDLE_CHK_PARSE_FAILED;
	dump_buf += ecore_read_param(dump_buf, &param_name, &param_str_val, &num_rules);
	if (OSAL_STRCMP(param_name, "num_rules"))
		return DBG_STATUS_IDLE_CHK_PARSE_FAILED;

	if (num_rules) {
		u32 rules_print_size;

		/* Print FW output: */
		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "FW_IDLE_CHECK:\n");
		rules_print_size = ecore_parse_idle_chk_dump_rules(p_hwfn, dump_buf, dump_buf_end, num_rules, true, results_buf ? results_buf + results_offset : OSAL_NULL, num_errors, num_warnings);
		results_offset += rules_print_size;
		if (!rules_print_size)
			return DBG_STATUS_IDLE_CHK_PARSE_FAILED;

		/* Print LSI output: */
		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "\nLSI_IDLE_CHECK:\n");
		rules_print_size = ecore_parse_idle_chk_dump_rules(p_hwfn, dump_buf, dump_buf_end, num_rules, false, results_buf ? results_buf + results_offset : OSAL_NULL, num_errors, num_warnings);
		results_offset += rules_print_size;
		if (!rules_print_size)
			return DBG_STATUS_IDLE_CHK_PARSE_FAILED;
	}

	/* Print errors/warnings count */
	if (*num_errors)
		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "\nIdle Check failed!!! (with %d errors and %d warnings)\n", *num_errors, *num_warnings);
	else if (*num_warnings)
		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "\nIdle Check completed successfully (with %d warnings)\n", *num_warnings);
	else
		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "\nIdle Check completed successfully\n");

	/* Add 1 for string OSAL_NULL termination */
	*parsed_results_bytes = results_offset + 1;

	return DBG_STATUS_OK;
}

/* Allocates and fills MCP Trace meta data based on the specified meta data
 * dump buffer. returns debug status code.
 */
static enum dbg_status ecore_mcp_trace_alloc_meta_data(struct ecore_hwfn *p_hwfn,
						       const u32 *meta_buf)
{
	struct dbg_tools_user_data *dev_user_data = ecore_dbg_get_user_data(p_hwfn);
	struct mcp_trace_meta *meta = &dev_user_data->mcp_trace_meta;
	u8 *meta_buf_bytes = (u8 *)meta_buf;
	u32 offset = 0, signature, i;

	/* Free the previous meta before loading a new one. */
	if (meta->is_allocated)
		ecore_mcp_trace_free_meta_data(p_hwfn);

	OSAL_MEMSET(meta, 0, sizeof(*meta));

	/* Read first signature */
	signature = ecore_read_dword_from_buf(meta_buf_bytes, &offset);
	if (signature != NVM_MAGIC_VALUE)
		return DBG_STATUS_INVALID_TRACE_SIGNATURE;

	/* Read no. of modules and allocate memory for their pointers */
	meta->modules_num = ecore_read_byte_from_buf(meta_buf_bytes, &offset);
	meta->modules = (char**)OSAL_ZALLOC(p_hwfn->p_dev, GFP_KERNEL, meta->modules_num * sizeof(char *));
	if (!meta->modules)
		return DBG_STATUS_VIRT_MEM_ALLOC_FAILED;

	/* Allocate and read all module strings */
	for (i = 0; i < meta->modules_num; i++) {
		u8 module_len = ecore_read_byte_from_buf(meta_buf_bytes, &offset);

		*(meta->modules + i) = (char *)OSAL_ZALLOC(p_hwfn->p_dev, GFP_KERNEL, module_len);
		if (!(*(meta->modules + i))) {
			/* Update number of modules to be released */
			meta->modules_num = i ? i - 1 : 0;
			return DBG_STATUS_VIRT_MEM_ALLOC_FAILED;
		}

		ecore_read_str_from_buf(meta_buf_bytes, &offset, module_len, *(meta->modules + i));
		if (module_len > MCP_TRACE_MAX_MODULE_LEN)
			(*(meta->modules + i))[MCP_TRACE_MAX_MODULE_LEN] = '\0';
	}

	/* Read second signature */
	signature = ecore_read_dword_from_buf(meta_buf_bytes, &offset);
	if (signature != NVM_MAGIC_VALUE)
		return DBG_STATUS_INVALID_TRACE_SIGNATURE;

	/* Read number of formats and allocate memory for all formats */
	meta->formats_num = ecore_read_dword_from_buf(meta_buf_bytes, &offset);
	meta->formats = (struct mcp_trace_format *)OSAL_ZALLOC(p_hwfn->p_dev, GFP_KERNEL, meta->formats_num * sizeof(struct mcp_trace_format));
	if (!meta->formats)
		return DBG_STATUS_VIRT_MEM_ALLOC_FAILED;

	/* Allocate and read all  strings */
	for (i = 0; i < meta->formats_num; i++) {
		struct mcp_trace_format *format_ptr = &meta->formats[i];
		u8 format_len;

		format_ptr->data = ecore_read_dword_from_buf(meta_buf_bytes, &offset);
		format_len = GET_MFW_FIELD(format_ptr->data, MCP_TRACE_FORMAT_LEN);
		format_ptr->format_str = (char *)OSAL_ZALLOC(p_hwfn->p_dev, GFP_KERNEL, format_len);
		if (!format_ptr->format_str) {
			/* Update number of modules to be released */
			meta->formats_num = i ? i - 1 : 0;
			return DBG_STATUS_VIRT_MEM_ALLOC_FAILED;
		}

		ecore_read_str_from_buf(meta_buf_bytes, &offset, format_len, format_ptr->format_str);
	}

	meta->is_allocated = true;

	return DBG_STATUS_OK;
}

/* Parses an MCP trace buffer. If result_buf is not OSAL_NULL, the MCP Trace results
 * are printed to it. The parsing status is returned.
 * Arguments:
 * trace_buf -	          MCP trace cyclic buffer
 * trace_buf_size -       MCP trace cyclic buffer size in bytes
 * data_offset -          offset in bytes of the data to parse in the MCP trace cyclic
 *		          buffer.
 * data_size -	          size in bytes of data to parse.
 * parsed_buf -	          destination buffer for parsed data.
 * parsed_results_bytes - size of parsed data in bytes.
 */
static enum dbg_status ecore_parse_mcp_trace_buf(struct ecore_hwfn *p_hwfn,
						 u8 *trace_buf,
						 u32 trace_buf_size,
						 u32 data_offset,
						 u32 data_size,
						 char *parsed_buf,
						 u32 *parsed_results_bytes)
{
	struct dbg_tools_user_data *dev_user_data = ecore_dbg_get_user_data(p_hwfn);
	struct mcp_trace_meta *meta = &dev_user_data->mcp_trace_meta;
	u32 param_mask, param_shift;
	enum dbg_status status;

	*parsed_results_bytes = 0;

	if (!meta->is_allocated)
		return DBG_STATUS_MCP_TRACE_BAD_DATA;

	status = DBG_STATUS_OK;

	while (data_size) {
		u32 params[3] = { 0, 0, 0 };
		struct mcp_trace_format *format_ptr;
		u32 header, format_idx, i;
		u8 format_level, format_module;

		if (data_size < MFW_TRACE_ENTRY_SIZE)
			return DBG_STATUS_MCP_TRACE_BAD_DATA;

		header = ecore_read_from_cyclic_buf(trace_buf, &data_offset, trace_buf_size, MFW_TRACE_ENTRY_SIZE);
		data_size -= MFW_TRACE_ENTRY_SIZE;
		format_idx = header & MFW_TRACE_EVENTID_MASK;

		/* Skip message if its  index doesn't exist in the meta
		 * data.
		 */
		if (format_idx >= meta->formats_num) {
			u8 format_size = (u8)GET_MFW_FIELD(header, MFW_TRACE_PRM_SIZE);

			if (data_size < format_size)
				return DBG_STATUS_MCP_TRACE_BAD_DATA;

			data_offset = ecore_cyclic_add(data_offset, format_size, trace_buf_size);
			data_size -= format_size;
			continue;
		}

		format_ptr = &meta->formats[format_idx];

		for (i = 0, param_mask = MCP_TRACE_FORMAT_P1_SIZE_MASK, param_shift = MCP_TRACE_FORMAT_P1_SIZE_OFFSET;
		     i < MCP_TRACE_FORMAT_MAX_PARAMS;
		     i++, param_mask <<= MCP_TRACE_FORMAT_PARAM_WIDTH, param_shift += MCP_TRACE_FORMAT_PARAM_WIDTH)
		{
			/* Extract param size (0..3) */
			u8 param_size = (u8)((format_ptr->data & param_mask) >> param_shift);

			/* If the param size is zero, there are no other parameters. */
			if (!param_size)
				break;

			/* Size is encoded using 2 bits, where 3 is used to encode 4. */
			if (param_size == 3)
				param_size = 4;

			if (data_size < param_size)
				return DBG_STATUS_MCP_TRACE_BAD_DATA;

			params[i] = ecore_read_from_cyclic_buf(trace_buf, &data_offset, trace_buf_size, param_size);
			data_size -= param_size;
		}

		format_level = (u8)GET_MFW_FIELD(format_ptr->data, MCP_TRACE_FORMAT_LEVEL);
		format_module = (u8)GET_MFW_FIELD(format_ptr->data, MCP_TRACE_FORMAT_MODULE);
		if (format_level >= OSAL_ARRAY_SIZE(s_mcp_trace_level_str))
			return DBG_STATUS_MCP_TRACE_BAD_DATA;

		/* Print current message to results buffer */
		*parsed_results_bytes += OSAL_SPRINTF(ecore_get_buf_ptr(parsed_buf, *parsed_results_bytes), "%s %-8s: ", s_mcp_trace_level_str[format_level], meta->modules[format_module]);
		*parsed_results_bytes += OSAL_SPRINTF(ecore_get_buf_ptr(parsed_buf, *parsed_results_bytes), format_ptr->format_str, params[0], params[1], params[2]);
	}

	/* Add string OSAL_NULL terminator */
	(*parsed_results_bytes)++;

	return status;
}

/* Parses an MCP Trace dump buffer. If result_buf is not OSAL_NULL, the MCP Trace
 * results are printed to it. In any case, the required results buffer size is
 * assigned to parsed_results_bytes. The parsing status is returned.
 */
static enum dbg_status ecore_parse_mcp_trace_dump(struct ecore_hwfn *p_hwfn,
						  u32 *dump_buf,
						  char *results_buf,
						  u32 *parsed_results_bytes,
						  bool free_meta_data)
{
	const char *section_name, *param_name, *param_str_val;
	u32 data_size, trace_data_dwords, trace_meta_dwords;
	u32 offset, results_offset, results_buf_bytes;
	u32 param_num_val, num_section_params;
	struct mcp_trace *trace;
	enum dbg_status status;
	const u32 *meta_buf;
	u8 *trace_buf;

	*parsed_results_bytes = 0;

	/* Read global_params section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "global_params"))
		return DBG_STATUS_MCP_TRACE_BAD_DATA;

	/* Print global params */
	dump_buf += ecore_print_section_params(dump_buf, num_section_params, results_buf, &results_offset);

	/* Read trace_data section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "mcp_trace_data") || num_section_params != 1)
		return DBG_STATUS_MCP_TRACE_BAD_DATA;
	dump_buf += ecore_read_param(dump_buf, &param_name, &param_str_val, &param_num_val);
	if (OSAL_STRCMP(param_name, "size"))
		return DBG_STATUS_MCP_TRACE_BAD_DATA;
	trace_data_dwords = param_num_val;

	/* Prepare trace info */
	trace = (struct mcp_trace *)dump_buf;
	if (trace->signature != MFW_TRACE_SIGNATURE || !trace->size)
		return DBG_STATUS_MCP_TRACE_BAD_DATA;

	trace_buf = (u8 *)dump_buf + sizeof(*trace);
	offset = trace->trace_oldest;
	data_size = ecore_cyclic_sub(trace->trace_prod, offset, trace->size);
	dump_buf += trace_data_dwords;

	/* Read meta_data section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "mcp_trace_meta"))
		return DBG_STATUS_MCP_TRACE_BAD_DATA;
	dump_buf += ecore_read_param(dump_buf, &param_name, &param_str_val, &param_num_val);
	if (OSAL_STRCMP(param_name, "size"))
		return DBG_STATUS_MCP_TRACE_BAD_DATA;
	trace_meta_dwords = param_num_val;

	/* Choose meta data buffer */
	if (!trace_meta_dwords) {
		/* Dump doesn't include meta data */
		struct dbg_tools_user_data *dev_user_data = ecore_dbg_get_user_data(p_hwfn);

		if (!dev_user_data->mcp_trace_user_meta_buf)
			return DBG_STATUS_MCP_TRACE_NO_META;

		meta_buf = dev_user_data->mcp_trace_user_meta_buf;
	} else {
		/* Dump includes meta data */
		meta_buf = dump_buf;
	}

	/* Allocate meta data memory */
	status = ecore_mcp_trace_alloc_meta_data(p_hwfn, meta_buf);
		if (status != DBG_STATUS_OK)
			return status;

	status = ecore_parse_mcp_trace_buf(p_hwfn, trace_buf, trace->size, offset, data_size, results_buf ? results_buf + results_offset : OSAL_NULL, &results_buf_bytes);
	if (status != DBG_STATUS_OK)
		return status;

	if (free_meta_data)
		ecore_mcp_trace_free_meta_data(p_hwfn);

	*parsed_results_bytes = results_offset + results_buf_bytes;

	return DBG_STATUS_OK;
}

/* Parses a Reg FIFO dump buffer. If result_buf is not OSAL_NULL, the Reg FIFO
 * results are printed to it. In any case, the required results buffer size is
 * assigned to parsed_results_bytes. The parsing status is returned.
 */
static enum dbg_status ecore_parse_reg_fifo_dump(u32 *dump_buf,
						 char *results_buf,
						 u32 *parsed_results_bytes)
{
	const char *section_name, *param_name, *param_str_val;
	u32 param_num_val, num_section_params, num_elements;
	struct reg_fifo_element *elements;
	u8 i, j, err_code, vf_val;
	u32 results_offset = 0;
	char vf_str[4];

	/* Read global_params section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "global_params"))
		return DBG_STATUS_REG_FIFO_BAD_DATA;

	/* Print global params */
	dump_buf += ecore_print_section_params(dump_buf, num_section_params, results_buf, &results_offset);

	/* Read reg_fifo_data section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "reg_fifo_data"))
		return DBG_STATUS_REG_FIFO_BAD_DATA;
	dump_buf += ecore_read_param(dump_buf, &param_name, &param_str_val, &param_num_val);
	if (OSAL_STRCMP(param_name, "size"))
		return DBG_STATUS_REG_FIFO_BAD_DATA;
	if (param_num_val % REG_FIFO_ELEMENT_DWORDS)
		return DBG_STATUS_REG_FIFO_BAD_DATA;
	num_elements = param_num_val / REG_FIFO_ELEMENT_DWORDS;
	elements = (struct reg_fifo_element *)dump_buf;

	/* Decode elements */
	for (i = 0; i < num_elements; i++) {
		const char *err_msg = OSAL_NULL;

		/* Discover if element belongs to a VF or a PF */
		vf_val = GET_FIELD(elements[i].data, REG_FIFO_ELEMENT_VF);
		if (vf_val == REG_FIFO_ELEMENT_IS_PF_VF_VAL)
			OSAL_SPRINTF(vf_str, "%s", "N/A");
		else
			OSAL_SPRINTF(vf_str, "%d", vf_val);

		/* Find error message */
		err_code = GET_FIELD(elements[i].data, REG_FIFO_ELEMENT_ERROR);
		for (j = 0; j < OSAL_ARRAY_SIZE(s_reg_fifo_errors) && !err_msg; j++)
			if (err_code == s_reg_fifo_errors[j].err_code)
				err_msg = s_reg_fifo_errors[j].err_msg;

		/* Add parsed element to parsed buffer */
		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "raw: 0x%016llx, address: 0x%07x, access: %-5s, pf: %2d, vf: %s, port: %d, privilege: %-3s, protection: %-12s, master: %-4s, error: %s\n", elements[i].data,
			(u32)GET_FIELD(elements[i].data, REG_FIFO_ELEMENT_ADDRESS) * REG_FIFO_ELEMENT_ADDR_FACTOR,
			s_access_strs[GET_FIELD(elements[i].data, REG_FIFO_ELEMENT_ACCESS)],
			(u32)GET_FIELD(elements[i].data, REG_FIFO_ELEMENT_PF),
			vf_str,
			(u32)GET_FIELD(elements[i].data, REG_FIFO_ELEMENT_PORT),
			s_privilege_strs[GET_FIELD(elements[i].data, REG_FIFO_ELEMENT_PRIVILEGE)],
			s_protection_strs[GET_FIELD(elements[i].data, REG_FIFO_ELEMENT_PROTECTION)],
			s_master_strs[GET_FIELD(elements[i].data, REG_FIFO_ELEMENT_MASTER)],
			err_msg ? err_msg : "unknown error code");
	}

	results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "fifo contained %d elements", num_elements);

	/* Add 1 for string OSAL_NULL termination */
	*parsed_results_bytes = results_offset + 1;

	return DBG_STATUS_OK;
}

static enum dbg_status ecore_parse_igu_fifo_element(struct igu_fifo_element *element,
						    char *results_buf,
						    u32 *results_offset)
{
	const struct igu_fifo_addr_data *found_addr = OSAL_NULL;
	char parsed_addr_data[32];
	char parsed_wr_data[256];
	u8 source, err_type, i;
	bool is_wr_cmd, is_pf;
	u16 cmd_addr;
	u64 dword12;

	/* Dword12 (dword index 1 and 2) contains bits 32..95 of the
	 * FIFO element.
	 */
	dword12 = ((u64)element->dword2 << 32) | element->dword1;
	is_wr_cmd = GET_FIELD(dword12, IGU_FIFO_ELEMENT_DWORD12_IS_WR_CMD);
	is_pf = GET_FIELD(element->dword0, IGU_FIFO_ELEMENT_DWORD0_IS_PF);
	cmd_addr = GET_FIELD(element->dword0, IGU_FIFO_ELEMENT_DWORD0_CMD_ADDR);
	source = GET_FIELD(element->dword0, IGU_FIFO_ELEMENT_DWORD0_SOURCE);
	err_type = GET_FIELD(element->dword0, IGU_FIFO_ELEMENT_DWORD0_ERR_TYPE);

	if (source >= OSAL_ARRAY_SIZE(s_igu_fifo_source_strs))
		return DBG_STATUS_IGU_FIFO_BAD_DATA;
	if (err_type >= OSAL_ARRAY_SIZE(s_igu_fifo_error_strs))
		return DBG_STATUS_IGU_FIFO_BAD_DATA;

	/* Find address data */
	for (i = 0; i < OSAL_ARRAY_SIZE(s_igu_fifo_addr_data) && !found_addr; i++) {
		const struct igu_fifo_addr_data *curr_addr = &s_igu_fifo_addr_data[i];

		if (cmd_addr >= curr_addr->start_addr && cmd_addr <= curr_addr->end_addr)
			found_addr = curr_addr;
	}

	if (!found_addr)
		return DBG_STATUS_IGU_FIFO_BAD_DATA;

	/* Prepare parsed address data */
	switch (found_addr->type) {
	case IGU_ADDR_TYPE_MSIX_MEM:
		OSAL_SPRINTF(parsed_addr_data, " vector_num = 0x%x", cmd_addr / 2); break;
	case IGU_ADDR_TYPE_WRITE_INT_ACK:
	case IGU_ADDR_TYPE_WRITE_PROD_UPDATE:
		OSAL_SPRINTF(parsed_addr_data, " SB = 0x%x", cmd_addr - found_addr->start_addr); break;
	default:
		parsed_addr_data[0] = '\0';
	}

	/* Prepare parsed write data */
	if (is_wr_cmd) {
		u32 wr_data, prod_cons;
		u8 is_cleanup;

		wr_data = GET_FIELD(dword12, IGU_FIFO_ELEMENT_DWORD12_WR_DATA);
		prod_cons = GET_FIELD(wr_data, IGU_FIFO_WR_DATA_PROD_CONS);
		is_cleanup = GET_FIELD(wr_data, IGU_FIFO_WR_DATA_CMD_TYPE);

		if (source == IGU_SRC_ATTN) {
			OSAL_SPRINTF(parsed_wr_data, "prod: 0x%x, ", prod_cons);
		} else {
			if (is_cleanup) {
				u8 cleanup_val, cleanup_type;

				cleanup_val = GET_FIELD(wr_data, IGU_FIFO_CLEANUP_WR_DATA_CLEANUP_VAL);
				cleanup_type = GET_FIELD(wr_data, IGU_FIFO_CLEANUP_WR_DATA_CLEANUP_TYPE);

				OSAL_SPRINTF(parsed_wr_data, "cmd_type: cleanup, cleanup_val: %s, cleanup_type : %d, ", cleanup_val ? "set" : "clear", cleanup_type);
			} else {
				u8 update_flag, en_dis_int_for_sb, segment, timer_mask;

				update_flag = GET_FIELD(wr_data, IGU_FIFO_WR_DATA_UPDATE_FLAG);
				en_dis_int_for_sb = GET_FIELD(wr_data, IGU_FIFO_WR_DATA_EN_DIS_INT_FOR_SB);
				segment = GET_FIELD(wr_data, IGU_FIFO_WR_DATA_SEGMENT);
				timer_mask = GET_FIELD(wr_data, IGU_FIFO_WR_DATA_TIMER_MASK);

				OSAL_SPRINTF(parsed_wr_data, "cmd_type: prod/cons update, prod/cons: 0x%x, update_flag: %s, en_dis_int_for_sb : %s, segment : %s, timer_mask = %d, ", prod_cons,	update_flag ? "update" : "nop",	en_dis_int_for_sb ? (en_dis_int_for_sb == 1 ? "disable" : "nop") : "enable", segment ? "attn" : "regular", timer_mask);
			}
		}
	} else {
		parsed_wr_data[0] = '\0';
	}

	/* Add parsed element to parsed buffer */
	*results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, *results_offset), "raw: 0x%01x%08x%08x, %s: %d, source : %s, type : %s, cmd_addr : 0x%x(%s%s), %serror: %s\n", element->dword2, element->dword1, element->dword0,
		is_pf ? "pf" : "vf",
		GET_FIELD(element->dword0, IGU_FIFO_ELEMENT_DWORD0_FID),
		s_igu_fifo_source_strs[source],
		is_wr_cmd ? "wr" : "rd",
		cmd_addr,
		(!is_pf && found_addr->vf_desc) ? found_addr->vf_desc : found_addr->desc,
		parsed_addr_data,
		parsed_wr_data,
		s_igu_fifo_error_strs[err_type]);

	return DBG_STATUS_OK;
}

/* Parses an IGU FIFO dump buffer. If result_buf is not OSAL_NULL, the IGU FIFO
 * results are printed to it. In any case, the required results buffer size is
 * assigned to parsed_results_bytes. The parsing status is returned.
 */
static enum dbg_status ecore_parse_igu_fifo_dump(u32 *dump_buf,
						 char *results_buf,
						 u32 *parsed_results_bytes)
{
	const char *section_name, *param_name, *param_str_val;
	u32 param_num_val, num_section_params, num_elements;
	struct igu_fifo_element *elements;
	enum dbg_status status;
	u32 results_offset = 0;
	u8 i;

	/* Read global_params section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "global_params"))
		return DBG_STATUS_IGU_FIFO_BAD_DATA;

	/* Print global params */
	dump_buf += ecore_print_section_params(dump_buf, num_section_params, results_buf, &results_offset);

	/* Read igu_fifo_data section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "igu_fifo_data"))
		return DBG_STATUS_IGU_FIFO_BAD_DATA;
	dump_buf += ecore_read_param(dump_buf, &param_name, &param_str_val, &param_num_val);
	if (OSAL_STRCMP(param_name, "size"))
		return DBG_STATUS_IGU_FIFO_BAD_DATA;
	if (param_num_val % IGU_FIFO_ELEMENT_DWORDS)
		return DBG_STATUS_IGU_FIFO_BAD_DATA;
	num_elements = param_num_val / IGU_FIFO_ELEMENT_DWORDS;
	elements = (struct igu_fifo_element *)dump_buf;

	/* Decode elements */
	for (i = 0; i < num_elements; i++)
		if ((status = ecore_parse_igu_fifo_element(&elements[i], results_buf, &results_offset)) != DBG_STATUS_OK)
			return status;

	results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "fifo contained %d elements", num_elements);

	/* Add 1 for string OSAL_NULL termination */
	*parsed_results_bytes = results_offset + 1;

	return DBG_STATUS_OK;
}

static enum dbg_status ecore_parse_protection_override_dump(u32 *dump_buf,
							    char *results_buf,
							    u32 *parsed_results_bytes)
{
	const char *section_name, *param_name, *param_str_val;
	u32 param_num_val, num_section_params, num_elements;
	struct protection_override_element *elements;
	u32 results_offset = 0;
	u8 i;

	/* Read global_params section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "global_params"))
		return DBG_STATUS_PROTECTION_OVERRIDE_BAD_DATA;

	/* Print global params */
	dump_buf += ecore_print_section_params(dump_buf, num_section_params, results_buf, &results_offset);

	/* Read protection_override_data section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "protection_override_data"))
		return DBG_STATUS_PROTECTION_OVERRIDE_BAD_DATA;
	dump_buf += ecore_read_param(dump_buf, &param_name, &param_str_val, &param_num_val);
	if (OSAL_STRCMP(param_name, "size"))
		return DBG_STATUS_PROTECTION_OVERRIDE_BAD_DATA;
	if (param_num_val % PROTECTION_OVERRIDE_ELEMENT_DWORDS)
		return DBG_STATUS_PROTECTION_OVERRIDE_BAD_DATA;
	num_elements = param_num_val / PROTECTION_OVERRIDE_ELEMENT_DWORDS;
	elements = (struct protection_override_element *)dump_buf;

	/* Decode elements */
	for (i = 0; i < num_elements; i++) {
		u32 address = GET_FIELD(elements[i].data, PROTECTION_OVERRIDE_ELEMENT_ADDRESS) * PROTECTION_OVERRIDE_ELEMENT_ADDR_FACTOR;

		results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "window %2d, address: 0x%07x, size: %7d regs, read: %d, write: %d, read protection: %-12s, write protection: %-12s\n", i, address,
			(u32)GET_FIELD(elements[i].data, PROTECTION_OVERRIDE_ELEMENT_WINDOW_SIZE),
			(u32)GET_FIELD(elements[i].data, PROTECTION_OVERRIDE_ELEMENT_READ),
			(u32)GET_FIELD(elements[i].data, PROTECTION_OVERRIDE_ELEMENT_WRITE),
			s_protection_strs[GET_FIELD(elements[i].data, PROTECTION_OVERRIDE_ELEMENT_READ_PROTECTION)],
			s_protection_strs[GET_FIELD(elements[i].data, PROTECTION_OVERRIDE_ELEMENT_WRITE_PROTECTION)]);
	}

	results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "protection override contained %d elements", num_elements);

	/* Add 1 for string OSAL_NULL termination */
	*parsed_results_bytes = results_offset + 1;

	return DBG_STATUS_OK;
}

/* Parses a FW Asserts dump buffer. if result_buf is not OSAL_NULL, the FW Asserts
 * results are printed to it. in any case, the required results buffer size is
 * assigned to parsed_results_bytes. the parsing status is returned.
 */
static enum dbg_status ecore_parse_fw_asserts_dump(u32 *dump_buf,
						   char *results_buf,
						   u32 *parsed_results_bytes)
{
	u32 num_section_params, param_num_val, i, results_offset = 0;
	const char *param_name, *param_str_val, *section_name;
	bool last_section_found = false;

	*parsed_results_bytes = 0;

	/* Read global_params section */
	dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
	if (OSAL_STRCMP(section_name, "global_params"))
		return DBG_STATUS_FW_ASSERTS_PARSE_FAILED;

	/* Print global params */
	dump_buf += ecore_print_section_params(dump_buf, num_section_params, results_buf, &results_offset);

	while (!last_section_found) {
		dump_buf += ecore_read_section_hdr(dump_buf, &section_name, &num_section_params);
		if (!OSAL_STRCMP(section_name, "fw_asserts")) {
			/* Extract params */
			const char *storm_letter = OSAL_NULL;
			u32 storm_dump_size = 0;

			for (i = 0; i < num_section_params; i++) {
				dump_buf += ecore_read_param(dump_buf, &param_name, &param_str_val, &param_num_val);
				if (!OSAL_STRCMP(param_name, "storm"))
					storm_letter = param_str_val;
				else if (!OSAL_STRCMP(param_name, "size"))
					storm_dump_size = param_num_val;
				else
					return DBG_STATUS_FW_ASSERTS_PARSE_FAILED;
			}

			if (!storm_letter || !storm_dump_size)
				return DBG_STATUS_FW_ASSERTS_PARSE_FAILED;

			/* Print data */
			results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "\n%sSTORM_ASSERT: size=%d\n", storm_letter, storm_dump_size);
				for (i = 0; i < storm_dump_size; i++, dump_buf++)
					results_offset += OSAL_SPRINTF(ecore_get_buf_ptr(results_buf, results_offset), "%08x\n", *dump_buf);
		} else if (!OSAL_STRCMP(section_name, "last")) {
			last_section_found = true;
		} else {
			return DBG_STATUS_FW_ASSERTS_PARSE_FAILED;
		}
	};

	/* Add 1 for string OSAL_NULL termination */
	*parsed_results_bytes = results_offset + 1;

	return DBG_STATUS_OK;
}

/***************************** Public Functions *******************************/

enum dbg_status ecore_dbg_user_set_bin_ptr(struct ecore_hwfn *p_hwfn,
					   const u8 *const bin_ptr)
{
	struct bin_buffer_hdr *buf_hdrs = (struct bin_buffer_hdr *)bin_ptr;
	u8 buf_id;

	/* Convert binary data to debug arrays */
	for (buf_id = 0; buf_id < MAX_BIN_DBG_BUFFER_TYPE; buf_id++)
		ecore_set_dbg_bin_buf(p_hwfn, (enum bin_dbg_buffer_type)buf_id, (u32 *)(bin_ptr + buf_hdrs[buf_id].offset), buf_hdrs[buf_id].length);

	return DBG_STATUS_OK;
}

enum dbg_status ecore_dbg_alloc_user_data(struct ecore_hwfn *p_hwfn,
					  void **user_data_ptr)
{
	*user_data_ptr = OSAL_ZALLOC(p_hwfn->p_dev, GFP_KERNEL, sizeof(struct dbg_tools_user_data));
	if (!(*user_data_ptr))
		return DBG_STATUS_VIRT_MEM_ALLOC_FAILED;

	OSAL_MEMSET(*user_data_ptr, 0, sizeof(struct dbg_tools_user_data));

	/* Set debug binary buffers */
#ifndef USE_DBG_BIN_FILE
	ecore_set_dbg_bin_buf(p_hwfn, BIN_BUF_DBG_IDLE_CHK_PARSING_DATA, idle_chk_rule_parsing_data, sizeof(idle_chk_rule_parsing_data));
	ecore_set_dbg_bin_buf(p_hwfn, BIN_BUF_DBG_ATTN_INDEXES, (u32 *)attn_idx, sizeof(attn_idx));
	ecore_set_dbg_bin_buf(p_hwfn, BIN_BUF_DBG_ATTN_NAME_OFFSETS, attn_name_offsets, sizeof(attn_name_offsets));
	ecore_set_dbg_bin_buf(p_hwfn, BIN_BUF_DBG_BLOCKS_USER_DATA, dbg_blocks_user, sizeof(dbg_blocks_user));
	ecore_set_dbg_bin_buf(p_hwfn, BIN_BUF_DBG_BLOCKS_CHIP_USER_DATA, dbg_blocks_user_per_chip, sizeof(dbg_blocks_user_per_chip));
	ecore_set_dbg_bin_buf(p_hwfn, BIN_BUF_DBG_BUS_LINE_NAME_OFFSETS, dbg_bus_line_name_offsets, sizeof(dbg_bus_line_name_offsets));
	ecore_set_dbg_bin_buf(p_hwfn, BIN_BUF_DBG_PARSING_STRINGS, (u32 *)dbg_strings, sizeof(dbg_strings));
#endif

	return DBG_STATUS_OK;
}

enum dbg_storms ecore_dbg_get_storm_id(const char *storm_name)
{
	u8 i;

	for (i = 0; i < MAX_DBG_STORMS; i++)
		if (!OSAL_STRCMP(s_storm_str[i], storm_name))
			return (enum dbg_storms)i;

	return MAX_DBG_STORMS;
}

enum block_id ecore_dbg_get_block_id(struct ecore_hwfn *p_hwfn,
				     const char *block_name)
{
	const struct dbg_block_user *block;
	u32 block_id;

	for (block_id = 0; block_id < MAX_BLOCK_ID; block_id++) {
		block = (const struct dbg_block_user *)p_hwfn->dbg_arrays[BIN_BUF_DBG_BLOCKS_USER_DATA].ptr + block_id;
		if (!OSAL_STRCMP((const char *)block->name, block_name))
			return (enum block_id)block_id;
	}

	return MAX_BLOCK_ID;
}

enum dbg_bus_storm_modes ecore_dbg_get_storm_mode_id(const char *storm_mode_name)
{
	u8 i;

	for (i = 0; i < MAX_DBG_BUS_STORM_MODES; i++)
		if (!OSAL_STRCMP(s_storm_mode_str[i], storm_mode_name))
			return (enum dbg_bus_storm_modes)i;

	return MAX_DBG_BUS_STORM_MODES;
}

enum dbg_bus_constraint_ops ecore_dbg_get_constraint_op_id(const char *op_name)
{
	u8 i;

	for (i = 0; i < MAX_DBG_BUS_CONSTRAINT_OPS; i++)
		if (!OSAL_STRCMP(s_constraint_op_str[i], op_name))
			return (enum dbg_bus_constraint_ops)i;

	return MAX_DBG_BUS_CONSTRAINT_OPS;
}

const char *ecore_dbg_get_status_str(enum dbg_status status)
{
	return (status < MAX_DBG_STATUS) ? s_status_str[status] : "Invalid debug status";
}

enum dbg_grc_params ecore_dbg_get_grc_param_id(const char *param_name)
{
	u8 i;

	for (i = 0; i < MAX_DBG_GRC_PARAMS; i++)
		if (!OSAL_STRCMP(s_grc_param_str[i], param_name))
			return (enum dbg_grc_params)i;

	return MAX_DBG_GRC_PARAMS;
}

int ecore_dbg_get_dbg_bus_line(struct ecore_hwfn *p_hwfn,
			       enum block_id block_id,
			       enum chip_ids chip_id,
			       const char *line)
{
	const struct dbg_block_chip_user *block = (const struct dbg_block_chip_user *)p_hwfn->dbg_arrays[BIN_BUF_DBG_BLOCKS_CHIP_USER_DATA].ptr + block_id * MAX_CHIP_IDS + chip_id;

	/* Check if line is a number */
	if (ecore_is_number(line)) {
		int rc;
		unsigned long line_num = 0;

		rc = OSAL_STRTOUL(line, 10, &line_num);
		if (rc)
			return -1;
		return ((int)line_num < NUM_DBG_LINES_USER(block)) ? (int)line_num : -1;
	} else if (!OSAL_STRCMP(line, DBG_BUS_SIGNATURE_LINE_NAME)) {
		return DBG_BUS_SIGNATURE_LINE_NUM;
	} else if (!OSAL_STRCMP(line, DBG_BUS_LATENCY_LINE_NAME)) {
		return block->has_latency_events ? DBG_BUS_LATENCY_LINE_NUM : -1;
	} else {
		/* Block-specific debug line string */
		const u32 *block_line_names_buf;
		const char *parsing_str_buf;
		u8 line_num;

		block_line_names_buf = (u32 *)p_hwfn->dbg_arrays[BIN_BUF_DBG_BUS_LINE_NAME_OFFSETS].ptr + block->names_offset;
		parsing_str_buf = (const char *)p_hwfn->dbg_arrays[BIN_BUF_DBG_PARSING_STRINGS].ptr;

		/* Search name in debug lines array */
		for (line_num = 0; line_num < block->num_of_dbg_bus_lines; line_num++)
			if (!OSAL_STRCMP(line, &parsing_str_buf[block_line_names_buf[line_num]]))
				return line_num + NUM_EXTRA_DBG_LINES_USER(block);

		return -1;
	}
}

enum dbg_status ecore_get_idle_chk_results_buf_size(struct ecore_hwfn *p_hwfn,
						    u32 *dump_buf,
						    u32 num_dumped_dwords,
						    u32 *results_buf_size)
{
	u32 num_errors, num_warnings;

	return ecore_parse_idle_chk_dump(p_hwfn, dump_buf, num_dumped_dwords, OSAL_NULL, results_buf_size, &num_errors, &num_warnings);
}

enum dbg_status ecore_print_idle_chk_results(struct ecore_hwfn *p_hwfn,
					     u32 *dump_buf,
					     u32 num_dumped_dwords,
					     char *results_buf,
					     u32 *num_errors,
					     u32 *num_warnings)
{
	u32 parsed_buf_size;

	return ecore_parse_idle_chk_dump(p_hwfn, dump_buf, num_dumped_dwords, results_buf, &parsed_buf_size, num_errors, num_warnings);
}

void ecore_dbg_mcp_trace_set_meta_data(struct ecore_hwfn *p_hwfn,
				       const u32 *meta_buf)
{
	struct dbg_tools_user_data *dev_user_data = ecore_dbg_get_user_data(p_hwfn);

	dev_user_data->mcp_trace_user_meta_buf = meta_buf;
}

enum dbg_status ecore_get_mcp_trace_results_buf_size(struct ecore_hwfn *p_hwfn,
						     u32 *dump_buf,
						     u32 OSAL_UNUSED num_dumped_dwords,
						     u32 *results_buf_size)
{
	return ecore_parse_mcp_trace_dump(p_hwfn, dump_buf, OSAL_NULL, results_buf_size, true);
}

enum dbg_status ecore_print_mcp_trace_results(struct ecore_hwfn *p_hwfn,
					      u32 *dump_buf,
					      u32 OSAL_UNUSED num_dumped_dwords,
					      char *results_buf)
{
	u32 parsed_buf_size;

	/* Doesn't do anything, needed for compile time asserts */
	ecore_user_static_asserts();

	return ecore_parse_mcp_trace_dump(p_hwfn, dump_buf, results_buf, &parsed_buf_size, true);
}

enum dbg_status ecore_print_mcp_trace_results_cont(struct ecore_hwfn *p_hwfn,
						   u32 *dump_buf,
						   u32 OSAL_UNUSED num_dumped_dwords,
						   char *results_buf)
{
	u32 parsed_buf_size;

	return ecore_parse_mcp_trace_dump(p_hwfn, dump_buf, results_buf, &parsed_buf_size, false);
}

enum dbg_status ecore_print_mcp_trace_line(struct ecore_hwfn *p_hwfn,
					   u8 *dump_buf,
					   u32 num_dumped_bytes,
					   char *results_buf)
{
	u32 parsed_results_bytes;

	return ecore_parse_mcp_trace_buf(p_hwfn, dump_buf, num_dumped_bytes, 0, num_dumped_bytes, results_buf, &parsed_results_bytes);
}

/* Frees the specified MCP Trace meta data */
void ecore_mcp_trace_free_meta_data(struct ecore_hwfn *p_hwfn)
{
	struct dbg_tools_user_data *dev_user_data = ecore_dbg_get_user_data(p_hwfn);
	struct mcp_trace_meta *meta = &dev_user_data->mcp_trace_meta;
	u32 i;

	if (!meta->is_allocated)
		return;

	/* Release modules */
	if (meta->modules) {
		for (i = 0; i < meta->modules_num; i++)
			OSAL_FREE(p_hwfn->p_dev, meta->modules[i]);
		OSAL_FREE(p_hwfn->p_dev, meta->modules);
	}

	/* Release formats */
	if (meta->formats) {
		for (i = 0; i < meta->formats_num; i++)
			OSAL_FREE(p_hwfn->p_dev, meta->formats[i].format_str);
		OSAL_FREE(p_hwfn->p_dev, meta->formats);
	}

	meta->is_allocated = false;
}

enum dbg_status ecore_get_reg_fifo_results_buf_size(struct ecore_hwfn OSAL_UNUSED *p_hwfn,
						    u32 *dump_buf,
						    u32 OSAL_UNUSED num_dumped_dwords,
						    u32 *results_buf_size)
{
	return ecore_parse_reg_fifo_dump(dump_buf, OSAL_NULL, results_buf_size);
}

enum dbg_status ecore_print_reg_fifo_results(struct ecore_hwfn OSAL_UNUSED *p_hwfn,
					     u32 *dump_buf,
					     u32 OSAL_UNUSED num_dumped_dwords,
					     char *results_buf)
{
	u32 parsed_buf_size;

	return ecore_parse_reg_fifo_dump(dump_buf, results_buf, &parsed_buf_size);
}

enum dbg_status ecore_get_igu_fifo_results_buf_size(struct ecore_hwfn OSAL_UNUSED *p_hwfn,
						    u32 *dump_buf,
						    u32 OSAL_UNUSED num_dumped_dwords,
						    u32 *results_buf_size)
{
	return ecore_parse_igu_fifo_dump(dump_buf, OSAL_NULL, results_buf_size);
}

enum dbg_status ecore_print_igu_fifo_results(struct ecore_hwfn OSAL_UNUSED *p_hwfn,
					     u32 *dump_buf,
					     u32 OSAL_UNUSED num_dumped_dwords,
					     char *results_buf)
{
	u32 parsed_buf_size;

	return ecore_parse_igu_fifo_dump(dump_buf, results_buf, &parsed_buf_size);
}

enum dbg_status ecore_get_protection_override_results_buf_size(struct ecore_hwfn OSAL_UNUSED *p_hwfn,
							       u32 *dump_buf,
							       u32 OSAL_UNUSED num_dumped_dwords,
							       u32 *results_buf_size)
{
	return ecore_parse_protection_override_dump(dump_buf, OSAL_NULL, results_buf_size);
}

enum dbg_status ecore_print_protection_override_results(struct ecore_hwfn OSAL_UNUSED *p_hwfn,
							u32 *dump_buf,
							u32 OSAL_UNUSED num_dumped_dwords,
							char *results_buf)
{
	u32 parsed_buf_size;

	return ecore_parse_protection_override_dump(dump_buf, results_buf, &parsed_buf_size);
}

enum dbg_status ecore_get_fw_asserts_results_buf_size(struct ecore_hwfn OSAL_UNUSED *p_hwfn,
						      u32 *dump_buf,
						      u32 OSAL_UNUSED num_dumped_dwords,
						      u32 *results_buf_size)
{
	return ecore_parse_fw_asserts_dump(dump_buf, OSAL_NULL, results_buf_size);
}

enum dbg_status ecore_print_fw_asserts_results(struct ecore_hwfn OSAL_UNUSED *p_hwfn,
					       u32 *dump_buf,
					       u32 OSAL_UNUSED num_dumped_dwords,
					       char *results_buf)
{
	u32 parsed_buf_size;

	return ecore_parse_fw_asserts_dump(dump_buf, results_buf, &parsed_buf_size);
}

enum dbg_status ecore_dbg_parse_attn(struct ecore_hwfn *p_hwfn,
				     struct dbg_attn_block_result *results)
{
	const u32 *block_attn_name_offsets;
	enum dbg_attn_type attn_type;
	const char *block_name;
	u8 num_regs, i, j;

	num_regs = GET_FIELD(results->data, DBG_ATTN_BLOCK_RESULT_NUM_REGS);
	attn_type = (enum dbg_attn_type)GET_FIELD(results->data, DBG_ATTN_BLOCK_RESULT_ATTN_TYPE);
	block_name = ecore_dbg_get_block_name(p_hwfn, (enum block_id)results->block_id);
	if (!block_name)
		return DBG_STATUS_INVALID_ARGS;

	if (!p_hwfn->dbg_arrays[BIN_BUF_DBG_ATTN_INDEXES].ptr || !p_hwfn->dbg_arrays[BIN_BUF_DBG_ATTN_NAME_OFFSETS].ptr || !p_hwfn->dbg_arrays[BIN_BUF_DBG_PARSING_STRINGS].ptr)
		return DBG_STATUS_DBG_ARRAY_NOT_SET;

	block_attn_name_offsets = (u32 *)p_hwfn->dbg_arrays[BIN_BUF_DBG_ATTN_NAME_OFFSETS].ptr + results->names_offset;

	/* Go over registers with a non-zero attention status */
	for (i = 0; i < num_regs; i++) {
		struct dbg_attn_bit_mapping *bit_mapping;
		struct dbg_attn_reg_result *reg_result;
		u8 num_reg_attn, bit_idx = 0;

		reg_result = &results->reg_results[i];
		num_reg_attn = GET_FIELD(reg_result->data, DBG_ATTN_REG_RESULT_NUM_REG_ATTN);
		bit_mapping = (struct dbg_attn_bit_mapping *)p_hwfn->dbg_arrays[BIN_BUF_DBG_ATTN_INDEXES].ptr + reg_result->block_attn_offset;

		/* Go over attention status bits */
		for (j = 0; j < num_reg_attn; j++) {
			u16 attn_idx_val = GET_FIELD(bit_mapping[j].data, DBG_ATTN_BIT_MAPPING_VAL);

			/* Check if bit mask should be advanced (due to unused
			 * bits).
			 */
			if (GET_FIELD(bit_mapping[j].data, DBG_ATTN_BIT_MAPPING_IS_UNUSED_BIT_CNT)) {
				bit_idx += (u8)attn_idx_val;
				continue;
			}

			/* Check current bit index */
			if (reg_result->sts_val & (1 << bit_idx)) {
				/* An attention bit with value=1 was found */
				const char *attn_name, *attn_type_str, *masked_str;
				u32 attn_name_offset;
				u32 sts_addr;

				/* Find attention name */
				attn_name_offset = block_attn_name_offsets[attn_idx_val];
				attn_name = (const char *)p_hwfn->dbg_arrays[BIN_BUF_DBG_PARSING_STRINGS].ptr + attn_name_offset;
				attn_type_str = (attn_type == ATTN_TYPE_INTERRUPT ? "Interrupt" : "Parity");
				masked_str = reg_result->mask_val & (1 << bit_idx) ? " [masked]" : "";
				sts_addr = GET_FIELD(reg_result->data, DBG_ATTN_REG_RESULT_STS_ADDRESS);
				DP_NOTICE(p_hwfn, false, "%s (%s) : %s [address 0x%08x, bit %d]%s\n", block_name, attn_type_str, attn_name, sts_addr * 4, bit_idx, masked_str);
			}

			bit_idx++;
		}
	}

	return DBG_STATUS_OK;
}

