#include "bcm_osal.h"

#include "ecore.h"
#include "ecore_status.h"
#include "ecore_ll2.h"
#include "ecore_sp_api.h"
#include "ecore_fcoe.h"
#include "ecore_iro.h"
#include "reg_addr.h"
#include "ecore_int.h"
#include "ecore_cxt.h"
#include "ecore_sp_commands.h"
#include "ecore_gtt_reg_addr.h"
#include "ecore_hw.h"
#include "ecore_dev_api.h"

#ifdef _NTDDK_
#pragma warning(push)
#pragma warning(disable : 28167)
#pragma warning(disable : 28123)
#endif

#define TSTORM_FCOE_CONN_AG_CTX_DUMMY_TIMER_CF_EN_MASK	0x1
#define TSTORM_FCOE_CONN_AG_CTX_DUMMY_TIMER_CF_EN_SHIFT	4

LNX_STATIC enum _ecore_status_t
ecore_sp_fcoe_func_start(struct ecore_hwfn *p_hwfn,
			 enum spq_mode comp_mode,
			 struct ecore_spq_comp_cb *p_comp_addr)
{
	struct ecore_fcoe_pf_params *fcoe_pf_params = OSAL_NULL;
	struct fcoe_init_ramrod_params *p_ramrod = OSAL_NULL;
	struct fcoe_init_func_ramrod_data *p_data;
	struct ecore_spq_entry *p_ent = OSAL_NULL;
	u32 dummy_cid, fcoe_conn_context_size;
	struct ecore_sp_init_data init_data;
	struct fcoe_conn_context *p_cxt;
	struct ecore_cxt_info cxt_info;
	u8 i, num_sbs, *p_flags3;
	enum _ecore_status_t rc = ECORE_SUCCESS;
	u16 tmp;

	/* Get SPQ entry */
	OSAL_MEMSET(&init_data, 0, sizeof(init_data));
	init_data.cid = ecore_spq_get_cid(p_hwfn);
	init_data.opaque_fid = p_hwfn->hw_info.opaque_fid;
	init_data.comp_mode = comp_mode;
	init_data.p_comp_data = p_comp_addr;

	rc = ecore_sp_init_request(p_hwfn, &p_ent,
				   FCOE_RAMROD_CMD_ID_INIT_FUNC,
				   PROTOCOLID_FCOE, &init_data);
	if (rc != ECORE_SUCCESS)
		return rc;

	p_ramrod = &p_ent->ramrod.fcoe_init;
	p_data = &p_ramrod->init_ramrod_data;
	fcoe_pf_params = &p_hwfn->pf_params.fcoe_pf_params;

	/* Sanity */
	if (fcoe_pf_params->num_cqs > p_hwfn->hw_info.feat_num[ECORE_FCOE_CQ]) {
		DP_ERR(p_hwfn,
		       "Cannot satisfy CQ amount. CQs requested %d, CQs available %d. Aborting function start\n",
		       fcoe_pf_params->num_cqs,
		       p_hwfn->hw_info.feat_num[ECORE_FCOE_CQ]);
		rc = ECORE_INVAL;
		goto err;
	}

	p_data->mtu = OSAL_CPU_TO_LE16(fcoe_pf_params->mtu);
	tmp = OSAL_CPU_TO_LE16(fcoe_pf_params->sq_num_pbl_pages);
	p_data->sq_num_pages_in_pbl = tmp;

	rc = ecore_cxt_acquire_cid(p_hwfn, PROTOCOLID_FCOE, &dummy_cid);
	if (rc != ECORE_SUCCESS)
		goto err;

	cxt_info.iid = dummy_cid;
	rc = ecore_cxt_get_cid_info(p_hwfn, &cxt_info);
	if (rc != ECORE_SUCCESS) {
		DP_NOTICE(p_hwfn, true, "Cannot find context info for dummy cid=%d\n",
			  dummy_cid);
		goto err;
	}

	p_cxt = cxt_info.p_cxt;
	fcoe_conn_context_size = sizeof(*p_cxt);
	p_flags3 = &p_cxt->tstorm_ag_context.flags3;

	/* @@@TBD we zero the context until we have ilt_reset implemented. */
	OSAL_MEM_ZERO(p_cxt, fcoe_conn_context_size);

	SET_FIELD(*p_flags3, TSTORM_FCOE_CONN_AG_CTX_DUMMY_TIMER_CF_EN, 1);

	fcoe_pf_params->dummy_icid = (u16)dummy_cid;

	tmp = OSAL_CPU_TO_LE16(fcoe_pf_params->num_tasks);
	p_data->func_params.num_tasks = tmp;
	p_data->func_params.log_page_size = fcoe_pf_params->log_page_size;
	p_data->func_params.debug_mode = fcoe_pf_params->debug_mode;

	DMA_REGPAIR_LE(p_data->q_params.glbl_q_params_addr,
		       fcoe_pf_params->glbl_q_params_addr);

	tmp = OSAL_CPU_TO_LE16(fcoe_pf_params->cq_num_entries);
	p_data->q_params.cq_num_entries = tmp;

	tmp = OSAL_CPU_TO_LE16(fcoe_pf_params->cmdq_num_entries);
	p_data->q_params.cmdq_num_entries = tmp;

	tmp = fcoe_pf_params->num_cqs;
	p_data->q_params.num_queues = (u8)tmp;

	tmp = (u16)p_hwfn->hw_info.resc_start[ECORE_CMDQS_CQS];
	p_data->q_params.queue_relative_offset = (u8)tmp;

	/* In storage target each CQ/CmdQ pair uses a single SB */
	num_sbs = fcoe_pf_params->num_cqs;
	if (fcoe_pf_params->is_target)
		num_sbs /= 2;
	for (i = 0; i < num_sbs; i++) {
		u16 igu_sb_id;

		igu_sb_id = ecore_get_igu_sb_id(p_hwfn, i);
		tmp = OSAL_CPU_TO_LE16(igu_sb_id);
		p_data->q_params.cq_cmdq_sb_num_arr[i] = tmp;
	}

	p_data->q_params.cq_sb_pi = fcoe_pf_params->gl_rq_pi;
	p_data->q_params.cmdq_sb_pi = fcoe_pf_params->gl_cmd_pi;

	p_data->q_params.storage_func_id = (u8)RESC_START(p_hwfn, ECORE_BDQ);

	DMA_REGPAIR_LE(p_data->q_params.bdq_pbl_base_address[BDQ_ID_RQ],
		       fcoe_pf_params->bdq_pbl_base_addr[BDQ_ID_RQ]);
	p_data->q_params.bdq_pbl_num_entries[BDQ_ID_RQ] =
		fcoe_pf_params->bdq_pbl_num_entries[BDQ_ID_RQ];
	tmp = fcoe_pf_params->bdq_xoff_threshold[BDQ_ID_RQ];
	p_data->q_params.bdq_xoff_threshold[BDQ_ID_RQ] = OSAL_CPU_TO_LE16(tmp);
	tmp = fcoe_pf_params->bdq_xon_threshold[BDQ_ID_RQ];
	p_data->q_params.bdq_xon_threshold[BDQ_ID_RQ] = OSAL_CPU_TO_LE16(tmp);

	DMA_REGPAIR_LE(p_data->q_params.bdq_pbl_base_address[BDQ_ID_IMM_DATA],
		       fcoe_pf_params->bdq_pbl_base_addr[BDQ_ID_IMM_DATA]);
	p_data->q_params.bdq_pbl_num_entries[BDQ_ID_IMM_DATA] =
		fcoe_pf_params->bdq_pbl_num_entries[BDQ_ID_IMM_DATA];
	tmp = fcoe_pf_params->bdq_xoff_threshold[BDQ_ID_IMM_DATA];
	p_data->q_params.bdq_xoff_threshold[BDQ_ID_IMM_DATA] =
		OSAL_CPU_TO_LE16(tmp);
	tmp = fcoe_pf_params->bdq_xon_threshold[BDQ_ID_IMM_DATA];
	p_data->q_params.bdq_xon_threshold[BDQ_ID_IMM_DATA] =
		OSAL_CPU_TO_LE16(tmp);
	tmp = fcoe_pf_params->rq_buffer_size;
	p_data->q_params.rq_buffer_size = OSAL_CPU_TO_LE16(tmp);

	if (fcoe_pf_params->is_target) {
		SET_FIELD(p_data->q_params.q_validity,
			  SCSI_INIT_FUNC_QUEUES_RQ_VALID, 1);
		if (p_data->q_params.bdq_pbl_num_entries[BDQ_ID_IMM_DATA]) {
			SET_FIELD(p_data->q_params.q_validity,
				  SCSI_INIT_FUNC_QUEUES_IMM_DATA_VALID, 1);
		}
		SET_FIELD(p_data->q_params.q_validity,
			  SCSI_INIT_FUNC_QUEUES_CMD_VALID, 1);
	} else {
		SET_FIELD(p_data->q_params.q_validity,
			  SCSI_INIT_FUNC_QUEUES_RQ_VALID, 1);
	}

	rc = ecore_spq_post(p_hwfn, p_ent, OSAL_NULL);

	return rc;

err:
	ecore_sp_destroy_request(p_hwfn, p_ent);
	return rc;
}

LNX_STATIC enum _ecore_status_t
ecore_sp_fcoe_conn_offload(struct ecore_hwfn *p_hwfn,
			   struct ecore_fcoe_conn *p_conn,
			   enum spq_mode comp_mode,
			   struct ecore_spq_comp_cb *p_comp_addr)
{
	struct fcoe_conn_offload_ramrod_params *p_ramrod = OSAL_NULL;
	struct fcoe_conn_offload_ramrod_data *p_data;
	struct ecore_spq_entry *p_ent = OSAL_NULL;
	struct ecore_sp_init_data init_data;
	u16 physical_q0, tmp;
	enum _ecore_status_t rc;

	/* Get SPQ entry */
	OSAL_MEMSET(&init_data, 0, sizeof(init_data));
	init_data.cid = p_conn->icid;
	init_data.opaque_fid = p_hwfn->hw_info.opaque_fid;
	init_data.comp_mode = comp_mode;
	init_data.p_comp_data = p_comp_addr;

	rc = ecore_sp_init_request(p_hwfn, &p_ent,
				   FCOE_RAMROD_CMD_ID_OFFLOAD_CONN,
				   PROTOCOLID_FCOE, &init_data);
	if (rc != ECORE_SUCCESS)
		return rc;

	p_ramrod = &p_ent->ramrod.fcoe_conn_ofld;
	p_data = &p_ramrod->offload_ramrod_data;

	/* Transmission PQ is the first of the PF */
	physical_q0 = ecore_get_cm_pq_idx(p_hwfn, PQ_FLAGS_OFLD);
	p_conn->physical_q0 = OSAL_CPU_TO_LE16(physical_q0);
	p_data->physical_q0 = OSAL_CPU_TO_LE16(physical_q0);

	p_data->conn_id = OSAL_CPU_TO_LE16(p_conn->conn_id);
	DMA_REGPAIR_LE(p_data->sq_pbl_addr, p_conn->sq_pbl_addr);
	DMA_REGPAIR_LE(p_data->sq_curr_page_addr, p_conn->sq_curr_page_addr);
	DMA_REGPAIR_LE(p_data->sq_next_page_addr, p_conn->sq_next_page_addr);
	DMA_REGPAIR_LE(p_data->xferq_pbl_addr, p_conn->xferq_pbl_addr);
	DMA_REGPAIR_LE(p_data->xferq_curr_page_addr, p_conn->xferq_addr[0]);
	DMA_REGPAIR_LE(p_data->xferq_next_page_addr, p_conn->xferq_addr[1]);

	DMA_REGPAIR_LE(p_data->respq_pbl_addr, p_conn->confq_pbl_addr);
	DMA_REGPAIR_LE(p_data->respq_curr_page_addr, p_conn->confq_addr[0]);
	DMA_REGPAIR_LE(p_data->respq_next_page_addr, p_conn->confq_addr[1]);

	p_data->dst_mac_addr_lo = OSAL_CPU_TO_LE16(p_conn->dst_mac_addr_lo);
	p_data->dst_mac_addr_mid = OSAL_CPU_TO_LE16(p_conn->dst_mac_addr_mid);
	p_data->dst_mac_addr_hi = OSAL_CPU_TO_LE16(p_conn->dst_mac_addr_hi);
	p_data->src_mac_addr_lo = OSAL_CPU_TO_LE16(p_conn->src_mac_addr_lo);
	p_data->src_mac_addr_mid = OSAL_CPU_TO_LE16(p_conn->src_mac_addr_mid);
	p_data->src_mac_addr_hi = OSAL_CPU_TO_LE16(p_conn->src_mac_addr_hi);

	tmp = OSAL_CPU_TO_LE16(p_conn->tx_max_fc_pay_len);
	p_data->tx_max_fc_pay_len = tmp;
	tmp = OSAL_CPU_TO_LE16(p_conn->e_d_tov_timer_val);
	p_data->e_d_tov_timer_val = tmp;
	tmp = OSAL_CPU_TO_LE16(p_conn->rec_tov_timer_val);
	p_data->rec_rr_tov_timer_val = tmp;
	tmp = OSAL_CPU_TO_LE16(p_conn->rx_max_fc_pay_len);
	p_data->rx_max_fc_pay_len = tmp;

	p_data->vlan_tag = OSAL_CPU_TO_LE16(p_conn->vlan_tag);
	p_data->s_id.addr_hi = p_conn->s_id.addr_hi;
	p_data->s_id.addr_mid = p_conn->s_id.addr_mid;
	p_data->s_id.addr_lo = p_conn->s_id.addr_lo;
	p_data->max_conc_seqs_c3 = p_conn->max_conc_seqs_c3;
	p_data->d_id.addr_hi = p_conn->d_id.addr_hi;
	p_data->d_id.addr_mid = p_conn->d_id.addr_mid;
	p_data->d_id.addr_lo = p_conn->d_id.addr_lo;
	p_data->flags = p_conn->flags;
	if (OSAL_TEST_BIT(ECORE_MF_UFP_SPECIFIC, &p_hwfn->p_dev->mf_bits))
		SET_FIELD(p_data->flags,
			  FCOE_CONN_OFFLOAD_RAMROD_DATA_B_SINGLE_VLAN, 1);
	p_data->def_q_idx = p_conn->def_q_idx;

	return ecore_spq_post(p_hwfn, p_ent, OSAL_NULL);
}

LNX_STATIC enum _ecore_status_t
ecore_sp_fcoe_conn_destroy(struct ecore_hwfn *p_hwfn,
			   struct ecore_fcoe_conn *p_conn,
			   enum spq_mode comp_mode,
			   struct ecore_spq_comp_cb *p_comp_addr)
{
	struct fcoe_conn_terminate_ramrod_params *p_ramrod = OSAL_NULL;
	struct ecore_spq_entry *p_ent = OSAL_NULL;
	struct ecore_sp_init_data init_data;
	enum _ecore_status_t rc	= ECORE_SUCCESS;

	/* Get SPQ entry */
	OSAL_MEMSET(&init_data, 0, sizeof(init_data));
	init_data.cid = p_conn->icid;
	init_data.opaque_fid = p_hwfn->hw_info.opaque_fid;
	init_data.comp_mode = comp_mode;
	init_data.p_comp_data = p_comp_addr;

	rc = ecore_sp_init_request(p_hwfn, &p_ent,
				   FCOE_RAMROD_CMD_ID_TERMINATE_CONN,
				   PROTOCOLID_FCOE, &init_data);
	if (rc != ECORE_SUCCESS)
		return rc;

	p_ramrod = &p_ent->ramrod.fcoe_conn_terminate;
	DMA_REGPAIR_LE(p_ramrod->terminate_ramrod_data.terminate_params_addr,
		       p_conn->terminate_params);

	return ecore_spq_post(p_hwfn, p_ent, OSAL_NULL);
}

LNX_STATIC enum _ecore_status_t
ecore_sp_fcoe_func_stop(struct ecore_hwfn *p_hwfn,
			struct ecore_ptt *p_ptt,
			enum spq_mode comp_mode,
			struct ecore_spq_comp_cb *p_comp_addr)
{
	struct ecore_spq_entry *p_ent = OSAL_NULL;
	struct ecore_sp_init_data init_data;
	u32 active_segs = 0;
	enum _ecore_status_t rc = ECORE_SUCCESS;

	/* Get SPQ entry */
	OSAL_MEMSET(&init_data, 0, sizeof(init_data));
	init_data.cid = p_hwfn->pf_params.fcoe_pf_params.dummy_icid;
	init_data.opaque_fid = p_hwfn->hw_info.opaque_fid;
	init_data.comp_mode = comp_mode;
	init_data.p_comp_data = p_comp_addr;

	rc = ecore_sp_init_request(p_hwfn, &p_ent,
				   FCOE_RAMROD_CMD_ID_DESTROY_FUNC,
				   PROTOCOLID_FCOE, &init_data);
	if (rc != ECORE_SUCCESS)
		return rc;

	active_segs = ecore_rd(p_hwfn, p_ptt, TM_REG_PF_ENABLE_TASK);
	active_segs &= ~(1 << ECORE_CXT_FCOE_TID_SEG);
	ecore_wr(p_hwfn, p_ptt, TM_REG_PF_ENABLE_TASK, active_segs);

	return ecore_spq_post(p_hwfn, p_ent, OSAL_NULL);
}

static enum _ecore_status_t
ecore_fcoe_allocate_connection(struct ecore_hwfn *p_hwfn,
			       struct ecore_fcoe_conn **p_out_conn)
{
	struct ecore_fcoe_conn *p_conn = OSAL_NULL;
	void *p_addr;
	u32 i;

	OSAL_SPIN_LOCK(&p_hwfn->p_fcoe_info->lock);
	if (!OSAL_LIST_IS_EMPTY(&p_hwfn->p_fcoe_info->free_list))
		p_conn = OSAL_LIST_FIRST_ENTRY(&p_hwfn->p_fcoe_info->free_list,
					       struct ecore_fcoe_conn,
					       list_entry);
	if (p_conn) {
		OSAL_LIST_REMOVE_ENTRY(&p_conn->list_entry,
				       &p_hwfn->p_fcoe_info->free_list);
		OSAL_SPIN_UNLOCK(&p_hwfn->p_fcoe_info->lock);
		*p_out_conn = p_conn;
		return ECORE_SUCCESS;
	}
	OSAL_SPIN_UNLOCK(&p_hwfn->p_fcoe_info->lock);

	p_conn = OSAL_ZALLOC(p_hwfn->p_dev, GFP_KERNEL,
			     sizeof(*p_conn));
	if (!p_conn)
		return ECORE_NOMEM;

	p_addr = OSAL_DMA_ALLOC_COHERENT(p_hwfn->p_dev,
					&p_conn->xferq_pbl_addr,
					ECORE_CHAIN_PAGE_SIZE);
	if (!p_addr)
		goto nomem_pbl_xferq;
	p_conn->xferq_pbl_addr_virt_addr = p_addr;

	for (i = 0; i < OSAL_ARRAY_SIZE(p_conn->xferq_addr); i++) {
		p_addr = OSAL_DMA_ALLOC_COHERENT(p_hwfn->p_dev,
						 &p_conn->xferq_addr[i],
						 ECORE_CHAIN_PAGE_SIZE);
		if (!p_addr)
			goto nomem_xferq;
		p_conn->xferq_addr_virt_addr[i] = p_addr;

		p_addr = p_conn->xferq_pbl_addr_virt_addr;
		((dma_addr_t *)p_addr)[i] = p_conn->xferq_addr[i];
	}

	p_addr = OSAL_DMA_ALLOC_COHERENT(p_hwfn->p_dev,
					 &p_conn->confq_pbl_addr,
					 ECORE_CHAIN_PAGE_SIZE);
	if (!p_addr)
		goto nomem_xferq;
	p_conn->confq_pbl_addr_virt_addr = p_addr;

	for (i = 0; i < OSAL_ARRAY_SIZE(p_conn->confq_addr); i++) {
		p_addr = OSAL_DMA_ALLOC_COHERENT(p_hwfn->p_dev,
						 &p_conn->confq_addr[i],
						 ECORE_CHAIN_PAGE_SIZE);
		if (!p_addr)
			goto nomem_confq;
		p_conn->confq_addr_virt_addr[i] = p_addr;

		p_addr = p_conn->confq_pbl_addr_virt_addr;
		((dma_addr_t *)p_addr)[i] = p_conn->confq_addr[i];
	}

	p_conn->free_on_delete = true;
	*p_out_conn = p_conn;
	return ECORE_SUCCESS;

nomem_confq:
	OSAL_DMA_FREE_COHERENT(p_hwfn->p_dev,
			       p_conn->confq_pbl_addr_virt_addr,
			       p_conn->confq_pbl_addr,
			       ECORE_CHAIN_PAGE_SIZE);
	for (i = 0; i < OSAL_ARRAY_SIZE(p_conn->confq_addr); i++) {
		if (p_conn->confq_addr_virt_addr[i])
			OSAL_DMA_FREE_COHERENT(p_hwfn->p_dev,
					       p_conn->confq_addr_virt_addr[i],
					       p_conn->confq_addr[i],
					       ECORE_CHAIN_PAGE_SIZE);
	}
nomem_xferq:
	OSAL_DMA_FREE_COHERENT(p_hwfn->p_dev,
			       p_conn->xferq_pbl_addr_virt_addr,
			       p_conn->xferq_pbl_addr,
			       ECORE_CHAIN_PAGE_SIZE);
	for (i = 0; i < OSAL_ARRAY_SIZE(p_conn->xferq_addr); i++) {
		if (p_conn->xferq_addr_virt_addr[i])
			OSAL_DMA_FREE_COHERENT(p_hwfn->p_dev,
					       p_conn->xferq_addr_virt_addr[i],
					       p_conn->xferq_addr[i],
					       ECORE_CHAIN_PAGE_SIZE);
	}
nomem_pbl_xferq:
	OSAL_FREE(p_hwfn->p_dev, p_conn);
	return ECORE_NOMEM;
}

static void ecore_fcoe_free_connection(struct ecore_hwfn *p_hwfn,
				       struct ecore_fcoe_conn *p_conn)
{
	u32 i;

	if (!p_conn)
		return;

	if (p_conn->confq_pbl_addr_virt_addr)
		OSAL_DMA_FREE_COHERENT(p_hwfn->p_dev,
				       p_conn->confq_pbl_addr_virt_addr,
				       p_conn->confq_pbl_addr,
				       ECORE_CHAIN_PAGE_SIZE);

	for (i = 0; i < OSAL_ARRAY_SIZE(p_conn->confq_addr); i++) {
		if (!p_conn->confq_addr_virt_addr[i])
			continue;
		OSAL_DMA_FREE_COHERENT(p_hwfn->p_dev,
				       p_conn->confq_addr_virt_addr[i],
				       p_conn->confq_addr[i],
				       ECORE_CHAIN_PAGE_SIZE);
	}

	if (p_conn->xferq_pbl_addr_virt_addr)
		OSAL_DMA_FREE_COHERENT(p_hwfn->p_dev,
				       p_conn->xferq_pbl_addr_virt_addr,
				       p_conn->xferq_pbl_addr,
				       ECORE_CHAIN_PAGE_SIZE);

	for (i = 0; i < OSAL_ARRAY_SIZE(p_conn->xferq_addr); i++) {
		if (!p_conn->xferq_addr_virt_addr[i])
			continue;
		OSAL_DMA_FREE_COHERENT(p_hwfn->p_dev,
				       p_conn->xferq_addr_virt_addr[i],
				       p_conn->xferq_addr[i],
				       ECORE_CHAIN_PAGE_SIZE);
	}

	OSAL_FREE(p_hwfn->p_dev, p_conn);
}

LNX_STATIC void OSAL_IOMEM *
ecore_fcoe_get_db_addr(struct ecore_hwfn *p_hwfn, u32 cid)
{
	return (u8 OSAL_IOMEM *)p_hwfn->doorbells +
				DB_ADDR(cid, DQ_DEMS_LEGACY);
}

#ifndef __EXTRACT__LINUX__THROW__
void OSAL_IOMEM *ecore_fcoe_get_global_cmdq_cons(struct ecore_hwfn *p_hwfn,
						u8 relative_q_id)
{
	u8 hw_q_id = (u8)p_hwfn->hw_info.resc_start[ECORE_CMDQS_CQS] +
				relative_q_id;

	return (u8 OSAL_IOMEM *)p_hwfn->regview +
				GTT_BAR0_MAP_REG_TSDM_RAM +
				TSTORM_SCSI_CMDQ_CONS_OFFSET(hw_q_id);
}
#endif

LNX_STATIC void OSAL_IOMEM *
ecore_fcoe_get_primary_bdq_prod(struct ecore_hwfn *p_hwfn, u8 bdq_id)
{
	if (RESC_NUM(p_hwfn, ECORE_BDQ)) {
		return (u8 OSAL_IOMEM *)p_hwfn->regview +
			GTT_BAR0_MAP_REG_MSDM_RAM +
			MSTORM_SCSI_BDQ_EXT_PROD_OFFSET(
			RESC_START(p_hwfn, ECORE_BDQ), bdq_id);
	} else {
		DP_NOTICE(p_hwfn, true, "BDQ is not allocated!\n");
		return OSAL_NULL;
	}
}

LNX_STATIC void OSAL_IOMEM *
ecore_fcoe_get_secondary_bdq_prod(struct ecore_hwfn *p_hwfn, u8 bdq_id)
{
	if (RESC_NUM(p_hwfn, ECORE_BDQ)) {
		return (u8 OSAL_IOMEM *)p_hwfn->regview +
			GTT_BAR0_MAP_REG_TSDM_RAM +
			TSTORM_SCSI_BDQ_EXT_PROD_OFFSET(
			RESC_START(p_hwfn, ECORE_BDQ), bdq_id);
	} else {
		DP_NOTICE(p_hwfn, true, "BDQ is not allocated!\n");
		return OSAL_NULL;
	}
}

enum _ecore_status_t ecore_fcoe_alloc(struct ecore_hwfn *p_hwfn)
{
	struct ecore_fcoe_info *p_fcoe_info;

	/* Allocate LL2's set struct */
	p_fcoe_info = OSAL_ZALLOC(p_hwfn->p_dev, GFP_KERNEL,
				  sizeof(*p_fcoe_info));
	if (!p_fcoe_info) {
		DP_NOTICE(p_hwfn, true,
			  "Failed to allocate ecore_fcoe_info'\n");
		return ECORE_NOMEM;
	}

	OSAL_LIST_INIT(&p_fcoe_info->free_list);
#ifdef CONFIG_ECORE_LOCK_ALLOC
	OSAL_SPIN_LOCK_ALLOC(p_hwfn, &p_fcoe_info->lock);
#endif

	p_hwfn->p_fcoe_info = p_fcoe_info;
	return ECORE_SUCCESS;
}

#define TSTORM_FCOE_TASK_AG_CTX_CONNECTION_TYPE_MASK	0xF
#define TSTORM_FCOE_TASK_AG_CTX_CONNECTION_TYPE_SHIFT	0

void ecore_fcoe_setup(struct ecore_hwfn *p_hwfn)
{
	struct fcoe_task_context *p_task_ctx = OSAL_NULL;
	__le32 *p_logical_client_0, *p_logical_client_1;
	u32 i, fcoe_task_context_size;
	u8 *p_flags0;
	enum _ecore_status_t rc;

	OSAL_SPIN_LOCK_INIT(&p_hwfn->p_fcoe_info->lock);
	for (i = 0; i < p_hwfn->pf_params.fcoe_pf_params.num_tasks; i++) {
		rc = ecore_cxt_get_task_ctx(p_hwfn, i,
					    ECORE_CTX_WORKING_MEM,
					    (void **)&p_task_ctx);
		if (rc != ECORE_SUCCESS)
			continue;

		fcoe_task_context_size = sizeof(*p_task_ctx);
		p_logical_client_0 =
			&p_task_ctx->timer_context.logical_client_0;
		p_logical_client_1 =
			&p_task_ctx->timer_context.logical_client_1;
		p_flags0 = &p_task_ctx->tstorm_ag_context.flags0;

		OSAL_MEM_ZERO(p_task_ctx, fcoe_task_context_size);
		SET_FIELD(*p_logical_client_0, TIMERS_CONTEXT_VALIDLC0, 1);
		SET_FIELD(*p_logical_client_1, TIMERS_CONTEXT_VALIDLC1, 1);
		SET_FIELD(*p_flags0, TSTORM_FCOE_TASK_AG_CTX_CONNECTION_TYPE,
			  1);
	}
}

void ecore_fcoe_free(struct ecore_hwfn *p_hwfn)
{
	struct ecore_fcoe_conn *p_conn = OSAL_NULL;

	if (!p_hwfn->p_fcoe_info)
		return;

	while (!OSAL_LIST_IS_EMPTY(&p_hwfn->p_fcoe_info->free_list)) {
		p_conn = OSAL_LIST_FIRST_ENTRY(&p_hwfn->p_fcoe_info->free_list,
					       struct ecore_fcoe_conn,
					       list_entry);
		if (p_conn == OSAL_NULL)
			break;
#if defined(_NTDDK_)
#pragma warning(suppress : 6011)
#endif
		OSAL_LIST_REMOVE_ENTRY(&p_conn->list_entry,
				       &p_hwfn->p_fcoe_info->free_list);
		ecore_fcoe_free_connection(p_hwfn, p_conn);
	}

#ifdef CONFIG_ECORE_LOCK_ALLOC
	OSAL_SPIN_LOCK_DEALLOC(&p_hwfn->p_fcoe_info->lock);
#endif

	OSAL_FREE(p_hwfn->p_dev, p_hwfn->p_fcoe_info);
	p_hwfn->p_fcoe_info = OSAL_NULL;
}

LNX_STATIC enum _ecore_status_t
ecore_fcoe_acquire_connection(struct ecore_hwfn *p_hwfn,
			      struct ecore_fcoe_conn *p_in_conn,
			      struct ecore_fcoe_conn **p_out_conn)
{
	struct ecore_fcoe_conn *p_conn = OSAL_NULL;
	enum _ecore_status_t rc = ECORE_SUCCESS;
	u32 icid;

	OSAL_SPIN_LOCK(&p_hwfn->p_fcoe_info->lock);
	rc = ecore_cxt_acquire_cid(p_hwfn, PROTOCOLID_FCOE, &icid);
	OSAL_SPIN_UNLOCK(&p_hwfn->p_fcoe_info->lock);
	if (rc != ECORE_SUCCESS)
		return rc;

	/* Use input connection [if provided] or allocate a new one */
	if (p_in_conn) {
		p_conn = p_in_conn;
	} else {
		rc = ecore_fcoe_allocate_connection(p_hwfn, &p_conn);
		if (rc != ECORE_SUCCESS) {
			OSAL_SPIN_LOCK(&p_hwfn->p_fcoe_info->lock);
			ecore_cxt_release_cid(p_hwfn, icid);
			OSAL_SPIN_UNLOCK(&p_hwfn->p_fcoe_info->lock);
			return rc;
		}
	}

	p_conn->icid = icid;
	p_conn->fw_cid = (p_hwfn->hw_info.opaque_fid << 16) | icid;
	*p_out_conn = p_conn;

	return rc;
}

#ifndef __EXTRACT__LINUX__THROW__
enum _ecore_status_t
ecore_fcoe_offload_connection(struct ecore_hwfn *p_hwfn,
			      struct ecore_fcoe_conn *p_conn)
{
	return ecore_sp_fcoe_conn_offload(p_hwfn, p_conn,
					  ECORE_SPQ_MODE_EBLOCK, OSAL_NULL);
}

enum _ecore_status_t
ecore_fcoe_terminate_connection(struct ecore_hwfn *p_hwfn,
				struct ecore_fcoe_conn *p_conn)
{
	return ecore_sp_fcoe_conn_destroy(p_hwfn, p_conn,
					  ECORE_SPQ_MODE_EBLOCK, OSAL_NULL);
}
#endif

LNX_STATIC void ecore_fcoe_release_connection(struct ecore_hwfn *p_hwfn,
					      struct ecore_fcoe_conn *p_conn)
{
	OSAL_SPIN_LOCK(&p_hwfn->p_fcoe_info->lock);
	OSAL_LIST_PUSH_TAIL(&p_conn->list_entry,
			    &p_hwfn->p_fcoe_info->free_list);
	ecore_cxt_release_cid(p_hwfn, p_conn->icid);
	OSAL_SPIN_UNLOCK(&p_hwfn->p_fcoe_info->lock);
}

static void _ecore_fcoe_get_tstats(struct ecore_hwfn *p_hwfn,
				   struct ecore_ptt *p_ptt,
				   struct ecore_fcoe_stats *p_stats)
{
	struct fcoe_rx_stat tstats;
	u32 tstats_addr;

	OSAL_MEMSET(&tstats, 0, sizeof(tstats));
	tstats_addr = BAR0_MAP_REG_TSDM_RAM +
		TSTORM_FCOE_RX_STATS_OFFSET(p_hwfn->rel_pf_id);
	ecore_memcpy_from(p_hwfn, p_ptt, &tstats, tstats_addr, sizeof(tstats));

	p_stats->fcoe_rx_byte_cnt =
		HILO_64_REGPAIR(tstats.fcoe_rx_byte_cnt);
	p_stats->fcoe_rx_data_pkt_cnt =
		HILO_64_REGPAIR(tstats.fcoe_rx_data_pkt_cnt);
	p_stats->fcoe_rx_xfer_pkt_cnt =
		HILO_64_REGPAIR(tstats.fcoe_rx_xfer_pkt_cnt);
	p_stats->fcoe_rx_other_pkt_cnt =
		HILO_64_REGPAIR(tstats.fcoe_rx_other_pkt_cnt);

	p_stats->fcoe_silent_drop_pkt_cmdq_full_cnt =
		OSAL_LE32_TO_CPU(tstats.fcoe_silent_drop_pkt_cmdq_full_cnt);
	p_stats->fcoe_silent_drop_pkt_rq_full_cnt =
		OSAL_LE32_TO_CPU(tstats.fcoe_silent_drop_pkt_rq_full_cnt);
	p_stats->fcoe_silent_drop_pkt_crc_error_cnt =
		OSAL_LE32_TO_CPU(tstats.fcoe_silent_drop_pkt_crc_error_cnt);
	p_stats->fcoe_silent_drop_pkt_task_invalid_cnt =
		OSAL_LE32_TO_CPU(tstats.fcoe_silent_drop_pkt_task_invalid_cnt);
	p_stats->fcoe_silent_drop_total_pkt_cnt =
		OSAL_LE32_TO_CPU(tstats.fcoe_silent_drop_total_pkt_cnt);
}

static void _ecore_fcoe_get_pstats(struct ecore_hwfn *p_hwfn,
				   struct ecore_ptt *p_ptt,
				   struct ecore_fcoe_stats *p_stats)
{
	struct fcoe_tx_stat pstats;
	u32 pstats_addr;

	OSAL_MEMSET(&pstats, 0, sizeof(pstats));
	pstats_addr = BAR0_MAP_REG_PSDM_RAM +
		PSTORM_FCOE_TX_STATS_OFFSET(p_hwfn->rel_pf_id);
	ecore_memcpy_from(p_hwfn, p_ptt, &pstats, pstats_addr, sizeof(pstats));

	p_stats->fcoe_tx_byte_cnt =
		HILO_64_REGPAIR(pstats.fcoe_tx_byte_cnt);
	p_stats->fcoe_tx_data_pkt_cnt =
		HILO_64_REGPAIR(pstats.fcoe_tx_data_pkt_cnt);
	p_stats->fcoe_tx_xfer_pkt_cnt =
		HILO_64_REGPAIR(pstats.fcoe_tx_xfer_pkt_cnt);
	p_stats->fcoe_tx_other_pkt_cnt =
		HILO_64_REGPAIR(pstats.fcoe_tx_other_pkt_cnt);
}

LNX_STATIC enum _ecore_status_t
ecore_fcoe_get_stats(struct ecore_hwfn *p_hwfn,
		     struct ecore_fcoe_stats *p_stats)
{
	struct ecore_ptt *p_ptt;

	OSAL_MEMSET(p_stats, 0, sizeof(*p_stats));

	p_ptt = ecore_ptt_acquire(p_hwfn);

	if (!p_ptt) {
		DP_ERR(p_hwfn, "Failed to acquire ptt\n");
		return ECORE_INVAL;
	}

	_ecore_fcoe_get_tstats(p_hwfn, p_ptt, p_stats);
	_ecore_fcoe_get_pstats(p_hwfn, p_ptt, p_stats);

	ecore_ptt_release(p_hwfn, p_ptt);

	return ECORE_SUCCESS;
}

#ifdef _NTDDK_
#pragma warning(pop)
#endif
