#ifndef __ECORE_HSI_INIT_FUNC__
#define __ECORE_HSI_INIT_FUNC__ 
/********************************/
/* HSI Init Functions constants */
/********************************/

/* Number of VLAN priorities */
#define NUM_OF_VLAN_PRIORITIES			8

/* Size of CRC8 lookup table */
#ifndef LINUX_REMOVE
#define CRC8_TABLE_SIZE					256
#endif

/*
 * GFS Context Command 0 enum
 */
enum gfs_cntx_cmd0
{
	e_cntx_cmd0_do_always /* do always */,
	e_cntx_cmd0_if_red_not_met /* if redirection condition is not met */,
	MAX_GFS_CNTX_CMD0
};


/*
 * GFS Context Command 1 enum
 */
enum gfs_cntx_cmd1
{
	e_cntx_cmd1_disable /* disabled */,
	e_cntx_cmd1_do_always /* do always */,
	e_cntx_cmd1_do_if_red_met /* do if redirection condition is met */,
	e_cntx_cmd1_do_if_samp_met /* do if  sampling  condition is met */,
	MAX_GFS_CNTX_CMD1
};


/*
 * GFS Context Command 2 enum
 */
enum gfs_cntx_cmd2
{
	e_cntx_cmd2_disable /* disabled */,
	e_cntx_cmd2_cpy_single_vport_always /* copy to a single Vport always */,
	e_cntx_cmd2_cpy_single_vport_cpy_met /* copy to a single Vport if copy condition is met */,
	e_cntx_cmd2_cpy_mul_vport_always /* copy to multiple Vports always */,
	e_cntx_cmd2_cpy_mul_vport_cpy_met /* copy to multiple Vports if copy condition is met */,
	e_cntx_cmd2_do_if_samp_met /* do if  sampling  condition is met */,
	MAX_GFS_CNTX_CMD2
};


/*
 * GFS context Descriptot QREG (0)
 */
struct gfs_context_qreg0
{
	u32 bitfields0;
#define GFS_CONTEXT_QREG0_RESERVED_VALID_MASK            0x1 /* Reserved for valid bit in searcher database */
#define GFS_CONTEXT_QREG0_RESERVED_VALID_SHIFT           0
#define GFS_CONTEXT_QREG0_RESERVED_LAST_MASK             0x1 /* Reserved for last bit in searcher database */
#define GFS_CONTEXT_QREG0_RESERVED_LAST_SHIFT            1
#define GFS_CONTEXT_QREG0_FW_RESERVED0_MASK              0x1F
#define GFS_CONTEXT_QREG0_FW_RESERVED0_SHIFT             2
#define GFS_CONTEXT_QREG0_SKIP_LAST_SEARCH_MASK          0x1 /* Skips the last search in the bundle according to the logic descried is GFS spec */
#define GFS_CONTEXT_QREG0_SKIP_LAST_SEARCH_SHIFT         7
#define GFS_CONTEXT_QREG0_CONTEXT_TYPE_MASK              0x3 /* Always use leading. (Leading, Accumulative, both leading and accumulative) (use enum gfs_context_type) */
#define GFS_CONTEXT_QREG0_CONTEXT_TYPE_SHIFT             8
#define GFS_CONTEXT_QREG0_COMMAND0_COND_MASK             0x1 /* 0   do always , 1   if redirection condition is not met (use enum gfs_cntx_cmd0) */
#define GFS_CONTEXT_QREG0_COMMAND0_COND_SHIFT            10
#define GFS_CONTEXT_QREG0_COMMAND0_CHANGE_VPORT_MASK     0x1 /* if set, change the vport */
#define GFS_CONTEXT_QREG0_COMMAND0_CHANGE_VPORT_SHIFT    11
#define GFS_CONTEXT_QREG0_COMMAND0_FW_HINT_MASK          0x7 /* Substituted to  resolution  structure for FW use (use enum gfs_res_struct_dst_type) */
#define GFS_CONTEXT_QREG0_COMMAND0_FW_HINT_SHIFT         12
#define GFS_CONTEXT_QREG0_COMMAND0_LOCATION_MASK         0x7F /* Command ID, 0   From the 3rd QREG of this context (QREG#2), 1-127   From canned Commands RAM */
#define GFS_CONTEXT_QREG0_COMMAND0_LOCATION_SHIFT        15
#define GFS_CONTEXT_QREG0_COMMAND0_VPORT_MASK            0xFF /* Used if  ChangeVport  is set */
#define GFS_CONTEXT_QREG0_COMMAND0_VPORT_SHIFT           22
#define GFS_CONTEXT_QREG0_COMMAND1_COND_MASK             0x3 /* 0   disabled, 1   do always, 2   do if redirection condition is met, 3   do if  sampling  condition is met (use enum gfs_cntx_cmd1) */
#define GFS_CONTEXT_QREG0_COMMAND1_COND_SHIFT            30
	u32 priority /* Priority of the leading context */;
	u32 bitfields1;
#define GFS_CONTEXT_QREG0_COMMAND2_COND_MASK             0x7 /* 0   disabled, 1   copy to a single Vport always ,2   copy to a single Vport if copy condition is met ,3   copy to multiple Vports always, 4   copy to multiple Vports if copy condition is met, 5   do if  sampling  condition is met (use enum gfs_cntx_cmd2) */
#define GFS_CONTEXT_QREG0_COMMAND2_COND_SHIFT            0
#define GFS_CONTEXT_QREG0_COMMAND1_CHANGE_VPORT_MASK     0x1 /* is set, change the vport */
#define GFS_CONTEXT_QREG0_COMMAND1_CHANGE_VPORT_SHIFT    3
#define GFS_CONTEXT_QREG0_COMMAND1_FW_HINT_MASK          0x7 /* Substituted to  resolution  structure for FW use (use enum gfs_res_struct_dst_type) */
#define GFS_CONTEXT_QREG0_COMMAND1_FW_HINT_SHIFT         4
#define GFS_CONTEXT_QREG0_COMMAND1_LOCATION_MASK         0x7F /* 0   From the 3rd QREG of this context (QREG#2), 1-127   From canned Commands RAM */
#define GFS_CONTEXT_QREG0_COMMAND1_LOCATION_SHIFT        7
#define GFS_CONTEXT_QREG0_COMMAND1_VPORT_MASK            0xFF /* Used if  ChangeVport  is set */
#define GFS_CONTEXT_QREG0_COMMAND1_VPORT_SHIFT           14
#define GFS_CONTEXT_QREG0_COPY_CONDITION_EN_MASK         0x3FF /* Matched with 8 TCP flags and 2 additional bits supplied in the GFS header. */
#define GFS_CONTEXT_QREG0_COPY_CONDITION_EN_SHIFT        22
	u32 bitfields2;
#define GFS_CONTEXT_QREG0_COMMAND2_CHANGE_VPORT_MASK     0x1 /* is set, change the vport */
#define GFS_CONTEXT_QREG0_COMMAND2_CHANGE_VPORT_SHIFT    0
#define GFS_CONTEXT_QREG0_COMMAND2_FW_HINT_MASK          0x7 /* Substituted to  resolution  structure for FW use (use enum gfs_res_struct_dst_type) */
#define GFS_CONTEXT_QREG0_COMMAND2_FW_HINT_SHIFT         1
#define GFS_CONTEXT_QREG0_COMMAND2_LOCATION_MASK         0x7F /* 0   From the 3rd QREG of this context (QREG#2), 1-127   From canned Commands RAM */
#define GFS_CONTEXT_QREG0_COMMAND2_LOCATION_SHIFT        4
#define GFS_CONTEXT_QREG0_COMMAND2_VPORT_MASK            0xFF /* For copy to a single Vport   Vport number.For copy to multiple Vports: 0-31   index of RAM entry that contains the Vports bitmask, 0xFF   the bitmask is located in QREG#2 and QREG#3 */
#define GFS_CONTEXT_QREG0_COMMAND2_VPORT_SHIFT           11
#define GFS_CONTEXT_QREG0_REDIRECTION_CONDITION_EN_MASK  0xF /* Matched with TTL==0 (bit 0), TTL==1 (bit 1) from eth_gfs_redirect_condition in action_data.redirect.condition, and 2 additional bits (2 and 3) supplied in the GFS header. */
#define GFS_CONTEXT_QREG0_REDIRECTION_CONDITION_EN_SHIFT 19
#define GFS_CONTEXT_QREG0_FW_RESERVED1_MASK              0x1FF
#define GFS_CONTEXT_QREG0_FW_RESERVED1_SHIFT             23
};


/*
 * GFS context type
 */
enum gfs_context_type
{
	e_gfs_cntx_lead /* leading context type */,
	e_gfs_cntx_accum /* accumulative context type */,
	e_gfs_cntx_lead_and_accum /* leading and accumulative context type */,
	MAX_GFS_CONTEXT_TYPE
};


/*
 * Modify tunnel GRE header data
 */
struct gfs_modify_tunnel_gre_header_data
{
	u16 key_bits0to15 /* Tunnel ID bits  0..15. */;
	u16 key_bits16to31 /* Tunnel ID bits 16..31. */;
	u16 bitfields0;
#define GFS_MODIFY_TUNNEL_GRE_HEADER_DATA_UPDATE_CHECKSUM_MASK  0x1 /* Update GRE checksum if exists */
#define GFS_MODIFY_TUNNEL_GRE_HEADER_DATA_UPDATE_CHECKSUM_SHIFT 0
#define GFS_MODIFY_TUNNEL_GRE_HEADER_DATA_SET_VALID_MASK        0x1 /* Set Key */
#define GFS_MODIFY_TUNNEL_GRE_HEADER_DATA_SET_VALID_SHIFT       1
#define GFS_MODIFY_TUNNEL_GRE_HEADER_DATA_RESERVED_MASK         0x3FFF
#define GFS_MODIFY_TUNNEL_GRE_HEADER_DATA_RESERVED_SHIFT        2
};


/*
 * Modify tunnel IPV4 header flags
 */
struct gfs_modify_tunnel_ipv4_header_flags
{
	u16 bitfields0;
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_MODIFY_TUNN_IP_SRC_ADDR_MASK    0x1 /* Modify tunnel source IP address. */
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_MODIFY_TUNN_IP_SRC_ADDR_SHIFT   0
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_MODIFY_TUNN_IP_DST_ADDR_MASK    0x1 /* Modify tunnel destination IP address. */
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_MODIFY_TUNN_IP_DST_ADDR_SHIFT   1
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_SET_TUNN_IP_TTL_HOP_LIMIT_MASK  0x1 /* Modify tunnel IP TTL or Hop Limit */
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_SET_TUNN_IP_TTL_HOP_LIMIT_SHIFT 2
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_DEC_TUNN_IP_TTL_HOP_LIMIT_MASK  0x1 /* Decrement tunnel IP TTL or Hop Limit */
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_DEC_TUNN_IP_TTL_HOP_LIMIT_SHIFT 3
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_MODIFY_TUNN_IP_DSCP_MASK        0x1 /* Modify tunnel IP DSCP */
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_FW_MODIFY_TUNN_IP_DSCP_SHIFT       4
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_RESERVED_MASK                      0x7FF
#define GFS_MODIFY_TUNNEL_IPV4_HEADER_FLAGS_RESERVED_SHIFT                     5
};


/*
 * Modify tunnel VXLAN header data
 */
struct gfs_modify_tunnel_vxlan_header_data
{
	u16 vni_bits0to15 /* Tunnel ID bits  0..15 */;
	u8 vni_bits16to23 /* Tunnel ID bits 16..23 */;
	u8 bitfields0;
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_RESERVED0_MASK        0x7
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_RESERVED0_SHIFT       0
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_UPDATE_CHECKSUM_MASK  0x1 /* Update UDP checksum if exists */
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_UPDATE_CHECKSUM_SHIFT 3
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_SET_VNI_MASK          0x1 /* Set VNI */
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_SET_VNI_SHIFT         4
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_RESERVED1_MASK        0x1
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_RESERVED1_SHIFT       5
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_SET_ENTROPY_MASK      0x1 /* Set       UDP source port. */
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_SET_ENTROPY_SHIFT     6
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_CALC_ENTROPY_MASK     0x1 /* Calculate UDP source port from 4-tuple. */
#define GFS_MODIFY_TUNNEL_VXLAN_HEADER_DATA_CALC_ENTROPY_SHIFT    7
	u16 entropy /* VXLAN UDP source value. */;
};


/*
 * Push tunnel GRE header data
 */
struct gfs_push_tunnel_gre_header_data
{
	u16 key_bits0to15 /* Tunnel ID bits  0..15. */;
	u16 key_bits16to31 /* Tunnel ID bits 16..31. */;
	u16 bitfields0;
#define GFS_PUSH_TUNNEL_GRE_HEADER_DATA_CHECKSUM_VALID_MASK  0x1 /* GRE checksum exists */
#define GFS_PUSH_TUNNEL_GRE_HEADER_DATA_CHECKSUM_VALID_SHIFT 0
#define GFS_PUSH_TUNNEL_GRE_HEADER_DATA_KEY_VALID_MASK       0x1 /* Key exists */
#define GFS_PUSH_TUNNEL_GRE_HEADER_DATA_KEY_VALID_SHIFT      1
#define GFS_PUSH_TUNNEL_GRE_HEADER_DATA_OOB_KEY_MASK         0x1 /* Use out of band KEY from RX path for hairpin traffic. */
#define GFS_PUSH_TUNNEL_GRE_HEADER_DATA_OOB_KEY_SHIFT        2
#define GFS_PUSH_TUNNEL_GRE_HEADER_DATA_RESERVED_MASK        0x1FFF
#define GFS_PUSH_TUNNEL_GRE_HEADER_DATA_RESERVED_SHIFT       3
};


/*
 * Push tunnel IPV4 header flags
 */
struct gfs_push_tunnel_ipv4_header_flags
{
	u16 bitfields0;
#define GFS_PUSH_TUNNEL_IPV4_HEADER_FLAGS_DONT_FRAG_FLAG_MASK             0x1
#define GFS_PUSH_TUNNEL_IPV4_HEADER_FLAGS_DONT_FRAG_FLAG_SHIFT            0
#define GFS_PUSH_TUNNEL_IPV4_HEADER_FLAGS_FW_TUNN_IPV4_ID_IND_INDEX_MASK  0x1FF /* Modify tunnel destination IP address. */
#define GFS_PUSH_TUNNEL_IPV4_HEADER_FLAGS_FW_TUNN_IPV4_ID_IND_INDEX_SHIFT 1
#define GFS_PUSH_TUNNEL_IPV4_HEADER_FLAGS_RESERVED_MASK                   0x3F
#define GFS_PUSH_TUNNEL_IPV4_HEADER_FLAGS_RESERVED_SHIFT                  10
};


/*
 * Push tunnel VXLAN header data
 */
struct gfs_push_tunnel_vxlan_header_data
{
	u16 vni_bits0to15 /* Tunnel ID bits  0..15 */;
	u8 vni_bits16to23 /* Tunnel ID bits 16..23 */;
	u8 bitfields0;
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_RESERVED0_MASK           0x7
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_RESERVED0_SHIFT          0
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_UDP_CHECKSUM_VALID_MASK  0x1 /* UDP checksum exists */
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_UDP_CHECKSUM_VALID_SHIFT 3
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_I_FRAG_MASK              0x1 /* VNI valid. */
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_I_FRAG_SHIFT             4
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_OOB_VNI_MASK             0x1 /* Use out of band VNI from RX path for hairpin traffic. */
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_OOB_VNI_SHIFT            5
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_RESERVED1_MASK           0x1
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_RESERVED1_SHIFT          6
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_CALC_ENTROPY_MASK        0x1 /* Calculate UDP source port from 4-tuple. */
#define GFS_PUSH_TUNNEL_VXLAN_HEADER_DATA_CALC_ENTROPY_SHIFT       7
	u16 entropy /* VXLAN UDP source value. */;
};


enum gfs_res_struct_dst_type
{
	e_gfs_dst_type_regular /* Perform regular TX-Switching classification  */,
	e_gfs_dst_type_rx_vport /* Redirect to RX Vport */,
	e_gfs_dst_type_tx_sw_bypass /* TX-Switching is bypassed, The TX destination is written in vport field in resolution struct */,
	MAX_GFS_RES_STRUCT_DST_TYPE
};


union gfs_tunnel_header_data
{
	struct gfs_push_tunnel_vxlan_header_data push_tunnel_vxlan_header_data /* Push   VXLAN tunnel header Data */;
	struct gfs_modify_tunnel_vxlan_header_data modify_tunnel_vxlan_header_data /* Modify VXLAN tunnel header Data */;
	struct gfs_push_tunnel_gre_header_data push_tunnel_gre_header_data /* Push   GRE   tunnel header Data */;
	struct gfs_modify_tunnel_gre_header_data modify_tunnel_gre_header_data /* Modify GRE   tunnel header Data */;
};


union gfs_tunnel_header_flags
{
	struct gfs_modify_tunnel_ipv4_header_flags modify_tunnel_ipv4_header_flags /* Modify tunnel IPV4 header Flags */;
	struct gfs_push_tunnel_ipv4_header_flags push_tunnel_ipv4_header_flags /* Push tunnel IPV4 header Flags */;
};


/*
 * GFS Context VLAN mode enum
 */
enum gfs_vlan_mode
{
	e_vlan_mode_nop,
	e_vlan_mode_pop,
	e_vlan_mode_push,
	e_vlan_mode_change_vid,
	e_vlan_mode_change_pri,
	e_vlan_mode_change_whole_tag,
	MAX_GFS_VLAN_MODE
};


/*
 * RGFS context Data QREG 1
 */
struct rgfs_context_data_qreg1
{
	u32 cid /* CID for replacement in CM header. Used for steering to TX queue. Only one copy can use CID. In TGFS used to change event ID   Select TGFS flow in PSTORM */;
	u16 bitfields0;
#define RGFS_CONTEXT_DATA_QREG1_CONNECTION_TYPE_MASK          0xF /* Connection Type for replacement in the CM header (Unused) */
#define RGFS_CONTEXT_DATA_QREG1_CONNECTION_TYPE_SHIFT         0
#define RGFS_CONTEXT_DATA_QREG1_CORE_AFFINITY_MASK            0x3 /* Core Affinity for replacement in the CM header (Unused) */
#define RGFS_CONTEXT_DATA_QREG1_CORE_AFFINITY_SHIFT           4
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_DST_IP_ADDRESS_MASK    0x1 /* Change dst IP address that RSS header refers to. Set 1 to replace inner IPV4 address */
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_DST_IP_ADDRESS_SHIFT   6
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_SRC_IP_ADDRESS_MASK    0x1 /* Change src IP address that RSS header refers to. Set 1 to replace inner IPV4 address */
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_SRC_IP_ADDRESS_SHIFT   7
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_DST_PORT_MASK          0x1 /* Change dst port that RSS header refers to. Set 1 to replace inner UDP/TCP port */
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_DST_PORT_SHIFT         8
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_SRC_PORT_MASK          0x1 /* Change src port that RSS header refers to. Set 1 to replace inner UDP/TCP port */
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_SRC_PORT_SHIFT         9
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_RSS_IPVER_TYPE_MASK    0x1 /* Change IpVerType field in the RSS header. Set 0 */
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_RSS_IPVER_TYPE_SHIFT   10
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_RSS_L4_TYPE_MASK       0x1 /* Change L4Type field in the RSS header. Set 0 */
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_RSS_L4_TYPE_SHIFT      11
#define RGFS_CONTEXT_DATA_QREG1_RSS_L4_TYPE_MASK              0x3 /* Unused */
#define RGFS_CONTEXT_DATA_QREG1_RSS_L4_TYPE_SHIFT             12
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_RSS_OVER_IP_PROT_MASK  0x1 /* Change OVER_IP_PROTOCOL field in the RSS header. Set 0 */
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_RSS_OVER_IP_PROT_SHIFT 14
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_RSS_IPV4_FRAG_MASK     0x1 /* Change Ipv4Frag field in the RSS header. Set 0 */
#define RGFS_CONTEXT_DATA_QREG1_CHANGE_RSS_IPV4_FRAG_SHIFT    15
	u16 dst_port /* Dst port value for replacement. L4 port */;
	u16 src_port /* Src port value for replacement. L4 port */;
	u16 bitfields1;
#define RGFS_CONTEXT_DATA_QREG1_RSS_IPVER_TYPE_MASK           0x3 /* Unused */
#define RGFS_CONTEXT_DATA_QREG1_RSS_IPVER_TYPE_SHIFT          0
#define RGFS_CONTEXT_DATA_QREG1_RSS_IPV4_FRAG_MASK            0x1 /* Unused */
#define RGFS_CONTEXT_DATA_QREG1_RSS_IPV4_FRAG_SHIFT           2
#define RGFS_CONTEXT_DATA_QREG1_RSS_OVER_IP_PROT_MASK         0xFF /* Unused */
#define RGFS_CONTEXT_DATA_QREG1_RSS_OVER_IP_PROT_SHIFT        3
#define RGFS_CONTEXT_DATA_QREG1_RESERVED_MASK                 0x1F
#define RGFS_CONTEXT_DATA_QREG1_RESERVED_SHIFT                11
	u32 flow_id /* Flow Id. Used for TMLD aggregation. Always set. Replace last register in TM message. */;
};


/*
 * RGFS context Data QREG 2
 */
struct rgfs_context_data_qreg2
{
	u32 src_ip /* HDR Modify   IPv4 source IP or IPv6 indirection index. 0 are invalid index. */;
	u16 bitfields0;
#define RGFS_CONTEXT_DATA_QREG2_FW_POP_TYPE_MASK                                           0x7
#define RGFS_CONTEXT_DATA_QREG2_FW_POP_TYPE_SHIFT                                          0
#define RGFS_CONTEXT_DATA_QREG2_FW_TID0_VALID_MASK                                         0x1 /* TID 0 Valid */
#define RGFS_CONTEXT_DATA_QREG2_FW_TID0_VALID_SHIFT                                        3
#define RGFS_CONTEXT_DATA_QREG2_FW_TID1_VALID_MASK                                         0x1 /* TID 1 Valid */
#define RGFS_CONTEXT_DATA_QREG2_FW_TID1_VALID_SHIFT                                        4
#define RGFS_CONTEXT_DATA_QREG2_FW_FIRST_HDR_VLAN_MODE_MASK                                0x3 /* POP or PUSH CVLAN out of band (use enum gfs_vlan_mode) */
#define RGFS_CONTEXT_DATA_QREG2_FW_FIRST_HDR_VLAN_MODE_SHIFT                               5
#define RGFS_CONTEXT_DATA_QREG2_RESERVED_DRV_HINT_TBD_WAS_FW_INNER_IPV4_ID_IND_INDEX_MASK  0x1FF /* Drv Hint (TBD). Was: Inner IPV4 ID indirection index. 0 are invalid index. */
#define RGFS_CONTEXT_DATA_QREG2_RESERVED_DRV_HINT_TBD_WAS_FW_INNER_IPV4_ID_IND_INDEX_SHIFT 7
	u16 fw_command1_sample_len /* 2nd Command sample length (1st command is command0) */;
	u32 fw_tid0 /* Counter 0 TID */;
	u32 fw_tid1 /* Counter 1 TID */;
};


/*
 * RGFS context Data QREG 3
 */
struct rgfs_context_data_qreg3
{
	u32 dst_ip /* HDR Modify   inner IPv4 destination IP or IPv6 indirection index. 0 are invalid index. */;
	u16 fw_first_hdr_vlan_val /* Tunnel VLAN value */;
	u16 bitfields0;
#define RGFS_CONTEXT_DATA_QREG3_XXLOCK_CMD_MASK                     0x7 /* CM HDR override - Clear for drop. No special allocation needed for drop fields. */
#define RGFS_CONTEXT_DATA_QREG3_XXLOCK_CMD_SHIFT                    0
#define RGFS_CONTEXT_DATA_QREG3_FW_MODIFY_INNER_4TUPLE_MASK         0x1 /* Modify 4 tuple  */
#define RGFS_CONTEXT_DATA_QREG3_FW_MODIFY_INNER_4TUPLE_SHIFT        3
#define RGFS_CONTEXT_DATA_QREG3_FW_MODIFY_INNER_MAC_MASK            0x1 /* Modify inner/single ETH header */
#define RGFS_CONTEXT_DATA_QREG3_FW_MODIFY_INNER_MAC_SHIFT           4
#define RGFS_CONTEXT_DATA_QREG3_FW_MODIFY_INNER_IP_DSCP_MASK        0x1 /* Modify inner IP DSCP */
#define RGFS_CONTEXT_DATA_QREG3_FW_MODIFY_INNER_IP_DSCP_SHIFT       5
#define RGFS_CONTEXT_DATA_QREG3_FW_SET_INNER_IP_TTL_HOP_LIMIT_MASK  0x1 /* Modify inner IP TTL or Hop Limit */
#define RGFS_CONTEXT_DATA_QREG3_FW_SET_INNER_IP_TTL_HOP_LIMIT_SHIFT 6
#define RGFS_CONTEXT_DATA_QREG3_FW_DEC_INNER_IP_TTL_HOP_LIMIT_MASK  0x1 /* Decrement inner IP TTL or Hop Limit */
#define RGFS_CONTEXT_DATA_QREG3_FW_DEC_INNER_IP_TTL_HOP_LIMIT_SHIFT 7
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_VLAN_MODIFY_FLG_MASK       0x3 /* NOP, change VID, change PRI, change whole tag */
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_VLAN_MODIFY_FLG_SHIFT      8
#define RGFS_CONTEXT_DATA_QREG3_LOAD_ST_CTX_FLG_MASK                0x1 /* CM HDR override - Clear for drop. No special allocation needed for drop fields. */
#define RGFS_CONTEXT_DATA_QREG3_LOAD_ST_CTX_FLG_SHIFT               10
#define RGFS_CONTEXT_DATA_QREG3_FW_TUNNEL_VLAN_MODIFY_FLG_MASK      0x7 /* NOP, POP, PUSH, change VID, change PRI, change whole tag */
#define RGFS_CONTEXT_DATA_QREG3_FW_TUNNEL_VLAN_MODIFY_FLG_SHIFT     11
#define RGFS_CONTEXT_DATA_QREG3_RESERVED_MASK                       0x3
#define RGFS_CONTEXT_DATA_QREG3_RESERVED_SHIFT                      14
	u16 fw_command2_sample_len /* 3rd Command sample length (1st command is command0) */;
	u16 fw_inner_vlan_val /* Inner VLAN value */;
	u32 bitfields1;
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_IP_DSCP_MASK               0x3F /* inner IP DSCP Set value */
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_IP_DSCP_SHIFT              0
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_SRC_MAC_IND_INDEX_MASK     0x1FF /* Inner source      MAC indirection index. 0 are invalid index. */
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_SRC_MAC_IND_INDEX_SHIFT    6
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_DEST_MAC_IND_INDEX_MASK    0x1FF /* Inner destination MAC indirection index. 0 are invalid index. */
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_DEST_MAC_IND_INDEX_SHIFT   15
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_IP_TTL_HOP_LIMIT_MASK      0xFF /* inner IP TTL or Hop Limit Set value */
#define RGFS_CONTEXT_DATA_QREG3_FW_INNER_IP_TTL_HOP_LIMIT_SHIFT     24
};


/*
 * RGFS context Data QREG 4
 */
struct rgfs_context_data_qreg4
{
	u32 fw_tunn_src_ip /* HDR Modify   tunnel IPv4 source IP or IPv6 indirection index. Used by FW. */;
	u32 flow_mark /* Filter marker. Always set. Replace per-pf config in basic block. Reported by CQE  */;
	u32 fw_tunn_dest_ip /* HDR Modify   tunnel IPv4 destination IP or IPv6 indirection index. Used by FW. */;
	u8 fw_tunn_ip_ttl_hop_limit /* Tunnel IP TTL or Hop Limit Set value */;
	u8 reserved[3];
};


/*
 * RGFS context Data QREG 5
 */
struct rgfs_context_data_qreg5
{
	union gfs_tunnel_header_flags tunnel_ip_header_flags /* Tunnel IP tunnel header flag union. */;
	u16 bitfields0;
#define RGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_SRC_MAC_IND_INDEX_MASK   0x1FF /* Tunnel source MAC indirection index. 0 are invalid index. */
#define RGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_SRC_MAC_IND_INDEX_SHIFT  0
#define RGFS_CONTEXT_DATA_QREG5_FW_PUSH_TYPE_MASK                  0xF /* Push headers type. */
#define RGFS_CONTEXT_DATA_QREG5_FW_PUSH_TYPE_SHIFT                 9
#define RGFS_CONTEXT_DATA_QREG5_CID_OVERWRITE_MASK                 0x1 /* RSS HDR override - for CID steering. Disable CID override. */
#define RGFS_CONTEXT_DATA_QREG5_CID_OVERWRITE_SHIFT                13
#define RGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_IP_ECN_MASK              0x3 /* Tunnel IP header flags for PUSH flow: 2 LSBits of tos field. */
#define RGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_IP_ECN_SHIFT             14
	union gfs_tunnel_header_data tunnel_header_data /* VXLAN/GRE tunnel header union. */;
	u16 bitfields1;
#define RGFS_CONTEXT_DATA_QREG5_RESERVED2_MASK                     0x7F /* Modify tunnel source IP address. */
#define RGFS_CONTEXT_DATA_QREG5_RESERVED2_SHIFT                    0
#define RGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_DEST_MAC_IND_INDEX_MASK  0x1FF /* Tunnel destination MAC indirection index. 0 are invalid index. */
#define RGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_DEST_MAC_IND_INDEX_SHIFT 7
	u32 bitfields2;
#define RGFS_CONTEXT_DATA_QREG5_KEY_ADDR_LDR_HEADER_REGION_MASK    0xFFF /* RSS HDR override - for queue/CID steering. Only one copy can use queue ID. Limited by 256. Bits 9-11 can be used by context.   */
#define RGFS_CONTEXT_DATA_QREG5_KEY_ADDR_LDR_HEADER_REGION_SHIFT   0
#define RGFS_CONTEXT_DATA_QREG5_CALC_RSS_AND_IND_TBL_MASK          0xF /* RSS HDR override - For steering. Only one copy can use queue ID. Limited by 1. Bits 1-3 can be used by context. * Cannot Move *  */
#define RGFS_CONTEXT_DATA_QREG5_CALC_RSS_AND_IND_TBL_SHIFT         12
#define RGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_IP_DSCP_MASK             0x3F /* Tunnel IP DSCP Set value */
#define RGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_IP_DSCP_SHIFT            16
#define RGFS_CONTEXT_DATA_QREG5_DEFAULT_QUEUE_ID_MASK              0x3FF /* RSS HDR override - For steering. Only one copy can use queue ID. Limited by 511. Bits 9 can be used by context. * Cannot Move *  */
#define RGFS_CONTEXT_DATA_QREG5_DEFAULT_QUEUE_ID_SHIFT             22
};


/*
 * TGFS context Data QREG 1
 */
struct tgfs_context_data_qreg1
{
	u16 fw_first_hdr_vlan_val /* Tunnel or single VLAN value */;
	u16 bitfields0;
#define TGFS_CONTEXT_DATA_QREG1_FW_FIRST_HDR_VLAN_MODE_MASK  0x7 /* First ETH header VLAN modification: NOP, POP, PUSH, change VID, change PRI, change whole tag. Replace basic block register 8 (use enum gfs_vlan_mode) */
#define TGFS_CONTEXT_DATA_QREG1_FW_FIRST_HDR_VLAN_MODE_SHIFT 0
#define TGFS_CONTEXT_DATA_QREG1_FW_TID0_VALID_MASK           0x1 /* TID 0 Valid */
#define TGFS_CONTEXT_DATA_QREG1_FW_TID0_VALID_SHIFT          3
#define TGFS_CONTEXT_DATA_QREG1_FW_TID1_VALID_MASK           0x1 /* TID 1 Valid */
#define TGFS_CONTEXT_DATA_QREG1_FW_TID1_VALID_SHIFT          4
#define TGFS_CONTEXT_DATA_QREG1_RESERVED_MASK                0x7FF
#define TGFS_CONTEXT_DATA_QREG1_RESERVED_SHIFT               5
	u16 fw_command1_sample_len /* Command 2 sample length */;
	u16 fw_command2_sample_len /* Command 3 sample length */;
	u32 fw_tid0 /* Counter TID. */;
	u32 fw_tid1 /* Counter TID. */;
};


/*
 * TGFS context Data QREG 2
 */
struct tgfs_context_data_qreg2
{
	u32 src_ip /* Inner IPv4 source IP or IPv6 indirection index. */;
	u32 dst_ip /* Inner IPv4 destination IP or IPv6 indirection index. */;
	u16 dst_port /* L4 destination port. */;
	u16 src_port /* L4 source port. */;
	u32 bitfields;
#define TGFS_CONTEXT_DATA_QREG2_FW_INNER_SRC_MAC_IND_INDEX_MASK     0x1FF /* Inner source MAC indirection index. 0 are invalid index. */
#define TGFS_CONTEXT_DATA_QREG2_FW_INNER_SRC_MAC_IND_INDEX_SHIFT    0
#define TGFS_CONTEXT_DATA_QREG2_FW_INNER_DEST_MAC_IND_INDEX_MASK    0x1FF /* Inner destination MAC indirection index. 0 are invalid index. */
#define TGFS_CONTEXT_DATA_QREG2_FW_INNER_DEST_MAC_IND_INDEX_SHIFT   9
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_SRC_IP_ADDR_MASK    0x1 /* Modify inner/single source IP address. */
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_SRC_IP_ADDR_SHIFT   18
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_DEST_IP_ADDR_MASK   0x1 /* Modify inner/single destination IP address. */
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_DEST_IP_ADDR_SHIFT  19
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_L4_SRC_PORT_MASK    0x1 /* Modify inner/single L4 header source port */
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_L4_SRC_PORT_SHIFT   20
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_L4_DEST_PORT_MASK   0x1 /* Modify inner/single L4 header destination port */
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_L4_DEST_PORT_SHIFT  21
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_MAC_MASK            0x1 /* Modify MACs in inner ETH header */
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_MAC_SHIFT           22
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_IP_DSCP_MASK        0x1 /* Modify inner IP DSCP */
#define TGFS_CONTEXT_DATA_QREG2_FW_MODIFY_INNER_IP_DSCP_SHIFT       23
#define TGFS_CONTEXT_DATA_QREG2_FW_SET_INNER_IP_TTL_HOP_LIMIT_MASK  0x1 /* Modify inner IP TTL or Hop Limit */
#define TGFS_CONTEXT_DATA_QREG2_FW_SET_INNER_IP_TTL_HOP_LIMIT_SHIFT 24
#define TGFS_CONTEXT_DATA_QREG2_FW_DEC_INNER_IP_TTL_HOP_LIMIT_MASK  0x1 /* Decrement inner IP TTL or Hop Limit */
#define TGFS_CONTEXT_DATA_QREG2_FW_DEC_INNER_IP_TTL_HOP_LIMIT_SHIFT 25
#define TGFS_CONTEXT_DATA_QREG2_RESERVED_MASK                       0x3
#define TGFS_CONTEXT_DATA_QREG2_RESERVED_SHIFT                      26
#define TGFS_CONTEXT_DATA_QREG2_FW_SET_CHECKSUM_OFFLOAD_MASK        0xF /* Set checksum offload due to modify fields. FP must verify if checksum exist. */
#define TGFS_CONTEXT_DATA_QREG2_FW_SET_CHECKSUM_OFFLOAD_SHIFT       28
};


/*
 * TGFS context Data QREG 3
 */
struct tgfs_context_data_qreg3
{
	u32 bitfields0;
#define TGFS_CONTEXT_DATA_QREG3_FW_FIRST_HDR_DEST_MAC_IND_INDEX_MASK  0x1FF /* First ETH header destination MAC indirection index. 0 for unchanged. Replace basic block register 8. */
#define TGFS_CONTEXT_DATA_QREG3_FW_FIRST_HDR_DEST_MAC_IND_INDEX_SHIFT 0
#define TGFS_CONTEXT_DATA_QREG3_FW_FIRST_HDR_SRC_MAC_IND_INDEX_MASK   0x1FF /* First ETH header source MAC indirection index. 0 for unchanged. Replace basic block register 8 */
#define TGFS_CONTEXT_DATA_QREG3_FW_FIRST_HDR_SRC_MAC_IND_INDEX_SHIFT  9
#define TGFS_CONTEXT_DATA_QREG3_FW_POP_TYPE_MASK                      0x7 /* Reserved for last bit in searcher database */
#define TGFS_CONTEXT_DATA_QREG3_FW_POP_TYPE_SHIFT                     18
#define TGFS_CONTEXT_DATA_QREG3_FW_INNER_HDR_VLAN_MODE_MASK           0x7 /* Inner ETH header VLAN modification: NOP, POP, PUSH, change VID, change PRI, change whole tag. Replace basic block register 8 (use enum gfs_vlan_mode) */
#define TGFS_CONTEXT_DATA_QREG3_FW_INNER_HDR_VLAN_MODE_SHIFT          21
#define TGFS_CONTEXT_DATA_QREG3_FW_INNER_IP_TTL_HOP_LIMIT_MASK        0xFF /* inner IP TTL or Hop Limit Set value. Replace basic block register 8. */
#define TGFS_CONTEXT_DATA_QREG3_FW_INNER_IP_TTL_HOP_LIMIT_SHIFT       24
	u16 bitfields1;
#define TGFS_CONTEXT_DATA_QREG3_FW_PUSH_TYPE_MASK                     0xF /* Push headers type. */
#define TGFS_CONTEXT_DATA_QREG3_FW_PUSH_TYPE_SHIFT                    0
#define TGFS_CONTEXT_DATA_QREG3_FW_INNER_IP_DSCP_MASK                 0x3F /* inner IP DSCP Set value */
#define TGFS_CONTEXT_DATA_QREG3_FW_INNER_IP_DSCP_SHIFT                4
#define TGFS_CONTEXT_DATA_QREG3_RESERVED0_MASK                        0x3F
#define TGFS_CONTEXT_DATA_QREG3_RESERVED0_SHIFT                       10
	u16 fw_inner_vlan_val /* Inner VLAN value */;
	u32 flow_mark /* Filter marker. Pass to Rx if destination is RX_VPORT */;
	u32 reserved1;
};


/*
 * TGFS context Data QREG 4
 */
struct tgfs_context_data_qreg4
{
	u32 flow_id /* Flow Id. Replace basic block register 8. */;
	u32 reserved;
	u32 fw_tunn_dest_ip /* HDR Modify   tunnel IPv4 destination IP or IPv6 indirection index. */;
	u32 fw_tunn_src_ip /* HDR Modify   tunnel IPv4 source IP or IPv6 indirection index. */;
};


/*
 * TGFS context Data QREG 5
 */
struct tgfs_context_data_qreg5
{
	u8 bitfields0;
#define TGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_IP_ECN_MASK      0x3 /* Tunnel IP header flags for PUSH flow: 2 LSBits of tos field. */
#define TGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_IP_ECN_SHIFT     0
#define TGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_IP_DSCP_MASK     0x3F /* Tunnel IP DSCP Set value */
#define TGFS_CONTEXT_DATA_QREG5_FW_TUNNEL_IP_DSCP_SHIFT    2
	u8 fw_tunn_ip_ttl_hop_limit /* Tunnel IP TTL or Hop Limit Set value */;
	u8 event_id /* This field is used to replace part of event ID in MCM header. */;
	u8 bitfields1;
#define TGFS_CONTEXT_DATA_QREG5_FW_ORIGINAL_COPY_FLG_MASK  0x1 /* Original flag set for some copy. */
#define TGFS_CONTEXT_DATA_QREG5_FW_ORIGINAL_COPY_FLG_SHIFT 0
#define TGFS_CONTEXT_DATA_QREG5_RSERVED0_MASK              0x7F /* Original flag set for some copy. */
#define TGFS_CONTEXT_DATA_QREG5_RSERVED0_SHIFT             1
	union gfs_tunnel_header_data tunnel_header_data /* VXLAN/GRE tunnel header union. */;
	union gfs_tunnel_header_flags tunnel_ip_header_flags /* Tunnel IP tunnel header flag union. */;
	u32 reserved2;
};


/*
 * BRB RAM init requirements
 */
struct init_brb_ram_req
{
	u32 guranteed_per_tc /* guaranteed size per TC, in bytes */;
	u32 headroom_per_tc /* headroom size per TC, in bytes */;
	u32 min_pkt_size /* min packet size, in bytes */;
	u32 max_ports_per_engine /* min packet size, in bytes */;
	u8 num_active_tcs[MAX_NUM_PORTS] /* number of active TCs per port */;
};


/*
 * ETS per-TC init requirements
 */
struct init_ets_tc_req
{
	u8 use_sp /* if set, this TC participates in the arbitration with a strict priority (the priority is equal to the TC ID) */;
	u8 use_wfq /* if set, this TC participates in the arbitration with a WFQ weight (indicated by the weight field) */;
	u16 weight /* An arbitration weight. Valid only if use_wfq is set. */;
};

/*
 * ETS init requirements
 */
struct init_ets_req
{
	u32 mtu /* Max packet size (in bytes) */;
	struct init_ets_tc_req tc_req[NUM_OF_TCS] /* ETS initialization requirements per TC. */;
};



/*
 * NIG LB RL init requirements
 */
struct init_nig_lb_rl_req
{
	u16 lb_mac_rate /* Global MAC+LB RL rate (in Mbps). If set to 0, the RL will be disabled. */;
	u16 lb_rate /* Global LB RL rate (in Mbps). If set to 0, the RL will be disabled. */;
	u32 mtu /* Max packet size (in bytes) */;
	u16 tc_rate[NUM_OF_PHYS_TCS] /* RL rate per physical TC (in Mbps). If set to 0, the RL will be disabled. */;
};


/*
 * NIG TC mapping for each priority
 */
struct init_nig_pri_tc_map_entry
{
	u8 tc_id /* the mapped TC ID */;
	u8 valid /* indicates if the mapping entry is valid */;
};


/*
 * NIG priority to TC map init requirements
 */
struct init_nig_pri_tc_map_req
{
	struct init_nig_pri_tc_map_entry pri[NUM_OF_VLAN_PRIORITIES];
};


/*
 * QM per global RL init parameters
 */
struct init_qm_global_rl_params
{
	u32 rate_limit /* Rate limit in Mb/sec units. If set to zero, the link speed is uwsed instead. */;
};


/*
 * QM per port init parameters
 */
struct init_qm_port_params
{
	u16 active_phys_tcs /* Vector of valid bits for active TCs used by this port */;
	u16 num_pbf_cmd_lines /* Number of PBF command lines that can be used by this port. In E4 each line is 256b, and in E5 each line is 512b. */;
	u16 num_btb_blocks /* Number of BTB blocks that can be used by this port */;
	u8 active /* Indicates if this port is active */;
	u8 reserved;
};


/*
 * QM per PQ init parameters
 */
struct init_qm_pq_params
{
	u8 vport_id /* VPORT ID */;
	u8 tc_id /* TC ID */;
	u8 wrr_group /* WRR group */;
	u8 rl_valid /* Indicates if a rate limiter should be allocated for the PQ (0/1) */;
	u16 rl_id /* RL ID, valid only if rl_valid is true */;
	u8 port_id /* Port ID */;
	u8 reserved;
};


/*
 * QM per VPORT init parameters
 */
struct init_qm_vport_params
{
	u16 wfq /* WFQ weight. A value of 0 means dont configure. ignored if VPORT WFQ is globally disabled. */;
	u16 first_tx_pq_id[NUM_OF_TCS] /* the first Tx PQ ID associated with this VPORT for each TC. */;
};

#endif /* __ECORE_HSI_INIT_FUNC__ */
