#ifndef __ECORE_HSI_RDMA__
#define __ECORE_HSI_RDMA__ 
/************************************************************************/
/* Add include to common rdma target for both eCore and protocol rdma driver */
/************************************************************************/
#include "rdma_common.h"

/*
 * The roce task context of Mstorm
 */
struct mstorm_rdma_task_st_ctx
{
	struct regpair temp[4];
};


/*
 * rdma function init ramrod data
 */
struct rdma_close_func_ramrod_data
{
	u8 cnq_start_offset;
	u8 num_cnqs;
	u8 vf_id /* This field should be assigned to Virtual Function ID if vf_valid == 1. Otherwise its dont care */;
	u8 vf_valid;
	u8 reserved[4];
};


/*
 * rdma function init CNQ parameters
 */
struct rdma_cnq_params
{
	__le16 sb_num /* Status block number used by the queue */;
	u8 sb_index /* Status block index used by the queue */;
	u8 num_pbl_pages /* Number of pages in the PBL allocated for this queue */;
	__le32 reserved;
	struct regpair pbl_base_addr /* Address to the first entry of the queue PBL */;
	__le16 queue_zone_num /* Queue Zone ID used for CNQ consumer update */;
	u8 reserved1[6];
};


/*
 * rdma create cq ramrod data
 */
struct rdma_create_cq_ramrod_data
{
	struct regpair cq_handle;
	struct regpair pbl_addr;
	__le32 max_cqes;
	__le16 pbl_num_pages;
	__le16 dpi;
	u8 is_two_level_pbl;
	u8 cnq_id;
	u8 pbl_log_page_size;
	u8 toggle_bit;
	__le16 int_timeout /* Timeout used for interrupt moderation */;
	u8 vf_id /* Only used in E5 */;
	u8 flags;
#define RDMA_CREATE_CQ_RAMROD_DATA_VF_ID_VALID_MASK  0x1 /* Bit indicating that this VF */
#define RDMA_CREATE_CQ_RAMROD_DATA_VF_ID_VALID_SHIFT 0
#define RDMA_CREATE_CQ_RAMROD_DATA_RESERVED1_MASK    0x7F
#define RDMA_CREATE_CQ_RAMROD_DATA_RESERVED1_SHIFT   1
};


/*
 * rdma deregister tid ramrod data
 */
struct rdma_deregister_tid_ramrod_data
{
	__le32 itid;
	__le32 reserved;
};


/*
 * rdma destroy cq output params
 */
struct rdma_destroy_cq_output_params
{
	__le16 cnq_num /* Sequence number of completion notification sent for the cq on the associated CNQ */;
	__le16 reserved0;
	__le32 reserved1;
};


/*
 * rdma destroy cq ramrod data
 */
struct rdma_destroy_cq_ramrod_data
{
	struct regpair output_params_addr;
};


/*
 * RDMA slow path EQ cmd IDs
 */
enum rdma_event_opcode
{
	RDMA_EVENT_UNUSED,
	RDMA_EVENT_FUNC_INIT,
	RDMA_EVENT_FUNC_CLOSE,
	RDMA_EVENT_REGISTER_MR,
	RDMA_EVENT_DEREGISTER_MR,
	RDMA_EVENT_CREATE_CQ,
	RDMA_EVENT_RESIZE_CQ,
	RDMA_EVENT_DESTROY_CQ,
	RDMA_EVENT_CREATE_SRQ,
	RDMA_EVENT_MODIFY_SRQ,
	RDMA_EVENT_DESTROY_SRQ,
	MAX_RDMA_EVENT_OPCODE
};


/*
 * RDMA FW return code for slow path ramrods
 */
enum rdma_fw_return_code
{
	RDMA_RETURN_OK=0,
	RDMA_RETURN_REGISTER_MR_BAD_STATE_ERR,
	RDMA_RETURN_DEREGISTER_MR_BAD_STATE_ERR,
	RDMA_RETURN_RESIZE_CQ_ERR,
	RDMA_RETURN_NIG_DRAIN_REQ,
	RDMA_RETURN_GENERAL_ERR,
	MAX_RDMA_FW_RETURN_CODE
};


/*
 * rdma function init header
 */
struct rdma_init_func_hdr
{
	u8 cnq_start_offset /* First RDMA CNQ */;
	u8 num_cnqs /* Number of CNQs */;
	u8 cq_ring_mode /* 0 for 32 bit cq producer and consumer counters and 1 for 16 bit */;
	u8 vf_id /* This field should be assigned to Virtual Function ID if vf_valid == 1. Otherwise its dont care */;
	u8 vf_valid;
	u8 relaxed_ordering /* 1 for using relaxed ordering PCI writes */;
	__le16 first_reg_srq_id /* The SRQ ID of thr first regular (non XRC) SRQ */;
	__le32 reg_srq_base_addr /* Logical base address of first regular (non XRC) SRQ */;
	u8 searcher_mode /* RoCE PF Init only: 1 for using the Searcher, 0 otherwise. boolean can be changed to bitfield  */;
	u8 pvrdma_mode /* RoCE PF Init only: 1 for enabling PVRDMA, 0 otherwise. boolean can be changed to bitfield  */;
	u8 max_num_ns_log /* ROCE PVRDMA mode only: log2 of maximum number of namespaces */;
	u8 reserved;
};


/*
 * rdma function init ramrod data
 */
struct rdma_init_func_ramrod_data
{
	struct rdma_init_func_hdr params_header;
	struct rdma_cnq_params cnq_params[NUM_OF_GLOBAL_QUEUES];
};


/*
 * RDMA ramrod command IDs
 */
enum rdma_ramrod_cmd_id
{
	RDMA_RAMROD_UNUSED,
	RDMA_RAMROD_FUNC_INIT,
	RDMA_RAMROD_FUNC_CLOSE,
	RDMA_RAMROD_REGISTER_MR,
	RDMA_RAMROD_DEREGISTER_MR,
	RDMA_RAMROD_CREATE_CQ,
	RDMA_RAMROD_RESIZE_CQ,
	RDMA_RAMROD_DESTROY_CQ,
	RDMA_RAMROD_CREATE_SRQ,
	RDMA_RAMROD_MODIFY_SRQ,
	RDMA_RAMROD_DESTROY_SRQ,
	MAX_RDMA_RAMROD_CMD_ID
};


/*
 * rdma register tid ramrod data
 */
struct rdma_register_tid_ramrod_data
{
	__le16 flags;
#define RDMA_REGISTER_TID_RAMROD_DATA_PAGE_SIZE_LOG_MASK      0x1F
#define RDMA_REGISTER_TID_RAMROD_DATA_PAGE_SIZE_LOG_SHIFT     0
#define RDMA_REGISTER_TID_RAMROD_DATA_TWO_LEVEL_PBL_MASK      0x1
#define RDMA_REGISTER_TID_RAMROD_DATA_TWO_LEVEL_PBL_SHIFT     5
#define RDMA_REGISTER_TID_RAMROD_DATA_ZERO_BASED_MASK         0x1
#define RDMA_REGISTER_TID_RAMROD_DATA_ZERO_BASED_SHIFT        6
#define RDMA_REGISTER_TID_RAMROD_DATA_PHY_MR_MASK             0x1
#define RDMA_REGISTER_TID_RAMROD_DATA_PHY_MR_SHIFT            7
#define RDMA_REGISTER_TID_RAMROD_DATA_REMOTE_READ_MASK        0x1
#define RDMA_REGISTER_TID_RAMROD_DATA_REMOTE_READ_SHIFT       8
#define RDMA_REGISTER_TID_RAMROD_DATA_REMOTE_WRITE_MASK       0x1
#define RDMA_REGISTER_TID_RAMROD_DATA_REMOTE_WRITE_SHIFT      9
#define RDMA_REGISTER_TID_RAMROD_DATA_REMOTE_ATOMIC_MASK      0x1
#define RDMA_REGISTER_TID_RAMROD_DATA_REMOTE_ATOMIC_SHIFT     10
#define RDMA_REGISTER_TID_RAMROD_DATA_LOCAL_WRITE_MASK        0x1
#define RDMA_REGISTER_TID_RAMROD_DATA_LOCAL_WRITE_SHIFT       11
#define RDMA_REGISTER_TID_RAMROD_DATA_LOCAL_READ_MASK         0x1
#define RDMA_REGISTER_TID_RAMROD_DATA_LOCAL_READ_SHIFT        12
#define RDMA_REGISTER_TID_RAMROD_DATA_ENABLE_MW_BIND_MASK     0x1
#define RDMA_REGISTER_TID_RAMROD_DATA_ENABLE_MW_BIND_SHIFT    13
#define RDMA_REGISTER_TID_RAMROD_DATA_RESERVED_MASK           0x3
#define RDMA_REGISTER_TID_RAMROD_DATA_RESERVED_SHIFT          14
	u8 flags1;
#define RDMA_REGISTER_TID_RAMROD_DATA_PBL_PAGE_SIZE_LOG_MASK  0x1F
#define RDMA_REGISTER_TID_RAMROD_DATA_PBL_PAGE_SIZE_LOG_SHIFT 0
#define RDMA_REGISTER_TID_RAMROD_DATA_TID_TYPE_MASK           0x7
#define RDMA_REGISTER_TID_RAMROD_DATA_TID_TYPE_SHIFT          5
	u8 flags2;
#define RDMA_REGISTER_TID_RAMROD_DATA_DMA_MR_MASK             0x1 /* Bit indicating that this MR is DMA_MR meaning SGEs that use it have the physical address on them */
#define RDMA_REGISTER_TID_RAMROD_DATA_DMA_MR_SHIFT            0
#define RDMA_REGISTER_TID_RAMROD_DATA_DIF_ON_HOST_FLG_MASK    0x1 /* Bit indicating that this MR has DIF protection enabled. */
#define RDMA_REGISTER_TID_RAMROD_DATA_DIF_ON_HOST_FLG_SHIFT   1
#define RDMA_REGISTER_TID_RAMROD_DATA_RESERVED1_MASK          0x3F
#define RDMA_REGISTER_TID_RAMROD_DATA_RESERVED1_SHIFT         2
	u8 key;
	u8 length_hi;
	u8 vf_id /* This field should be assigned to Virtual Function ID if vf_valid == 1. Otherwise its dont care */;
	u8 vf_valid;
	__le16 pd;
	__le16 reserved2;
	__le32 length_lo /* lower 32 bits of the registered MR length. */;
	__le32 itid;
	__le32 reserved3;
	struct regpair va;
	struct regpair pbl_base;
	struct regpair dif_error_addr /* DIF TX IO writes error information to this location when memory region is invalidated. */;
	__le32 reserved4[4];
};


/*
 * rdma resize cq output params
 */
struct rdma_resize_cq_output_params
{
	__le32 old_cq_cons /* cq consumer value on old PBL */;
	__le32 old_cq_prod /* cq producer value on old PBL */;
};


/*
 * rdma resize cq ramrod data
 */
struct rdma_resize_cq_ramrod_data
{
	u8 flags;
#define RDMA_RESIZE_CQ_RAMROD_DATA_TOGGLE_BIT_MASK        0x1
#define RDMA_RESIZE_CQ_RAMROD_DATA_TOGGLE_BIT_SHIFT       0
#define RDMA_RESIZE_CQ_RAMROD_DATA_IS_TWO_LEVEL_PBL_MASK  0x1
#define RDMA_RESIZE_CQ_RAMROD_DATA_IS_TWO_LEVEL_PBL_SHIFT 1
#define RDMA_RESIZE_CQ_RAMROD_DATA_VF_ID_VALID_MASK       0x1
#define RDMA_RESIZE_CQ_RAMROD_DATA_VF_ID_VALID_SHIFT      2
#define RDMA_RESIZE_CQ_RAMROD_DATA_RESERVED_MASK          0x1F
#define RDMA_RESIZE_CQ_RAMROD_DATA_RESERVED_SHIFT         3
	u8 pbl_log_page_size;
	__le16 pbl_num_pages;
	__le32 max_cqes;
	struct regpair pbl_addr;
	struct regpair output_params_addr;
	u8 vf_id;
	u8 reserved1[7];
};


/*
 * The rdma SRQ context
 */
struct rdma_srq_context
{
	struct regpair temp[8];
};


/*
 * rdma create qp requester ramrod data
 */
struct rdma_srq_create_ramrod_data
{
	u8 flags;
#define RDMA_SRQ_CREATE_RAMROD_DATA_XRC_FLAG_MASK         0x1
#define RDMA_SRQ_CREATE_RAMROD_DATA_XRC_FLAG_SHIFT        0
#define RDMA_SRQ_CREATE_RAMROD_DATA_RESERVED_KEY_EN_MASK  0x1 /* Only applicable when xrc_flag is set */
#define RDMA_SRQ_CREATE_RAMROD_DATA_RESERVED_KEY_EN_SHIFT 1
#define RDMA_SRQ_CREATE_RAMROD_DATA_RESERVED1_MASK        0x3F
#define RDMA_SRQ_CREATE_RAMROD_DATA_RESERVED1_SHIFT       2
	u8 reserved2;
	__le16 xrc_domain /* Only applicable when xrc_flag is set */;
	__le32 xrc_srq_cq_cid /* Only applicable when xrc_flag is set */;
	struct regpair pbl_base_addr /* SRQ PBL base address */;
	__le16 pages_in_srq_pbl /* Number of pages in PBL */;
	__le16 pd_id;
	struct rdma_srq_id srq_id /* SRQ Index */;
	__le16 page_size /* Page size in SGEs(16 bytes) elements. Supports up to 2M bytes page size */;
	__le16 reserved3;
	__le32 reserved4;
	struct regpair producers_addr /* SRQ PBL base address */;
};


/*
 * rdma create qp requester ramrod data
 */
struct rdma_srq_destroy_ramrod_data
{
	struct rdma_srq_id srq_id /* SRQ Index */;
	__le32 reserved;
};


/*
 * rdma create qp requester ramrod data
 */
struct rdma_srq_modify_ramrod_data
{
	struct rdma_srq_id srq_id /* SRQ Index */;
	__le32 wqe_limit;
};


/*
 * The rdma task context of Mstorm
 */
struct ystorm_rdma_task_st_ctx
{
	struct regpair temp[4];
};

struct ystorm_rdma_task_ag_ctx
{
	u8 reserved /* cdu_validation */;
	u8 byte1 /* state */;
	__le16 msem_ctx_upd_seq /* icid */;
	u8 flags0;
#define YSTORM_RDMA_TASK_AG_CTX_CONNECTION_TYPE_MASK  0xF /* connection_type */
#define YSTORM_RDMA_TASK_AG_CTX_CONNECTION_TYPE_SHIFT 0
#define YSTORM_RDMA_TASK_AG_CTX_EXIST_IN_QM0_MASK     0x1 /* exist_in_qm0 */
#define YSTORM_RDMA_TASK_AG_CTX_EXIST_IN_QM0_SHIFT    4
#define YSTORM_RDMA_TASK_AG_CTX_BIT1_MASK             0x1 /* exist_in_qm1 */
#define YSTORM_RDMA_TASK_AG_CTX_BIT1_SHIFT            5
#define YSTORM_RDMA_TASK_AG_CTX_VALID_MASK            0x1 /* bit2 */
#define YSTORM_RDMA_TASK_AG_CTX_VALID_SHIFT           6
#define YSTORM_RDMA_TASK_AG_CTX_DIF_FIRST_IO_MASK     0x1 /* bit3 */
#define YSTORM_RDMA_TASK_AG_CTX_DIF_FIRST_IO_SHIFT    7
	u8 flags1;
#define YSTORM_RDMA_TASK_AG_CTX_CF0_MASK              0x3 /* cf0 */
#define YSTORM_RDMA_TASK_AG_CTX_CF0_SHIFT             0
#define YSTORM_RDMA_TASK_AG_CTX_CF1_MASK              0x3 /* cf1 */
#define YSTORM_RDMA_TASK_AG_CTX_CF1_SHIFT             2
#define YSTORM_RDMA_TASK_AG_CTX_CF2SPECIAL_MASK       0x3 /* cf2special */
#define YSTORM_RDMA_TASK_AG_CTX_CF2SPECIAL_SHIFT      4
#define YSTORM_RDMA_TASK_AG_CTX_CF0EN_MASK            0x1 /* cf0en */
#define YSTORM_RDMA_TASK_AG_CTX_CF0EN_SHIFT           6
#define YSTORM_RDMA_TASK_AG_CTX_CF1EN_MASK            0x1 /* cf1en */
#define YSTORM_RDMA_TASK_AG_CTX_CF1EN_SHIFT           7
	u8 flags2;
#define YSTORM_RDMA_TASK_AG_CTX_BIT4_MASK             0x1 /* bit4 */
#define YSTORM_RDMA_TASK_AG_CTX_BIT4_SHIFT            0
#define YSTORM_RDMA_TASK_AG_CTX_RULE0EN_MASK          0x1 /* rule0en */
#define YSTORM_RDMA_TASK_AG_CTX_RULE0EN_SHIFT         1
#define YSTORM_RDMA_TASK_AG_CTX_RULE1EN_MASK          0x1 /* rule1en */
#define YSTORM_RDMA_TASK_AG_CTX_RULE1EN_SHIFT         2
#define YSTORM_RDMA_TASK_AG_CTX_RULE2EN_MASK          0x1 /* rule2en */
#define YSTORM_RDMA_TASK_AG_CTX_RULE2EN_SHIFT         3
#define YSTORM_RDMA_TASK_AG_CTX_RULE3EN_MASK          0x1 /* rule3en */
#define YSTORM_RDMA_TASK_AG_CTX_RULE3EN_SHIFT         4
#define YSTORM_RDMA_TASK_AG_CTX_RULE4EN_MASK          0x1 /* rule4en */
#define YSTORM_RDMA_TASK_AG_CTX_RULE4EN_SHIFT         5
#define YSTORM_RDMA_TASK_AG_CTX_RULE5EN_MASK          0x1 /* rule5en */
#define YSTORM_RDMA_TASK_AG_CTX_RULE5EN_SHIFT         6
#define YSTORM_RDMA_TASK_AG_CTX_RULE6EN_MASK          0x1 /* rule6en */
#define YSTORM_RDMA_TASK_AG_CTX_RULE6EN_SHIFT         7
	u8 key /* byte2 */;
	__le32 mw_cnt_or_qp_id /* reg0 */;
	u8 ref_cnt_seq /* byte3 */;
	u8 ctx_upd_seq /* byte4 */;
	__le16 dif_flags /* word1 */;
	__le16 tx_ref_count /* word2 */;
	__le16 last_used_ltid /* word3 */;
	__le16 parent_mr_lo /* word4 */;
	__le16 parent_mr_hi /* word5 */;
	__le32 fbo_lo /* reg1 */;
	__le32 fbo_hi /* reg2 */;
};

struct mstorm_rdma_task_ag_ctx
{
	u8 reserved /* cdu_validation */;
	u8 byte1 /* state */;
	__le16 icid /* icid */;
	u8 flags0;
#define MSTORM_RDMA_TASK_AG_CTX_CONNECTION_TYPE_MASK  0xF /* connection_type */
#define MSTORM_RDMA_TASK_AG_CTX_CONNECTION_TYPE_SHIFT 0
#define MSTORM_RDMA_TASK_AG_CTX_EXIST_IN_QM0_MASK     0x1 /* exist_in_qm0 */
#define MSTORM_RDMA_TASK_AG_CTX_EXIST_IN_QM0_SHIFT    4
#define MSTORM_RDMA_TASK_AG_CTX_BIT1_MASK             0x1 /* exist_in_qm1 */
#define MSTORM_RDMA_TASK_AG_CTX_BIT1_SHIFT            5
#define MSTORM_RDMA_TASK_AG_CTX_BIT2_MASK             0x1 /* bit2 */
#define MSTORM_RDMA_TASK_AG_CTX_BIT2_SHIFT            6
#define MSTORM_RDMA_TASK_AG_CTX_DIF_FIRST_IO_MASK     0x1 /* bit3 */
#define MSTORM_RDMA_TASK_AG_CTX_DIF_FIRST_IO_SHIFT    7
	u8 flags1;
#define MSTORM_RDMA_TASK_AG_CTX_CF0_MASK              0x3 /* cf0 */
#define MSTORM_RDMA_TASK_AG_CTX_CF0_SHIFT             0
#define MSTORM_RDMA_TASK_AG_CTX_CF1_MASK              0x3 /* cf1 */
#define MSTORM_RDMA_TASK_AG_CTX_CF1_SHIFT             2
#define MSTORM_RDMA_TASK_AG_CTX_CF2_MASK              0x3 /* cf2 */
#define MSTORM_RDMA_TASK_AG_CTX_CF2_SHIFT             4
#define MSTORM_RDMA_TASK_AG_CTX_CF0EN_MASK            0x1 /* cf0en */
#define MSTORM_RDMA_TASK_AG_CTX_CF0EN_SHIFT           6
#define MSTORM_RDMA_TASK_AG_CTX_CF1EN_MASK            0x1 /* cf1en */
#define MSTORM_RDMA_TASK_AG_CTX_CF1EN_SHIFT           7
	u8 flags2;
#define MSTORM_RDMA_TASK_AG_CTX_CF2EN_MASK            0x1 /* cf2en */
#define MSTORM_RDMA_TASK_AG_CTX_CF2EN_SHIFT           0
#define MSTORM_RDMA_TASK_AG_CTX_RULE0EN_MASK          0x1 /* rule0en */
#define MSTORM_RDMA_TASK_AG_CTX_RULE0EN_SHIFT         1
#define MSTORM_RDMA_TASK_AG_CTX_RULE1EN_MASK          0x1 /* rule1en */
#define MSTORM_RDMA_TASK_AG_CTX_RULE1EN_SHIFT         2
#define MSTORM_RDMA_TASK_AG_CTX_RULE2EN_MASK          0x1 /* rule2en */
#define MSTORM_RDMA_TASK_AG_CTX_RULE2EN_SHIFT         3
#define MSTORM_RDMA_TASK_AG_CTX_RULE3EN_MASK          0x1 /* rule3en */
#define MSTORM_RDMA_TASK_AG_CTX_RULE3EN_SHIFT         4
#define MSTORM_RDMA_TASK_AG_CTX_RULE4EN_MASK          0x1 /* rule4en */
#define MSTORM_RDMA_TASK_AG_CTX_RULE4EN_SHIFT         5
#define MSTORM_RDMA_TASK_AG_CTX_RULE5EN_MASK          0x1 /* rule5en */
#define MSTORM_RDMA_TASK_AG_CTX_RULE5EN_SHIFT         6
#define MSTORM_RDMA_TASK_AG_CTX_RULE6EN_MASK          0x1 /* rule6en */
#define MSTORM_RDMA_TASK_AG_CTX_RULE6EN_SHIFT         7
	u8 key /* byte2 */;
	__le32 mw_cnt_or_qp_id /* reg0 */;
	u8 ref_cnt_seq /* byte3 */;
	u8 ctx_upd_seq /* byte4 */;
	__le16 dif_flags /* word1 */;
	__le16 tx_ref_count /* word2 */;
	__le16 last_used_ltid /* word3 */;
	__le16 parent_mr_lo /* word4 */;
	__le16 parent_mr_hi /* word5 */;
	__le32 fbo_lo /* reg1 */;
	__le32 fbo_hi /* reg2 */;
};

/*
 * The roce task context of Ustorm
 */
struct ustorm_rdma_task_st_ctx
{
	struct regpair temp[6];
};

struct ustorm_rdma_task_ag_ctx
{
	u8 reserved /* cdu_validation */;
	u8 state /* state */;
	__le16 icid /* icid */;
	u8 flags0;
#define USTORM_RDMA_TASK_AG_CTX_CONNECTION_TYPE_MASK         0xF /* connection_type */
#define USTORM_RDMA_TASK_AG_CTX_CONNECTION_TYPE_SHIFT        0
#define USTORM_RDMA_TASK_AG_CTX_EXIST_IN_QM0_MASK            0x1 /* exist_in_qm0 */
#define USTORM_RDMA_TASK_AG_CTX_EXIST_IN_QM0_SHIFT           4
#define USTORM_RDMA_TASK_AG_CTX_BIT1_MASK                    0x1 /* exist_in_qm1 */
#define USTORM_RDMA_TASK_AG_CTX_BIT1_SHIFT                   5
#define USTORM_RDMA_TASK_AG_CTX_DIF_WRITE_RESULT_CF_MASK     0x3 /* timer0cf */
#define USTORM_RDMA_TASK_AG_CTX_DIF_WRITE_RESULT_CF_SHIFT    6
	u8 flags1;
#define USTORM_RDMA_TASK_AG_CTX_DIF_RESULT_TOGGLE_BIT_MASK   0x3 /* timer1cf */
#define USTORM_RDMA_TASK_AG_CTX_DIF_RESULT_TOGGLE_BIT_SHIFT  0
#define USTORM_RDMA_TASK_AG_CTX_DIF_TX_IO_FLG_MASK           0x3 /* timer2cf */
#define USTORM_RDMA_TASK_AG_CTX_DIF_TX_IO_FLG_SHIFT          2
#define USTORM_RDMA_TASK_AG_CTX_DIF_BLOCK_SIZE_MASK          0x3 /* timer_stop_all */
#define USTORM_RDMA_TASK_AG_CTX_DIF_BLOCK_SIZE_SHIFT         4
#define USTORM_RDMA_TASK_AG_CTX_DIF_ERROR_CF_MASK            0x3 /* cf4 */
#define USTORM_RDMA_TASK_AG_CTX_DIF_ERROR_CF_SHIFT           6
	u8 flags2;
#define USTORM_RDMA_TASK_AG_CTX_DIF_WRITE_RESULT_CF_EN_MASK  0x1 /* cf0en */
#define USTORM_RDMA_TASK_AG_CTX_DIF_WRITE_RESULT_CF_EN_SHIFT 0
#define USTORM_RDMA_TASK_AG_CTX_RESERVED2_MASK               0x1 /* cf1en */
#define USTORM_RDMA_TASK_AG_CTX_RESERVED2_SHIFT              1
#define USTORM_RDMA_TASK_AG_CTX_RESERVED3_MASK               0x1 /* cf2en */
#define USTORM_RDMA_TASK_AG_CTX_RESERVED3_SHIFT              2
#define USTORM_RDMA_TASK_AG_CTX_RESERVED4_MASK               0x1 /* cf3en */
#define USTORM_RDMA_TASK_AG_CTX_RESERVED4_SHIFT              3
#define USTORM_RDMA_TASK_AG_CTX_DIF_ERROR_CF_EN_MASK         0x1 /* cf4en */
#define USTORM_RDMA_TASK_AG_CTX_DIF_ERROR_CF_EN_SHIFT        4
#define USTORM_RDMA_TASK_AG_CTX_RULE0EN_MASK                 0x1 /* rule0en */
#define USTORM_RDMA_TASK_AG_CTX_RULE0EN_SHIFT                5
#define USTORM_RDMA_TASK_AG_CTX_RULE1EN_MASK                 0x1 /* rule1en */
#define USTORM_RDMA_TASK_AG_CTX_RULE1EN_SHIFT                6
#define USTORM_RDMA_TASK_AG_CTX_RULE2EN_MASK                 0x1 /* rule2en */
#define USTORM_RDMA_TASK_AG_CTX_RULE2EN_SHIFT                7
	u8 flags3;
#define USTORM_RDMA_TASK_AG_CTX_DIF_RXMIT_PROD_CONS_EN_MASK  0x1 /* rule3en */
#define USTORM_RDMA_TASK_AG_CTX_DIF_RXMIT_PROD_CONS_EN_SHIFT 0
#define USTORM_RDMA_TASK_AG_CTX_RULE4EN_MASK                 0x1 /* rule4en */
#define USTORM_RDMA_TASK_AG_CTX_RULE4EN_SHIFT                1
#define USTORM_RDMA_TASK_AG_CTX_DIF_WRITE_PROD_CONS_EN_MASK  0x1 /* rule5en */
#define USTORM_RDMA_TASK_AG_CTX_DIF_WRITE_PROD_CONS_EN_SHIFT 2
#define USTORM_RDMA_TASK_AG_CTX_RULE6EN_MASK                 0x1 /* rule6en */
#define USTORM_RDMA_TASK_AG_CTX_RULE6EN_SHIFT                3
#define USTORM_RDMA_TASK_AG_CTX_DIF_ERROR_TYPE_MASK          0xF /* nibble1 */
#define USTORM_RDMA_TASK_AG_CTX_DIF_ERROR_TYPE_SHIFT         4
	__le32 dif_err_intervals /* reg0 */;
	__le32 dif_error_1st_interval /* reg1 */;
	__le32 dif_rxmit_cons /* reg2 */;
	__le32 dif_rxmit_prod /* reg3 */;
	__le32 sge_index /* reg4 */;
	__le32 sq_cons /* reg5 */;
	u8 byte2 /* byte2 */;
	u8 byte3 /* byte3 */;
	__le16 dif_write_cons /* word1 */;
	__le16 dif_write_prod /* word2 */;
	__le16 word3 /* word3 */;
	__le32 dif_error_buffer_address_lo /* reg6 */;
	__le32 dif_error_buffer_address_hi /* reg7 */;
};

/*
 * RDMA task context
 */
struct rdma_task_context
{
	struct ystorm_rdma_task_st_ctx ystorm_st_context /* ystorm storm context */;
	struct ystorm_rdma_task_ag_ctx ystorm_ag_context /* ystorm aggregative context */;
	struct tdif_task_context tdif_context /* tdif context */;
	struct mstorm_rdma_task_ag_ctx mstorm_ag_context /* mstorm aggregative context */;
	struct mstorm_rdma_task_st_ctx mstorm_st_context /* mstorm storm context */;
	struct rdif_task_context rdif_context /* rdif context */;
	struct ustorm_rdma_task_st_ctx ustorm_st_context /* ustorm storm context */;
	struct regpair ustorm_st_padding[2] /* padding */;
	struct ustorm_rdma_task_ag_ctx ustorm_ag_context /* ustorm aggregative context */;
};


/*
 * RDMA Tid type enumeration (for register_tid ramrod)
 */
enum rdma_tid_type
{
	RDMA_TID_REGISTERED_MR,
	RDMA_TID_FMR,
	RDMA_TID_MW,
	MAX_RDMA_TID_TYPE
};


/*
 * The rdma XRC SRQ context
 */
struct rdma_xrc_srq_context
{
	struct regpair temp[9];
};





struct tstorm_rdma_task_ag_ctx
{
	u8 byte0 /* cdu_validation */;
	u8 byte1 /* state */;
	__le16 word0 /* icid */;
	u8 flags0;
#define TSTORM_RDMA_TASK_AG_CTX_NIBBLE0_MASK  0xF /* connection_type */
#define TSTORM_RDMA_TASK_AG_CTX_NIBBLE0_SHIFT 0
#define TSTORM_RDMA_TASK_AG_CTX_BIT0_MASK     0x1 /* exist_in_qm0 */
#define TSTORM_RDMA_TASK_AG_CTX_BIT0_SHIFT    4
#define TSTORM_RDMA_TASK_AG_CTX_BIT1_MASK     0x1 /* exist_in_qm1 */
#define TSTORM_RDMA_TASK_AG_CTX_BIT1_SHIFT    5
#define TSTORM_RDMA_TASK_AG_CTX_BIT2_MASK     0x1 /* bit2 */
#define TSTORM_RDMA_TASK_AG_CTX_BIT2_SHIFT    6
#define TSTORM_RDMA_TASK_AG_CTX_BIT3_MASK     0x1 /* bit3 */
#define TSTORM_RDMA_TASK_AG_CTX_BIT3_SHIFT    7
	u8 flags1;
#define TSTORM_RDMA_TASK_AG_CTX_BIT4_MASK     0x1 /* bit4 */
#define TSTORM_RDMA_TASK_AG_CTX_BIT4_SHIFT    0
#define TSTORM_RDMA_TASK_AG_CTX_BIT5_MASK     0x1 /* bit5 */
#define TSTORM_RDMA_TASK_AG_CTX_BIT5_SHIFT    1
#define TSTORM_RDMA_TASK_AG_CTX_CF0_MASK      0x3 /* timer0cf */
#define TSTORM_RDMA_TASK_AG_CTX_CF0_SHIFT     2
#define TSTORM_RDMA_TASK_AG_CTX_CF1_MASK      0x3 /* timer1cf */
#define TSTORM_RDMA_TASK_AG_CTX_CF1_SHIFT     4
#define TSTORM_RDMA_TASK_AG_CTX_CF2_MASK      0x3 /* timer2cf */
#define TSTORM_RDMA_TASK_AG_CTX_CF2_SHIFT     6
	u8 flags2;
#define TSTORM_RDMA_TASK_AG_CTX_CF3_MASK      0x3 /* timer_stop_all */
#define TSTORM_RDMA_TASK_AG_CTX_CF3_SHIFT     0
#define TSTORM_RDMA_TASK_AG_CTX_CF4_MASK      0x3 /* cf4 */
#define TSTORM_RDMA_TASK_AG_CTX_CF4_SHIFT     2
#define TSTORM_RDMA_TASK_AG_CTX_CF5_MASK      0x3 /* cf5 */
#define TSTORM_RDMA_TASK_AG_CTX_CF5_SHIFT     4
#define TSTORM_RDMA_TASK_AG_CTX_CF6_MASK      0x3 /* cf6 */
#define TSTORM_RDMA_TASK_AG_CTX_CF6_SHIFT     6
	u8 flags3;
#define TSTORM_RDMA_TASK_AG_CTX_CF7_MASK      0x3 /* cf7 */
#define TSTORM_RDMA_TASK_AG_CTX_CF7_SHIFT     0
#define TSTORM_RDMA_TASK_AG_CTX_CF0EN_MASK    0x1 /* cf0en */
#define TSTORM_RDMA_TASK_AG_CTX_CF0EN_SHIFT   2
#define TSTORM_RDMA_TASK_AG_CTX_CF1EN_MASK    0x1 /* cf1en */
#define TSTORM_RDMA_TASK_AG_CTX_CF1EN_SHIFT   3
#define TSTORM_RDMA_TASK_AG_CTX_CF2EN_MASK    0x1 /* cf2en */
#define TSTORM_RDMA_TASK_AG_CTX_CF2EN_SHIFT   4
#define TSTORM_RDMA_TASK_AG_CTX_CF3EN_MASK    0x1 /* cf3en */
#define TSTORM_RDMA_TASK_AG_CTX_CF3EN_SHIFT   5
#define TSTORM_RDMA_TASK_AG_CTX_CF4EN_MASK    0x1 /* cf4en */
#define TSTORM_RDMA_TASK_AG_CTX_CF4EN_SHIFT   6
#define TSTORM_RDMA_TASK_AG_CTX_CF5EN_MASK    0x1 /* cf5en */
#define TSTORM_RDMA_TASK_AG_CTX_CF5EN_SHIFT   7
	u8 flags4;
#define TSTORM_RDMA_TASK_AG_CTX_CF6EN_MASK    0x1 /* cf6en */
#define TSTORM_RDMA_TASK_AG_CTX_CF6EN_SHIFT   0
#define TSTORM_RDMA_TASK_AG_CTX_CF7EN_MASK    0x1 /* cf7en */
#define TSTORM_RDMA_TASK_AG_CTX_CF7EN_SHIFT   1
#define TSTORM_RDMA_TASK_AG_CTX_RULE0EN_MASK  0x1 /* rule0en */
#define TSTORM_RDMA_TASK_AG_CTX_RULE0EN_SHIFT 2
#define TSTORM_RDMA_TASK_AG_CTX_RULE1EN_MASK  0x1 /* rule1en */
#define TSTORM_RDMA_TASK_AG_CTX_RULE1EN_SHIFT 3
#define TSTORM_RDMA_TASK_AG_CTX_RULE2EN_MASK  0x1 /* rule2en */
#define TSTORM_RDMA_TASK_AG_CTX_RULE2EN_SHIFT 4
#define TSTORM_RDMA_TASK_AG_CTX_RULE3EN_MASK  0x1 /* rule3en */
#define TSTORM_RDMA_TASK_AG_CTX_RULE3EN_SHIFT 5
#define TSTORM_RDMA_TASK_AG_CTX_RULE4EN_MASK  0x1 /* rule4en */
#define TSTORM_RDMA_TASK_AG_CTX_RULE4EN_SHIFT 6
#define TSTORM_RDMA_TASK_AG_CTX_RULE5EN_MASK  0x1 /* rule5en */
#define TSTORM_RDMA_TASK_AG_CTX_RULE5EN_SHIFT 7
	u8 byte2 /* byte2 */;
	__le16 word1 /* word1 */;
	__le32 reg0 /* reg0 */;
	u8 byte3 /* byte3 */;
	u8 byte4 /* byte4 */;
	__le16 word2 /* word2 */;
	__le16 word3 /* word3 */;
	__le16 word4 /* word4 */;
	__le32 reg1 /* reg1 */;
	__le32 reg2 /* reg2 */;
};


struct ustorm_rdma_conn_ag_ctx
{
	u8 reserved /* cdu_validation */;
	u8 byte1 /* state */;
	u8 flags0;
#define USTORM_RDMA_CONN_AG_CTX_EXIST_IN_QM0_MASK        0x1 /* exist_in_qm0 */
#define USTORM_RDMA_CONN_AG_CTX_EXIST_IN_QM0_SHIFT       0
#define USTORM_RDMA_CONN_AG_CTX_DIF_ERROR_REPORTED_MASK  0x1 /* exist_in_qm1 */
#define USTORM_RDMA_CONN_AG_CTX_DIF_ERROR_REPORTED_SHIFT 1
#define USTORM_RDMA_CONN_AG_CTX_FLUSH_Q0_CF_MASK         0x3 /* timer0cf */
#define USTORM_RDMA_CONN_AG_CTX_FLUSH_Q0_CF_SHIFT        2
#define USTORM_RDMA_CONN_AG_CTX_CF1_MASK                 0x3 /* timer1cf */
#define USTORM_RDMA_CONN_AG_CTX_CF1_SHIFT                4
#define USTORM_RDMA_CONN_AG_CTX_CF2_MASK                 0x3 /* timer2cf */
#define USTORM_RDMA_CONN_AG_CTX_CF2_SHIFT                6
	u8 flags1;
#define USTORM_RDMA_CONN_AG_CTX_CF3_MASK                 0x3 /* timer_stop_all */
#define USTORM_RDMA_CONN_AG_CTX_CF3_SHIFT                0
#define USTORM_RDMA_CONN_AG_CTX_CQ_ARM_SE_CF_MASK        0x3 /* cf4 */
#define USTORM_RDMA_CONN_AG_CTX_CQ_ARM_SE_CF_SHIFT       2
#define USTORM_RDMA_CONN_AG_CTX_CQ_ARM_CF_MASK           0x3 /* cf5 */
#define USTORM_RDMA_CONN_AG_CTX_CQ_ARM_CF_SHIFT          4
#define USTORM_RDMA_CONN_AG_CTX_CF6_MASK                 0x3 /* cf6 */
#define USTORM_RDMA_CONN_AG_CTX_CF6_SHIFT                6
	u8 flags2;
#define USTORM_RDMA_CONN_AG_CTX_FLUSH_Q0_CF_EN_MASK      0x1 /* cf0en */
#define USTORM_RDMA_CONN_AG_CTX_FLUSH_Q0_CF_EN_SHIFT     0
#define USTORM_RDMA_CONN_AG_CTX_CF1EN_MASK               0x1 /* cf1en */
#define USTORM_RDMA_CONN_AG_CTX_CF1EN_SHIFT              1
#define USTORM_RDMA_CONN_AG_CTX_CF2EN_MASK               0x1 /* cf2en */
#define USTORM_RDMA_CONN_AG_CTX_CF2EN_SHIFT              2
#define USTORM_RDMA_CONN_AG_CTX_CF3EN_MASK               0x1 /* cf3en */
#define USTORM_RDMA_CONN_AG_CTX_CF3EN_SHIFT              3
#define USTORM_RDMA_CONN_AG_CTX_CQ_ARM_SE_CF_EN_MASK     0x1 /* cf4en */
#define USTORM_RDMA_CONN_AG_CTX_CQ_ARM_SE_CF_EN_SHIFT    4
#define USTORM_RDMA_CONN_AG_CTX_CQ_ARM_CF_EN_MASK        0x1 /* cf5en */
#define USTORM_RDMA_CONN_AG_CTX_CQ_ARM_CF_EN_SHIFT       5
#define USTORM_RDMA_CONN_AG_CTX_CF6EN_MASK               0x1 /* cf6en */
#define USTORM_RDMA_CONN_AG_CTX_CF6EN_SHIFT              6
#define USTORM_RDMA_CONN_AG_CTX_CQ_SE_EN_MASK            0x1 /* rule0en */
#define USTORM_RDMA_CONN_AG_CTX_CQ_SE_EN_SHIFT           7
	u8 flags3;
#define USTORM_RDMA_CONN_AG_CTX_CQ_EN_MASK               0x1 /* rule1en */
#define USTORM_RDMA_CONN_AG_CTX_CQ_EN_SHIFT              0
#define USTORM_RDMA_CONN_AG_CTX_RULE2EN_MASK             0x1 /* rule2en */
#define USTORM_RDMA_CONN_AG_CTX_RULE2EN_SHIFT            1
#define USTORM_RDMA_CONN_AG_CTX_RULE3EN_MASK             0x1 /* rule3en */
#define USTORM_RDMA_CONN_AG_CTX_RULE3EN_SHIFT            2
#define USTORM_RDMA_CONN_AG_CTX_RULE4EN_MASK             0x1 /* rule4en */
#define USTORM_RDMA_CONN_AG_CTX_RULE4EN_SHIFT            3
#define USTORM_RDMA_CONN_AG_CTX_RULE5EN_MASK             0x1 /* rule5en */
#define USTORM_RDMA_CONN_AG_CTX_RULE5EN_SHIFT            4
#define USTORM_RDMA_CONN_AG_CTX_RULE6EN_MASK             0x1 /* rule6en */
#define USTORM_RDMA_CONN_AG_CTX_RULE6EN_SHIFT            5
#define USTORM_RDMA_CONN_AG_CTX_RULE7EN_MASK             0x1 /* rule7en */
#define USTORM_RDMA_CONN_AG_CTX_RULE7EN_SHIFT            6
#define USTORM_RDMA_CONN_AG_CTX_RULE8EN_MASK             0x1 /* rule8en */
#define USTORM_RDMA_CONN_AG_CTX_RULE8EN_SHIFT            7
	u8 byte2 /* byte2 */;
	u8 nvmf_only /* byte3 */;
	__le16 conn_dpi /* conn_dpi */;
	__le16 word1 /* word1 */;
	__le32 cq_cons /* reg0 */;
	__le32 cq_se_prod /* reg1 */;
	__le32 cq_prod /* reg2 */;
	__le32 reg3 /* reg3 */;
	__le16 int_timeout /* word2 */;
	__le16 word3 /* word3 */;
};



#endif /* __ECORE_HSI_RDMA__ */
