#ifndef __ECORE_OOO_H__
#define __ECORE_OOO_H__

#include "ecore.h"

#define ECORE_MAX_NUM_ISLES	256
#define ECORE_MAX_NUM_OOO_HISTORY_ENTRIES	512

#define ECORE_OOO_LEFT_BUF	0
#define ECORE_OOO_RIGHT_BUF	1

struct ecore_ooo_buffer {
	osal_list_entry_t	list_entry;
	void			*rx_buffer_virt_addr;
	dma_addr_t		rx_buffer_phys_addr;
	u32			rx_buffer_size;
	u16			packet_length;
	u16			parse_flags;
	u16			vlan;
	u8			placement_offset;
};

struct ecore_ooo_isle {
	osal_list_entry_t	list_entry;
	osal_list_t		buffers_list;
};

struct ecore_ooo_archipelago {
	osal_list_t		isles_list;
};

struct ecore_ooo_history {
	struct ooo_opaque	*p_cqes;
	u32			head_idx;
	u32			num_of_cqes;
};

struct ecore_ooo_info {
	osal_list_t	 free_buffers_list;
	osal_list_t	 ready_buffers_list;
	osal_list_t	 free_isles_list;
	struct ecore_ooo_archipelago	*p_archipelagos_mem;
	struct ecore_ooo_isle	*p_isles_mem;
	struct ecore_ooo_history	ooo_history;
	u32		cur_isles_number;
	u32		max_isles_number;
	u32		gen_isles_number;
	u16		max_num_archipelagos;
	u16		cid_base;
};

#ifdef CONFIG_ECORE_OOO
void ecore_ooo_save_history_entry(struct ecore_ooo_info *p_ooo_info,
				  struct ooo_opaque *p_cqe);

enum _ecore_status_t ecore_ooo_alloc(struct ecore_hwfn *p_hwfn);

void ecore_ooo_setup(struct ecore_hwfn *p_hwfn);

void ecore_ooo_free(struct ecore_hwfn *p_hwfn);

void ecore_ooo_release_connection_isles(struct ecore_ooo_info *p_ooo_info,
					u32 cid);

void ecore_ooo_release_all_isles(struct ecore_ooo_info *p_ooo_info);

void ecore_ooo_put_free_buffer(struct ecore_ooo_info *p_ooo_info,
			       struct ecore_ooo_buffer *p_buffer);

struct ecore_ooo_buffer *
ecore_ooo_get_free_buffer(struct ecore_ooo_info *p_ooo_info);

void ecore_ooo_put_ready_buffer(struct ecore_ooo_info *p_ooo_info,
				struct ecore_ooo_buffer *p_buffer, u8 on_tail);

struct ecore_ooo_buffer *
ecore_ooo_get_ready_buffer(struct ecore_ooo_info *p_ooo_info);

void ecore_ooo_delete_isles(struct ecore_hwfn *p_hwfn,
			   struct ecore_ooo_info *p_ooo_info,
			   u32 cid,
			   u8 drop_isle,
			   u8 drop_size);

void ecore_ooo_add_new_isle(struct ecore_hwfn *p_hwfn,
			   struct ecore_ooo_info *p_ooo_info,
			   u32 cid,
			   u8 ooo_isle,
			   struct ecore_ooo_buffer *p_buffer);

void ecore_ooo_add_new_buffer(struct ecore_hwfn *p_hwfn,
			     struct ecore_ooo_info *p_ooo_info,
			     u32 cid,
			     u8 ooo_isle,
			     struct ecore_ooo_buffer *p_buffer,
			     u8 buffer_side);

void ecore_ooo_join_isles(struct ecore_hwfn	*p_hwfn,
			 struct ecore_ooo_info *p_ooo_info,
			 u32 cid,
			 u8 left_isle);

void ecore_ooo_dump_rx_event(struct ecore_hwfn *p_hwfn,
			     struct ooo_opaque *iscsi_ooo,
			     struct ecore_ooo_buffer *p_buffer);
#else
static OSAL_INLINE void ecore_ooo_save_history_entry(struct ecore_ooo_info OSAL_UNUSED *p_ooo_info, struct ooo_opaque OSAL_UNUSED *p_cqe) {}
static OSAL_INLINE enum _ecore_status_t ecore_ooo_alloc(struct ecore_hwfn OSAL_UNUSED *p_hwfn) { return ECORE_INVAL; }
static OSAL_INLINE void ecore_ooo_setup(struct ecore_hwfn OSAL_UNUSED *p_hwfn) {}
static OSAL_INLINE void ecore_ooo_free(struct ecore_hwfn OSAL_UNUSED *p_hwfn) {}
static OSAL_INLINE void ecore_ooo_release_connection_isles(struct ecore_ooo_info OSAL_UNUSED *p_ooo_info, u32 OSAL_UNUSED cid) {}
static OSAL_INLINE void ecore_ooo_release_all_isles(struct ecore_ooo_info OSAL_UNUSED *p_ooo_info) {}
static OSAL_INLINE void ecore_ooo_put_free_buffer(struct ecore_ooo_info OSAL_UNUSED *p_ooo_info, struct ecore_ooo_buffer OSAL_UNUSED *p_buffer) {}
static OSAL_INLINE struct ecore_ooo_buffer *ecore_ooo_get_free_buffer(struct ecore_ooo_info OSAL_UNUSED *p_ooo_info) { return OSAL_NULL; }
static OSAL_INLINE void ecore_ooo_put_ready_buffer(struct ecore_ooo_info OSAL_UNUSED *p_ooo_info, struct ecore_ooo_buffer OSAL_UNUSED *p_buffer, u8 OSAL_UNUSED on_tail) {}
static OSAL_INLINE struct ecore_ooo_buffer *ecore_ooo_get_ready_buffer(struct ecore_ooo_info OSAL_UNUSED *p_ooo_info) { return OSAL_NULL; }
static OSAL_INLINE void ecore_ooo_delete_isles(struct ecore_hwfn OSAL_UNUSED *p_hwfn, struct ecore_ooo_info OSAL_UNUSED *p_ooo_info, u32 OSAL_UNUSED cid, u8 OSAL_UNUSED drop_isle, OSAL_UNUSED u8 drop_size) {}
static OSAL_INLINE void ecore_ooo_add_new_isle(struct ecore_hwfn OSAL_UNUSED *p_hwfn, struct ecore_ooo_info OSAL_UNUSED *p_ooo_info, u32 OSAL_UNUSED cid, u8 OSAL_UNUSED ooo_isle, struct ecore_ooo_buffer OSAL_UNUSED *p_buffer) {}
static OSAL_INLINE void ecore_ooo_add_new_buffer(struct ecore_hwfn OSAL_UNUSED *p_hwfn, struct ecore_ooo_info OSAL_UNUSED *p_ooo_info, u32 OSAL_UNUSED cid, u8 OSAL_UNUSED ooo_isle, struct ecore_ooo_buffer OSAL_UNUSED *p_buffer, u8 OSAL_UNUSED buffer_side) {}
static OSAL_INLINE void ecore_ooo_join_isles(struct ecore_hwfn OSAL_UNUSED *p_hwfn, struct ecore_ooo_info OSAL_UNUSED *p_ooo_info, u32 OSAL_UNUSED cid, u8 OSAL_UNUSED left_isle) {}
static OSAL_INLINE void ecore_ooo_dump_rx_event(struct ecore_hwfn OSAL_UNUSED *p_hwfn, struct ooo_opaque OSAL_UNUSED *iscsi_ooo, struct ecore_ooo_buffer OSAL_UNUSED *p_buffer) {}
#endif

#endif  /*__ECORE_OOO_H__*/

