#ifndef __ECORE_ROCE_H__
#define __ECORE_ROCE_H__

#include "ecore_status.h"

/* functions for enabling/disabling edpm in rdma PFs according to existence of
 * qps during DCBx update or bar size
 */
void
ecore_roce_dpm_dcbx(struct ecore_hwfn *p_hwfn, struct ecore_ptt *p_ptt);

void
ecore_rdma_dpm_bar(struct ecore_hwfn *p_hwfn, struct ecore_ptt *p_ptt);

enum _ecore_status_t
ecore_roce_dcqcn_cfg(struct ecore_hwfn *p_hwfn,
		     struct ecore_roce_dcqcn_params *params,
		     struct roce_init_func_ramrod_data *p_ramrod,
		     struct ecore_ptt *p_ptt);

enum _ecore_status_t
ecore_roce_setup(struct ecore_hwfn *p_hwfn);

enum _ecore_status_t
ecore_roce_stop_rl(struct ecore_hwfn *p_hwfn);

enum _ecore_status_t
ecore_roce_stop(struct ecore_hwfn *p_hwfn, struct ecore_rdma_info *rdma_info);

enum _ecore_status_t
ecore_roce_query_qp(struct ecore_hwfn *p_hwfn,
		    struct ecore_rdma_qp *qp,
		    struct ecore_rdma_query_qp_out_params *out_params,
		    struct ecore_rdma_info *rdma_info);

enum _ecore_status_t
ecore_roce_destroy_qp(struct ecore_hwfn *p_hwfn,
		      struct ecore_rdma_qp *qp,
		      struct ecore_rdma_destroy_qp_out_params *out_params,
		      struct ecore_rdma_info *rdma_info);

enum _ecore_status_t
ecore_roce_alloc_qp_idx(struct ecore_hwfn *p_hwfn,
			u16 *qp_idx16, struct ecore_rdma_info *rdma_info);

enum _ecore_status_t
ecore_roce_reserve_qp_idx(struct ecore_hwfn *p_hwfn,
			  struct ecore_rdma_info *rdma_info);

void
ecore_roce_free_reserved_qp_idx(struct ecore_hwfn *p_hwfn,
				struct ecore_rdma_info *rdma_info);

#define IS_ECORE_DCQCN(p_hwfn)	\
	(!!(p_hwfn->pf_params.rdma_pf_params.enable_dcqcn))

struct ecore_roce_info {
	struct roce_events_stats	event_stats;
	struct roce_dcqcn_received_stats dcqcn_rx_stats;
	struct roce_dcqcn_sent_stats	dcqcn_tx_stats;
	struct roce_cqe_stats		cqe_stats;
	struct roce_error_stats		error_stats;
	u8				dcqcn_enabled;
	u8				dcqcn_reaction_point;
};

enum _ecore_status_t
ecore_roce_alloc_irq(struct ecore_hwfn *p_hwfn,
		     struct ecore_rdma_qp *qp);
void
ecore_roce_free_irq(struct ecore_hwfn *p_hwfn,
		    struct ecore_rdma_qp *qp);
enum _ecore_status_t
ecore_roce_alloc_orq(struct ecore_hwfn *p_hwfn,
		     struct ecore_rdma_qp *qp);
void
ecore_roce_free_orq(struct ecore_hwfn *p_hwfn,
		    struct ecore_rdma_qp *qp);

enum _ecore_status_t
ecore_roce_modify_qp(struct ecore_hwfn *p_hwfn,
		     struct ecore_rdma_qp *qp,
		     enum ecore_roce_qp_state prev_state,
		     struct ecore_rdma_modify_qp_in_params *params,
		     struct ecore_rdma_info *rdma_info);

u16 ecore_roce_qp_idx_to_icid(struct ecore_hwfn *p_hwfn, u16 qp_idx, u8 vf_id);
u16 ecore_roce_icid_to_qp_idx(struct ecore_hwfn *p_hwfn, u16 icid);

#endif /*__ECORE_RDMA_H__*/
